@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------*/
 /* elements.c --- xcircuit routines for creating basic elements		*/
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University            */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University            */
 /*----------------------------------------------------------------------*/
 
 /*----------------------------------------------------------------------*/
@@ -18,7 +18,12 @@
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#else
 #include "Xw/Xw.h"
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Local includes                                                       */
@@ -39,18 +44,16 @@
 extern short eventmode;      /* keep track of the mode the screen is in */
 extern short textpos, textend;  /* keep track of the cursor position in text */
 extern Display  *dpy;    /* Works well to make this globally accessible */
-extern Window win;
 extern int *appcolors;
 extern Cursor   appcursors[NUM_CURSORS];
 extern Clientdata areastruct;
 extern Globaldata xobjs;
-extern Widget   top;
+extern xcWidget   top;
 extern fontinfo *fonts;
 extern short fontcount;
 extern short attachto;
 extern char  _STR[150], _STR2[250];
 extern stringpart *labelbuf;
-extern objectpair *pushlist;
 
 extern double atan2();
 
@@ -69,7 +72,6 @@ unsigned char texttype;
 
 void pathdefaults(pathptr newpath, int x, int y)
 {
-   newpath->color = DEFAULTCOLOR;
    newpath->style = NORMAL;
    newpath->width = areastruct.linewidth;
    newpath->style = areastruct.style;
@@ -91,10 +93,19 @@ void objectdefaults(objinstptr newinst, objectptr thisobj, int x, int y)
    newinst->rotation = 0;
    newinst->scale = 1.0;
    newinst->thisobject = thisobj;
-   newinst->color = DEFAULTCOLOR;
+   newinst->color = areastruct.color;
    newinst->params = NULL;
    newinst->passed = NULL;
    newinst->num_params = 0;
+
+   newinst->bbox.lowerleft.x = thisobj->bbox.lowerleft.x;
+   newinst->bbox.lowerleft.y = thisobj->bbox.lowerleft.y;
+   newinst->bbox.width = thisobj->bbox.width;
+   newinst->bbox.height = thisobj->bbox.height;
+
+#ifdef SCHEMA
+   newinst->schembbox = NULL;
+#endif
 }
 
 /*--------------------------------------*/
@@ -110,13 +121,13 @@ void drawdot(int xpos, int ypos)
    /* Find the object "dot" in the builtin library, or else use an arc */
    
    if ((dotobj = finddot()) != (objectptr)NULL) {
-      NEW_OBJINST(newdot, objectdata);
-      objectdata->parts++;
+      NEW_OBJINST(newdot, topobject);
+      topobject->parts++;
       objectdefaults(*newdot, dotobj, xpos, ypos);
    }
    else {
-      NEW_ARC(newarc, objectdata);
-      objectdata->parts++;
+      NEW_ARC(newarc, topobject);
+      topobject->parts++;
       arcdefaults(*newarc, xpos, ypos);
       (*newarc)->radius = 6;
       (*newarc)->yaxis = 6;
@@ -126,14 +137,14 @@ void drawdot(int xpos, int ypos)
       (*newarc)->passed = NULL;
       calcarc(*newarc);
    }
-   incr_changes(objectdata);
+   incr_changes(topobject);
 }
 
 /*----------------------*/
 /* Start spline mode.   */
 /*----------------------*/
 
-void startspline(Widget button, caddr_t clientdata, caddr_t calldata)
+void startspline(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
       Wprintf("Click button1 and drag for spline.");
@@ -145,7 +156,7 @@ void startspline(Widget button, caddr_t clientdata, caddr_t calldata)
 /* Start label mode.    */
 /*----------------------*/
 
-void starttext(Widget button, u_int mode, caddr_t calldata)
+void starttext(xcWidget button, pointertype mode, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
       Wprintf("Click button1 for text.");
@@ -176,8 +187,8 @@ void labeldefaults(labelptr newlabel, u_char dopin, int x, int y)
    newlabel->pin = False;
    if (areastruct.schemon) {
       newlabel->pin = dopin;
-      if (dopin == LOCAL) newlabel->color = SNAPCOLOR;
-      else if (dopin == INFO) newlabel->color = AUXCOLOR;
+      if (dopin == LOCAL) newlabel->color = LOCALPINCOLOR;
+      else if (dopin == INFO) newlabel->color = INFOLABELCOLOR;
    }
 #endif
 
@@ -196,14 +207,14 @@ void textbutton(u_char dopin, XButtonEvent *event)
    XPoint userpt;
    short tmpheight;
 
-   XDefineCursor(dpy, win, TEXTPTR);
+   XDefineCursor(dpy, areastruct.areawin, TEXTPTR);
    Wprintf("Click to end or cancel.");
 
    if (fontcount == 0)
       Wprintf("Warning:  No fonts available!");
 
-   NEW_LABEL(newlabel, objectdata);
-   areastruct.editpart = objectdata->parts;
+   NEW_LABEL(newlabel, topobject);
+   areastruct.editpart = topobject->parts;
    snap(event->x, event->y, &userpt);
    labeldefaults(*newlabel, dopin, userpt.x, userpt.y);
 
@@ -230,7 +241,7 @@ void charreport(labelptr curlabel)
    _STR2[0] = '\0';
    for (i = textpos - MAXCHARS; i <= textpos + MAXCHARS - 1; i++) {
       if (i < 0) continue; 
-      strptr = findstringpart(i, &locpos, curlabel->string, NORMINST);
+      strptr = findstringpart(i, &locpos, curlabel->string, areastruct.topinstance);
       if (i == textpos) {
 	 strncat(_STR2, "| ", cleft);
 	 cleft -= 2;
@@ -254,7 +265,7 @@ labelptr findlabelcopy(labelptr curlabel, stringpart *curstring)
    genericptr *tgen;
    labelptr tlab;
 
-   for (tgen = objectdata->plist; tgen < objectdata->plist + objectdata->parts; tgen++) {
+   for (tgen = topobject->plist; tgen < topobject->plist + topobject->parts; tgen++) {
       if ((*tgen)->type == LABEL) {
          tlab = TOLABEL(tgen);
 	 if (tlab->pin == False) continue;
@@ -278,6 +289,7 @@ void labeltext(int keypressed, char *clientdata)
    int locpos;
    Boolean do_redraw = False;
    short tmplength, tmpheight, cfont;
+   TextExtents tmpext;
 
    curlabel = TOLABEL(EDITPART);
 
@@ -287,25 +299,26 @@ void labeltext(int keypressed, char *clientdata)
    }
 
    /* find text segment of the current position */
-   curpos = findstringpart(textpos, &locpos, curlabel->string, NORMINST);
+   curpos = findstringpart(textpos, &locpos, curlabel->string, areastruct.topinstance);
 
    UDrawTLine(curlabel);
 
    if (isbound(keypressed, XCF_Text_Delete) && textpos > 1) {
-      int curloc, curpos;
+      int curloc, strpos;
       stringpart *strptr;
 
       if (textend == 0) textend = textpos - 1;
 
       undrawtext(curlabel);
-      for (curpos = textpos - 1; curpos >= textend; curpos--) {
-	 strptr = findstringpart(curpos, &curloc, curlabel->string, NORMINST);
+      for (strpos = textpos - 1; strpos >= textend; strpos--) {
+	 strptr = findstringpart(strpos, &curloc, curlabel->string,
+			 areastruct.topinstance);
 	 if (curloc >= 0) {
 	    memmove(strptr->data.string + curloc,
 			strptr->data.string + curloc + 1,
 			strlen(strptr->data.string + curloc + 1) + 1);
 	    if (strlen(strptr->data.string) == 0)
-	       deletestring(strptr, &curlabel->string, NORMINST);
+	       deletestring(strptr, &curlabel->string, areastruct.topinstance);
 	 }
 
          /* Don't delete any parameter boundaries---must use	*/
@@ -313,7 +326,7 @@ void labeltext(int keypressed, char *clientdata)
 
 	 else if (strptr != NULL) {
 	    if ((strptr->type != PARAM_START) && (strptr->type != PARAM_END))
-	       deletestring(strptr, &curlabel->string, NORMINST);
+	       deletestring(strptr, &curlabel->string, areastruct.topinstance);
 	    else
 	       textpos++;
 	 }
@@ -333,22 +346,26 @@ void labeltext(int keypressed, char *clientdata)
 	 if (tmppos->type == PARAM_START) hasstuff = True;
 	 else if (tmppos->type == TEXT_STRING) hasstuff = True;
       }
-      XDefineCursor(dpy, win, CROSS);
+      XDefineCursor(dpy, areastruct.areawin, CROSS);
 
       Wprintf("");
 
       if (hasstuff && (eventmode != TEXT3_MODE && eventmode != CATTEXT_MODE)) {
-	 objectdata->parts++;
-	 incr_changes(objectdata);
+	 topobject->parts++;
+	 incr_changes(topobject);
+#ifdef SCHEMA
+         topobject->valid = False;
+#endif
+
       }
-      else if (!hasstuff && (eventmode == TEXT3_MODE)) objectdata->parts--;
+      else if (!hasstuff && (eventmode == TEXT3_MODE)) topobject->parts--;
 
       if ((!hasstuff) && (eventmode == CATTEXT_MODE)) {  /* can't have null labels! */ 
 	  freelabel(curlabel->string);
-	  curlabel->string = stringcopyback(labelbuf, NORMINST);
+	  curlabel->string = stringcopyback(labelbuf, areastruct.topinstance);
 	  freelabel(labelbuf);
 	  labelbuf = NULL;
-	  resolveparams(NORMINST);
+	  resolveparams(areastruct.topinstance);
           XcSetFunction(GXcopy);
 	  redrawtext(curlabel);
 	  Wprintf("Object must have a name!");
@@ -370,19 +387,30 @@ void labeltext(int keypressed, char *clientdata)
 	    short *selptr = areastruct.selectlist + areastruct.selects - 1;
 	    objinstptr selobj = SELTOOBJINST(selptr);
 	    strcpy(selobj->thisobject->name, curlabel->string->nextpart->data.string);
-	    checkname(selobj->thisobject);
-	    undrawtext(curlabel);
-	    curlabel->string->nextpart->data.string = (char *)realloc(
+	    /* If checkname() alters the name, it has to be copied back to */
+	    /* the catalog label for the object.			   */
+
+	    if (checkname(selobj->thisobject)) {
+	       undrawtext(curlabel);
+	       curlabel->string->nextpart->data.string = (char *)realloc(
 			curlabel->string->nextpart->data.string,
 		  	(strlen(_STR) + 1) * sizeof(char));
-	    strcpy(curlabel->string->nextpart->data.string, _STR);
-            XcSetFunction(GXcopy);
-	    redrawtext(curlabel);
+	       strcpy(curlabel->string->nextpart->data.string,
+			selobj->thisobject->name);
+               XcSetFunction(GXcopy);
+	       redrawtext(curlabel);
+	    }
 	 }
          eventmode = CATALOG_MODE;
 	 objectdeselect();
       }
-      else eventmode = NORMAL_MODE;
+      else {	/* (hasstuff && eventmode != CATTEXT_MODE) */
+	 eventmode = NORMAL_MODE;
+	 incr_changes(topobject);
+#ifdef SCHEMA
+	 if (curlabel->pin != False) topobject->valid = False;
+#endif
+      }
 
       setdefaultfontmarks();
       setcolormark(areastruct.color);
@@ -395,41 +423,106 @@ void labeltext(int keypressed, char *clientdata)
 	 /* schematic to match the new text.				*/
 
 	 if ((areastruct.schemon == True) && (curlabel->pin != False) && 
-		(objectdata->symschem != NULL)) {
+		(topobject->symschem != NULL)) {
 	    if ((findlabelcopy(curlabel, labelbuf) == NULL)
 			&& (findlabelcopy(curlabel, curlabel->string) == NULL)) {
 	       changeotherpins(curlabel, labelbuf);
-	       if (objectdata->schemtype == SCHEMATIC)
+	       if (topobject->schemtype == SCHEMATIC)
 	          Wprintf("Changed corresponding pin in associated symbol");
 	       else
 	          Wprintf("Changed corresponding pin in associated schematic");
+	       incr_changes(topobject->symschem);
+	       topobject->symschem->valid = False;
 	    }
 	 }
 #endif
 	
 	 freelabel(labelbuf);
 	 labelbuf = NULL;
-	 resolveparams(NORMINST);
+	 resolveparams(areastruct.topinstance);
 	 if (do_all >= 0)
-            calcbboxparam(objectdata, do_all);
+            updateinstparam(areastruct.topinstance, do_all);
 	 else /* do_all == -1 */
-	    calcbbox(objectdata);
+	    calcbbox(areastruct.topinstance);
 #ifdef SCHEMA
-	 setobjecttype(objectdata, NULL);
+	 setobjecttype(topobject);
 #endif
       }
       else
-         calcbbox(objectdata);
+         calcbbox(areastruct.topinstance);
       return;
    }
    else if (isbound(keypressed, XCF_Text_Right) && curpos != NULL)
       textpos++;
    else if (isbound(keypressed, XCF_Text_Left) && textpos > 1)
       textpos--;
+   else if (isbound(keypressed, XCF_Text_Down)) {
+      while (curpos != NULL) {
+	 textpos++;
+	 curpos = findstringpart(textpos, &locpos, curlabel->string,
+			areastruct.topinstance);
+	 if (curpos != NULL)
+	    if (curpos->type == RETURN)
+	       break;
+      }
+   }
+   else if (isbound(keypressed, XCF_Text_Up)) {
+      while (textpos > 1) {
+	 textpos--;
+	 curpos = findstringpart(textpos, &locpos, curlabel->string,
+	 		areastruct.topinstance);
+	 if (curpos->type == RETURN) {
+	    if (textpos > 1) textpos--;
+	    break;
+	 }
+      }
+   }
    else if (isbound(keypressed, XCF_Text_Home))
       textpos = 1;
    else if (isbound(keypressed, XCF_Text_End))
-      textpos = stringlength(curlabel->string, True, NORMINST);
+      textpos = stringlength(curlabel->string, True, areastruct.topinstance);
+   else if (isbound(keypressed, XCF_Text_Split)) {
+      labelptr *newlabel;
+      XPoint points[4], points1[4], points2[4];
+
+      /* Everything after the cursor gets dumped into a new label */
+
+      if ((textpos > 1) && (curpos != NULL)) {
+	 labelbbox(curlabel, points, areastruct.topinstance);
+         undrawtext(curlabel);
+	 NEW_LABEL(newlabel, topobject);
+	 labeldefaults(*newlabel, curlabel->pin, curlabel->position.x,
+		curlabel->position.y);
+         if (locpos > 0)
+            curpos = splitstring(textpos, &curlabel->string, areastruct.topinstance);
+	 /* move back one position to find end of top part of string */
+         curpos = splitstring(textpos - 1, &curlabel->string, areastruct.topinstance);
+	 if (curpos->nextpart->type == FONT_NAME) {
+	    freelabel((*newlabel)->string);
+	    (*newlabel)->string = curpos->nextpart;
+	 }
+	 else {
+	    (*newlabel)->string->data.font = curlabel->string->data.font;
+	    (*newlabel)->string->nextpart = curpos->nextpart;
+	 }
+	 curpos->nextpart = NULL;
+	 topobject->parts++;
+
+	 /* Adjust position of both labels to retain their original	*/
+	 /* relative positions.						*/
+
+	 labelbbox(curlabel, points1, areastruct.topinstance);
+	 labelbbox((*newlabel), points2, areastruct.topinstance);
+	 curlabel->position.x += (points[1].x - points1[1].x);
+	 curlabel->position.y += (points[1].y - points1[1].y);
+	 (*newlabel)->position.x += (points[3].x - points2[3].x);
+	 (*newlabel)->position.y += (points[3].y - points2[3].y);
+	
+         XcSetFunction(GXcopy);
+         redrawtext(*newlabel);
+         do_redraw = True;
+      }
+   }
 
    /* Write a font designator or other control into the string */
 
@@ -439,7 +532,7 @@ void labeltext(int keypressed, char *clientdata)
       /* erase first before redrawing unless the string is empty */
       undrawtext(curlabel);
       if (locpos > 0) {
-         curpos = splitstring(textpos, &curlabel->string, NORMINST);
+         curpos = splitstring(textpos, &curlabel->string, areastruct.topinstance);
 	 curpos = curpos->nextpart;
       }
       newpart = makesegment(&curlabel->string, curpos);
@@ -479,7 +572,7 @@ void labeltext(int keypressed, char *clientdata)
       if (locpos < 0) {
 
          /* Find part of string which is immediately in front of textpos */
-         lastpos = findstringpart(textpos - 1, &locpos, curlabel->string, NORMINST);
+         lastpos = findstringpart(textpos - 1, &locpos, curlabel->string, areastruct.topinstance);
 
 	 /* No text on either side to attach to: make a new text segment */
 	 if (locpos < 0) {
@@ -512,12 +605,11 @@ void labeltext(int keypressed, char *clientdata)
 
    if (do_redraw) {
       short beglength;
-      TextExtents tmpext;
 
-      tmpext = ULength(curlabel->string, NORMINST, curlabel->scale, textpos, NULL);
+      tmpext = ULength(curlabel->string, areastruct.topinstance, curlabel->scale, textpos, NULL);
       beglength = tmpext.width;
 	
-      tmpext = ULength(curlabel->string, NORMINST, curlabel->scale, 0, NULL);
+      tmpext = ULength(curlabel->string, areastruct.topinstance, curlabel->scale, 0, NULL);
       tmplength = tmpext.width;
       tmpheight = (short)(curlabel->scale * TEXTHEIGHT);
       areastruct.origin.x = curlabel->position.x + (curlabel->justify & NOTLEFT
@@ -542,7 +634,7 @@ void labeltext(int keypressed, char *clientdata)
 
    /* find current font and adjust menubuttons as necessary */
 
-   cfont = findcurfont(textpos, curlabel->string, NORMINST);
+   cfont = findcurfont(textpos, curlabel->string, areastruct.topinstance);
    if (cfont < 0) {
       Wprintf("Error:  Illegal label string");
       return;
@@ -608,8 +700,8 @@ void rejustify(short mode)
    if (curlabel == NULL)
       Wprintf("No labels chosen to rejustify");
    else if (changed) {
-      pwriteback(NORMINST);
-      incr_changes(objectdata);
+      pwriteback(areastruct.topinstance);
+      incr_changes(topobject);
 
    }
 }
@@ -645,8 +737,8 @@ void splinebutton(XButtonEvent *event)
    splineptr *newspline;
    XPoint userpt;
 
-   NEW_SPLINE(newspline, objectdata);
-   areastruct.editpart = objectdata->parts;
+   NEW_SPLINE(newspline, topobject);
+   areastruct.editpart = topobject->parts;
 
    snap(event->x, event->y, &userpt);
    areastruct.editcycle = 3;
@@ -654,10 +746,11 @@ void splinebutton(XButtonEvent *event)
 
    XcSetXORFg(areastruct.color, BACKGROUND);
    XcSetFunction(GXxor);
-   UDrawEditSpline(areastruct.topobject, *newspline);
+   UDrawEditSpline(*newspline);
+
+   xcAddEventHandler(areastruct.area, PointerMotionMask, False,
+        (xcEventHandler)trackspline, NULL);
 
-   XtAddEventHandler(areastruct.area, PointerMotionMask, False,
-        (XtEventHandler)trackspline, NULL);
    eventmode = SPLINE_MODE;
 }
 
@@ -665,7 +758,7 @@ void splinebutton(XButtonEvent *event)
 /* Track a spline during mouse motion */
 /*------------------------------------*/
 
-void trackspline(Widget w, caddr_t clientdata, caddr_t calldata)
+void trackspline(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    XPoint       newpos;
    splineptr    newspline;
@@ -677,7 +770,7 @@ void trackspline(Widget w, caddr_t clientdata, caddr_t calldata)
    u2u_snap(&newpos);
    if (areastruct.save.x == newpos.x && areastruct.save.y == newpos.y) return;
 
-   UDrawEditSpline(areastruct.topobject, newspline);
+   UDrawEditSpline(newspline);
 
    if (areastruct.editcycle == 0 || areastruct.editcycle == 3) {
       short cpoint = (areastruct.editcycle == 0) ? 1 : 2;
@@ -688,18 +781,30 @@ void trackspline(Widget w, caddr_t clientdata, caddr_t calldata)
    newspline->ctrl[areastruct.editcycle].y = newpos.y;
 
    calcspline(newspline);
-   UDrawEditSpline(areastruct.topobject, newspline);
+   UDrawEditSpline(newspline);
 
    printpos(newpos.x, newpos.y);
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
 }
 
+/*--------------------------*/
+/* Start drawing a polygon. */
+/*--------------------------*/
+
+void startpoly(xcWidget button, caddr_t clientdata, caddr_t calldata)
+{
+   if (eventmode == NORMAL_MODE) {
+      Wprintf("Click button1 to start the polygon.");
+      eventmode = POLY_MODE;
+   }
+}
+
 /*-----------------------*/
 /* Start drawing an arc. */
 /*-----------------------*/
 
-void startarc(Widget button, caddr_t clientdata, caddr_t calldata)
+void startarc(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
       Wprintf("Click button1 and drag to make arc.");
@@ -736,8 +841,8 @@ void arcbutton(XButtonEvent *event)
    arcptr *newarc;
    XPoint userpt;
 
-   NEW_ARC(newarc, objectdata);
-   areastruct.editpart = objectdata->parts;
+   NEW_ARC(newarc, topobject);
+   areastruct.editpart = topobject->parts;
    snap(event->x, event->y, &userpt);
    areastruct.editcycle = 0;
    saveratio = 1.0;
@@ -745,11 +850,12 @@ void arcbutton(XButtonEvent *event)
 
    XcSetXORFg(areastruct.color, BACKGROUND);
    XcSetFunction(GXxor);
-   UDrawArc(areastruct.topobject, *newarc);
+   UDrawArc(*newarc);
    UDrawXLine((*newarc)->position, (*newarc)->position);
 
-   XtAddEventHandler(areastruct.area, PointerMotionMask, False,
-        (XtEventHandler)trackarc, NULL);
+   xcAddEventHandler(areastruct.area, PointerMotionMask, False,
+        (xcEventHandler)trackarc, NULL);
+
    eventmode = ARC_MODE;
 }
 
@@ -757,7 +863,7 @@ void arcbutton(XButtonEvent *event)
 /* Track an arc during mouse motion */
 /*----------------------------------*/
 
-void trackarc(Widget w, caddr_t clientdata, caddr_t calldata)
+void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    XPoint newpos;
    arcptr newarc;
@@ -771,7 +877,7 @@ void trackarc(Widget w, caddr_t clientdata, caddr_t calldata)
    u2u_snap(&newpos);
    if (areastruct.save.x == newpos.x && areastruct.save.y == newpos.y) return;
 
-   UDrawArc(areastruct.topobject, newarc);
+   UDrawArc(newarc);
    UDrawXLine(areastruct.save, newarc->position);
 
    if (areastruct.editcycle == 1 || areastruct.editcycle == 2) {
@@ -812,7 +918,7 @@ void trackarc(Widget w, caddr_t clientdata, caddr_t calldata)
 
    calcarc(newarc);
 
-   UDrawArc(areastruct.topobject, newarc);
+   UDrawArc(newarc);
    UDrawXLine(newpos, newarc->position);
    printpos(newpos.x, newpos.y);
 
@@ -824,7 +930,7 @@ void trackarc(Widget w, caddr_t clientdata, caddr_t calldata)
 /* Start drawing a box. */
 /*----------------------*/
 
-void startbox(Widget button, caddr_t clientdata, caddr_t calldata)
+void startbox(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
       Wprintf("Click button1 and drag to make box.");
@@ -863,17 +969,18 @@ void boxbutton(XButtonEvent *event)
    polyptr *newbox;
    XPoint userpt;
 
-   NEW_POLY(newbox, objectdata);
-   areastruct.editpart = objectdata->parts;
+   NEW_POLY(newbox, topobject);
+   areastruct.editpart = topobject->parts;
    snap(event->x, event->y, &userpt);
    polydefaults(*newbox, 4, userpt.x, userpt.y);
 
    XcSetXORFg(areastruct.color, BACKGROUND);
    XcSetFunction(GXxor);
-   UDrawPolygon(areastruct.topobject, (*newbox));
+   UDrawPolygon(*newbox);
+
+   xcAddEventHandler(areastruct.area, PointerMotionMask, False,
+        (xcEventHandler)trackbox, NULL);
 
-   XtAddEventHandler(areastruct.area, PointerMotionMask, False,
-        (XtEventHandler)trackbox, NULL);
    eventmode = BOX_MODE;
 }
 
@@ -881,7 +988,7 @@ void boxbutton(XButtonEvent *event)
 /* Track a box during mouse motion */
 /*---------------------------------*/
 
-void trackbox(Widget w, caddr_t clientdata, caddr_t calldata)
+void trackbox(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    XPoint newpos;
    polyptr      newbox;
@@ -893,13 +1000,13 @@ void trackbox(Widget w, caddr_t clientdata, caddr_t calldata)
 
    if (areastruct.save.x == newpos.x && areastruct.save.y == newpos.y) return;
 
-   UDrawPolygon(areastruct.topobject, newbox);
+   UDrawPolygon(newbox);
 
    pointptr = newbox->points + 1; pointptr->y = newpos.y;
    pointptr++; pointptr->y = newpos.y; pointptr->x = newpos.x;
    pointptr++; pointptr->x = newpos.x;
    
-   UDrawPolygon(areastruct.topobject, newbox);
+   UDrawPolygon(newbox);
    printpos(newpos.x, newpos.y);
 
    areastruct.save.x = newpos.x;
@@ -910,7 +1017,7 @@ void trackbox(Widget w, caddr_t clientdata, caddr_t calldata)
 /* Track a wire during mouse motion */
 /*----------------------------------*/
 
-void trackwire(Widget w, caddr_t clientdata, caddr_t calldata)
+void trackwire(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    XPoint newpos, *tpoint;
    polyptr	newwire;
@@ -922,10 +1029,10 @@ void trackwire(Widget w, caddr_t clientdata, caddr_t calldata)
 
    if (areastruct.save.x != newpos.x || areastruct.save.y != newpos.y) {
       tpoint = newwire->points + newwire->number - 1;
-      UDrawPolygon(areastruct.topobject, newwire);
+      UDrawPolygon(newwire);
       tpoint->x = newpos.x;
       tpoint->y = newpos.y;
-      UDrawPolygon(areastruct.topobject, newwire);
+      UDrawPolygon(newwire);
       areastruct.save.x = newpos.x;
       areastruct.save.y = newpos.y;
       printpos(newpos.x, newpos.y);
@@ -941,8 +1048,8 @@ void startwire(XPoint userpt)
    polyptr *newwire;
    pointlist pointptr;
 
-   NEW_POLY(newwire, objectdata);
-   areastruct.editpart = objectdata->parts;
+   NEW_POLY(newwire, topobject);
+   areastruct.editpart = topobject->parts;
 
    /* always start unfilled, unclosed; can fix on next button-push. */
 
@@ -959,10 +1066,10 @@ void startwire(XPoint userpt)
 
    XcSetXORFg(areastruct.color, BACKGROUND);
    XcSetFunction(GXxor);
-   UDrawPolygon(areastruct.topobject, *newwire);
+   UDrawPolygon(*newwire);
 
-   XtAddEventHandler(areastruct.area, PointerMotionMask, False,
-          (XtEventHandler)trackwire, NULL);
+   xcAddEventHandler(areastruct.area, PointerMotionMask, False,
+          (xcEventHandler)trackwire, NULL);
 }
 
 /*----------------------------------------------------------*/
@@ -1047,7 +1154,7 @@ void finddir(polyptr lastpoly)
 /* Track movement of poly segments during edit mode */
 /*--------------------------------------------------*/
 
-void trackpoly(Widget w, caddr_t clientdata, caddr_t calldata)
+void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    XPoint newpos, *curpt;
    polyptr      newpoly;
@@ -1060,7 +1167,7 @@ void trackpoly(Widget w, caddr_t clientdata, caddr_t calldata)
    u2u_snap(&newpos);
    if (areastruct.save.x == newpos.x && areastruct.save.y == newpos.y) return;
 
-   UDrawPolygon(areastruct.topobject, newpoly);
+   UDrawPolygon(newpoly);
 
    /* find the point under consideration */
    curpt = newpoly->points + areastruct.editcycle;
@@ -1101,7 +1208,7 @@ void trackpoly(Widget w, caddr_t clientdata, caddr_t calldata)
       curpt->y = newpos.y;
    }
 
-   UDrawPolygon(areastruct.topobject, newpoly);
+   UDrawPolygon(newpoly);
    printpos(newpos.x, newpos.y);
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
@@ -1113,7 +1220,7 @@ void trackpoly(Widget w, caddr_t clientdata, caddr_t calldata)
 
 void setendpoint(short *scnt, short direc, XPoint **endpoint, XPoint *arcpoint)
 {
-   genericptr *sptr = objectdata->plist + (*scnt);
+   genericptr *sptr = topobject->plist + (*scnt);
 
    switch((*sptr)->type) {
       case POLYGON:
@@ -1185,13 +1292,13 @@ void reversefpoints(XfPoint *plist, short number)
 }
 
 /*--------------------------------------------------------------*/
-/* Permanently remove an element from the objectdata plist	*/
+/* Permanently remove an element from the topobject plist	*/
 /*	add = 1 if plist has (parts + 1) elements		*/
 /*--------------------------------------------------------------*/
 
 void freeparts(short *selectobj, short add)
 {
-   genericptr *oldelem = objectdata->plist + (*selectobj);
+   genericptr *oldelem = topobject->plist + (*selectobj);
    switch((*oldelem)->type) {
       case POLYGON:
 	 free((TOPOLY(oldelem))->points);
@@ -1214,12 +1321,12 @@ void freeparts(short *selectobj, short add)
 
 void removep(short *selectobj, short add)
 {
-   genericptr *oldelem = objectdata->plist + (*selectobj);
+   genericptr *oldelem = topobject->plist + (*selectobj);
 
-   for (++oldelem; oldelem < objectdata->plist + objectdata->parts + add; oldelem++)
+   for (++oldelem; oldelem < topobject->plist + topobject->parts + add; oldelem++)
 	    *(oldelem - 1) = *oldelem;
 
-   objectdata->parts--;
+   topobject->parts--;
 }
 
 /*-------------------------------------------------*/
@@ -1250,18 +1357,18 @@ void unjoin()
 
          /* undraw the path */
       
-         UDrawPath(areastruct.topobject, oldpath);
+         UDrawPath(oldpath);
       
          /* move components to the top level */
 
-	 objectdata->plist = (genericptr *)realloc(objectdata->plist,
-		(objectdata->parts + oldpath->parts) * sizeof(genericptr));
-	 newg = objectdata->plist + objectdata->parts;
+	 topobject->plist = (genericptr *)realloc(topobject->plist,
+		(topobject->parts + oldpath->parts) * sizeof(genericptr));
+	 newg = topobject->plist + topobject->parts;
 	 for (genp = oldpath->plist; genp < oldpath->plist +
 		oldpath->parts; genp++, newg++) {
 	    *newg = *genp;
 	 }
-	 objectdata->parts += oldpath->parts;
+	 topobject->parts += oldpath->parts;
 
          /* remove the path object and revise the selectlist */
 
@@ -1269,8 +1376,7 @@ void unjoin()
          reviseselect(selectobj);
       }
    }
-   if (areastruct.selects > 0) free(areastruct.selectlist);
-   areastruct.selects = 0;
+   clearselects();
    drawarea(NULL, NULL, NULL);
 }
 
@@ -1303,14 +1409,14 @@ void join()
    pathptr   *newpath;
    short     *scount, *sptr, *sptr2, *direc, *order;
    short     ordered, startpt = 0;
-   short     numpolys, numpoints, polytype;
+   short     numpolys, numlabels, numpoints, polytype;
    int	     polycolor;
    float     polywidth;
    XPoint    *testpoint, *testpoint2, *begpoint, *endpoint, arcpoint[4];
    XPoint    *begpoint2, *endpoint2;
    Boolean   allpolys = True;
 
-   numpolys = 0;
+   numpolys = numlabels = 0;
    for (selectobj = areastruct.selectlist; selectobj < areastruct.selectlist
 	+ areastruct.selects; selectobj++) {
       if (SELECTTYPE(selectobj) == POLYGON) {
@@ -1334,15 +1440,25 @@ void join()
 	 numpolys++;
 	 allpolys = False;
       }
+      else if (SELECTTYPE(selectobj) == LABEL)
+	 numlabels++;
    }
-   if (numpolys == 0) {
+   if ((numpolys == 0) && (numlabels == 0)) {
       Wprintf("No elements selected for joining.");
       return;
    }
-   else if (numpolys == 1) {
+   else if ((numpolys == 1) || (numlabels == 1)) {
       Wprintf("Only one element: nothing to join to.");
       return;
    }
+   else if ((numpolys > 1) && (numlabels > 1)) {
+      Wprintf("Selection mixes labels and line segments.  Ignoring.");
+      return;
+   }
+   else if (numlabels > 0) {
+      joinlabels();
+      return;
+   }
 
    /* scount is a table of element numbers 				*/
    /* order is an ordered table of end-to-end elements 			*/
@@ -1444,7 +1560,7 @@ void join()
    /* create the new polygon or path */
 
    if (allpolys) {
-      NEW_POLY(newpoly, objectdata);
+      NEW_POLY(newpoly, topobject);
 
       (*newpoly)->number = numpoints;
       (*newpoly)->points = (pointlist) malloc(numpoints * sizeof(XPoint));
@@ -1490,12 +1606,12 @@ void join()
 	    if (*sptr2 > *sptr) (*sptr2)--;
       }
       XcSetForeground((*newpoly)->color);
-      UDrawPolygon(areastruct.topobject, *newpoly);
+      UDrawPolygon(*newpoly);
    }
    else {	/* create a path */
       short newcount = 0;
 
-      NEW_PATH(newpath, objectdata);
+      NEW_PATH(newpath, topobject);
       (*newpath)->style = polytype;
       (*newpath)->color = polycolor;
       (*newpath)->width = polywidth;
@@ -1507,7 +1623,7 @@ void join()
       /* move the elements from the top level into the path structure */
 
       for (sptr = order; sptr < order + numpolys; sptr++, newcount++) {
-	 genericptr *oldelem = objectdata->plist + *sptr;
+	 genericptr *oldelem = topobject->plist + *sptr;
 	 genericptr *newelem = (*newpath)->plist + newcount;
 	 *newelem = *oldelem;
 
@@ -1538,15 +1654,14 @@ void join()
       }
       
       XcSetForeground((*newpath)->color);
-      UDrawPath(areastruct.topobject, *newpath);
+      UDrawPath(*newpath);
    }
 
    /* clean up */
 
-   objectdata->parts++;
-   incr_changes(objectdata);
-   if (areastruct.selects > 0) free(areastruct.selectlist);
-   areastruct.selects = 0;
+   topobject->parts++;
+   incr_changes(topobject);
+   clearselects();
    free(scount);
    free(order);
    free(direc);
@@ -1568,7 +1683,7 @@ void wirebutton(XButtonEvent *event)
  
    /* This undraws the wire */
 
-   UDrawPolygon(areastruct.topobject, newwire);
+   UDrawPolygon(newwire);
 
    tpoint = newwire->points + newwire->number - 1;
    tpoint->x = userpt.x;
@@ -1598,17 +1713,20 @@ void wirebutton(XButtonEvent *event)
       tpoint->y = userpt.y;
    }
    else if (newwire == NULL || event->button != Button3)
-      XtRemoveEventHandler(areastruct.area, PointerMotionMask, False,
-         (XtEventHandler)trackwire, NULL);
+      xcRemoveEventHandler(areastruct.area, PointerMotionMask, False,
+         (xcEventHandler)trackwire, NULL);
 
    if (newwire) {
       if (event->button == Button2) {
          XcSetFunction(GXcopy);
          XcSetForeground(newwire->color);
-	 objectdata->parts++;
-	 incr_changes(objectdata);
+	 topobject->parts++;
+	 incr_changes(topobject);
+#ifdef SCHEMA
+	 if (!nonnetwork(newwire)) topobject->valid = False;
+#endif
       }
-      UDrawPolygon(areastruct.topobject, newwire);
+      UDrawPolygon(newwire);
       if (event->button == Button3)
 	 checkwarp(newwire->points + newwire->number - 1);
    }
