@@ -15,7 +15,12 @@
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#else
 #include "Xw/Xw.h"
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes                                                          */
@@ -42,6 +47,10 @@ extern short beeper;
 
 extern float version;
 
+#ifdef TCL_WRAPPER
+extern Tcl_Interp *xcinterp;
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Find the file containing the encoding for the given font		*/
 /*----------------------------------------------------------------------*/
@@ -113,6 +122,14 @@ FILE *findfontfile(char *fontname)
 	    if ((dashptr = strrchr(tempname, '-')) != NULL) {
 	       *dashptr = '\0';
 	       if ((fd = findfontfile(tempname)) != NULL) return fd;
+
+	       /* And finally, because it's a common case, try adding	*/
+	       /* -Roman and trying again (but don't infinite loop!	*/
+
+	       if (strcmp(dashptr + 1, "Roman")) {
+	          strcat(dashptr, "-Roman");
+	          if ((fd = findfontfile(tempname)) != NULL) return fd;
+	       }
 	    }
 
             Wprintf("No font encoding file found.");
@@ -139,7 +156,19 @@ FILE *findfontfile(char *fontname)
                fonts[fontcount].flags = 0; 
                fonts[fontcount].scale = 1.0;
                fontcount++;
+#ifdef TCL_WRAPPER
+	       sprintf(_STR2, "newfontbutton %s", psname);
+	       Tcl_Eval(xcinterp, _STR2);
+#else
 	       makenewfontbutton();
+#endif
+	    }
+	    else {
+	       fprintf(stderr, "Error:  font encoding file missing for font \"%s\"\n",
+			fontname);
+	       fprintf(stderr, "No fonts exist for a subsitution.  Make sure "
+			"fonts are installed or that\nenvironment variable "
+			"XCIRUCIT_LIB_DIR points to a directory of valid fonts.\n");
 	    }
             return (FILE *)NULL;
          }
@@ -295,18 +324,27 @@ void loadfontfile(char *fontname)
 	 
 	 if (!strcmp(tempname, "special") || !strcmp(tempname, "Special")) {
 	    flags |= 0x80;
+#ifdef TCL_WRAPPER
+	    Tcl_Eval(xcinterp, "newencodingbutton special");
+#else
 	    makenewencodingbutton("Special", (char)1);
+#endif
 	 }
 
 	 /* ISO-LatinX encodings where X=1 to 6 */
 
 	 if (strstr(tempname, "ISO") != NULL) {
-	    char estr[15];
+	    char estr[30];
 	    for (i = 0; i < 6; i++) {
 	       if (strchr(tempname, '1' + (char)i) != NULL) {
 	          flags |= ((i + 2) << 7);
+#ifdef TCL_WRAPPER
+		  sprintf(estr, "newencodingbutton ISOLatin%d", i + 1);
+		  Tcl_Eval(xcinterp, estr);
+#else
 		  sprintf(estr, "ISO-Latin%d", i + 1);
 	          makenewencodingbutton(estr, (char)(i + 2));
+#endif
 		  break;
 	       }
 	    }
@@ -366,7 +404,13 @@ void loadfontfile(char *fontname)
 	    if (!strcmp(fonts[i].family, fonts[fontcount - 1].family))
 	       break;
     
-	 if (i == fontcount - 1) makenewfontbutton();
+	 if (i == fontcount - 1)
+#ifdef TCL_WRAPPER
+	    sprintf(_STR2, "newfontbutton %s", family);
+	    Tcl_Eval(xcinterp, _STR2);
+#else
+	    makenewfontbutton();
+#endif
       }
    }
 }
