@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------*/
 /* keybindings.c:  List of key bindings				*/
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University	*/
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University	*/
 /*--------------------------------------------------------------*/
 
 /*----------------------------------------------------------------------*/
@@ -29,6 +29,10 @@
 /* Local includes							*/
 /*----------------------------------------------------------------------*/
 
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#endif
+
 #include "xcircuit.h"
 
 /*----------------------------------------------------------------------*/
@@ -71,11 +75,13 @@ static char *function_names[NUM_FUNCTIONS] = {
    "Overline", "ISO Encoding", "Return", "Halfspace", "Quarterspace",
    "Special", "Tab Stop", "Tab Forward", "Tab Backward",
    "Text Return", "Text Delete", "Text Right", "Text Left",
+   "Text Up", "Text Down", "Text Split", 
    "Text Home", "Text End", "Return Char", "Parameter",
    "Parameterize Point", "Break at Point", "Delete Point", "Insert Point",
    "Next Point", "Attach", "Next Library", "Library Directory",
    "Library Move", "Library Copy", "Library Edit", "Library Delete",
-   "Library Duplicate", "Library Hide", "Page Directory ", "Library Pop",
+   "Library Duplicate", "Library Hide", "Library Virtual Copy",
+   "Page Directory ", "Library Pop",
    "Help", "Redraw", "View", "Zoom In", "Zoom Out", "Pan",
    "Double Snap", "Halve Snap", "Pan Left", "Pan Right", "Pan Up",
    "Pan Down", "Write", "Rotate", "Flip X", "Flip Y", "Snap",
@@ -86,8 +92,8 @@ static char *function_names[NUM_FUNCTIONS] = {
    "Nothing", "Exit"
 
 #ifdef SCHEMA
-   , "Netlist", "Swap", "Pin Label", "Info Label", "Connectivity",
-   "Sim", "SPICE", "PCB", "SPICE Flat"
+   , "Netlist", "Swap", "Pin Label", "Pin Global", "Info Label",
+   "Connectivity", "Sim", "SPICE", "PCB", "SPICE Flat"
 #endif
 };
 
@@ -525,9 +531,12 @@ void default_keybindings()
 
    add_binding(XK_Delete, XCF_Text_Delete);
    add_binding(XK_Return, XCF_Text_Return);
-   add_binding(XK_BackSpace, XCF_Text_Left);
+   add_binding(XK_BackSpace, XCF_Text_Delete);
    add_binding(XK_Left, XCF_Text_Left);
    add_binding(XK_Right, XCF_Text_Right);
+   add_binding(XK_Up, XCF_Text_Up);
+   add_binding(XK_Down, XCF_Text_Down);
+   add_binding(ALT | XK_x, XCF_Text_Split);
    add_binding(XK_Home, XCF_Text_Home);
    add_binding(XK_End, XCF_Text_End);
    add_binding(XK_Tab, XCF_TabForward);
@@ -567,9 +576,10 @@ void default_keybindings()
    add_binding(XK_D, XCF_Library_Delete);
    add_binding(XK_C, XCF_Library_Duplicate);
    add_binding(XK_H, XCF_Library_Hide);
+   add_binding(XK_V, XCF_Library_Virtual);
    add_binding(XK_M, XCF_Library_Move);
    add_binding(XK_P, XCF_Page_Directory);
-   add_binding(XK_greater, XCF_Library_Pop);
+   add_binding(XK_less, XCF_Library_Pop);
    add_binding(XK_h, XCF_Help);
    add_binding(XK_question, XCF_Help);
    add_binding(XK_space, XCF_Redraw);
@@ -631,6 +641,7 @@ void default_keybindings()
    add_binding(ALT | XK_q, XCF_Netlist);
    add_binding(XK_slash, XCF_Swap);
    add_binding(XK_T, XCF_Pin_Label);
+   add_binding(XK_G, XCF_Pin_Global);
    add_binding(XK_I, XCF_Info_Label);
    add_binding(ALT | XK_w, XCF_Connectivity);
    add_binding(ALT | XK_s, XCF_Sim);
