@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* menus.h								   */
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -8,7 +8,7 @@
 /*-------------------------------------------------------------------------*/
 /* Note:  underscore (_) before name denotes a color to paint the button.  */
 /*	  colon (:) before name denotes a stipple, defined by the data	   */
-/*		    passed to getfill().				   */
+/*		    passed to setfill().				   */
 /*-------------------------------------------------------------------------*/
 
 #ifdef _MENUDEP
@@ -127,12 +127,12 @@ menustruct Pages[] = {
 menustruct BorderStyles[] = {
 	{"Linewidth", action(getwwidth, NULL)},
 	{" ", noaction},
-	{"Solid", setaction(getline, Number(NORMAL))},
-	{"Dashed", action(getline, Number(DASHED))},
-	{"Dotted", action(getline, Number(DOTTED))},
-	{"Unbordered", action(getline, Number(NOBORDER))},
+	{"Solid", setaction(setline, Number(NORMAL))},
+	{"Dashed", action(setline, Number(DASHED))},
+	{"Dotted", action(setline, Number(DOTTED))},
+	{"Unbordered", action(setline, Number(NOBORDER))},
 	{" ", noaction},
-	{"Closed", action(getset, Number(UNCLOSED))}, 
+	{"Closed", action(setclosure, Number(UNCLOSED))}, 
 	{"Bounding Box", action(makebbox, Number(BBOX))},
 };
 
@@ -159,17 +159,18 @@ menustruct Parameterize[] = {
 };
 
 menustruct Stipples[] = {
-	{":Black",  action(getfill, Number(FILLED | FILLSOLID))},
-	{":Gray12", action(getfill, Number(FILLED | STIP2 | STIP1))},
-	{":Gray25", action(getfill, Number(FILLED | STIP2 | STIP0))},
-	{":Gray37", action(getfill, Number(FILLED | STIP2))},
-	{":Gray50", action(getfill, Number(FILLED | STIP0 | STIP1))},
-	{":Gray62", action(getfill, Number(FILLED | STIP1))},
-	{":Gray75", action(getfill, Number(FILLED | STIP0))},
-	{":Gray87", action(getfill, Number(FILLED))},
-	{":White", setaction(getfill, Number(FILLSOLID))},
+	{":Black",  action(setfill, Number(OPAQUE | FILLED | FILLSOLID))},
+	{":Gray12", action(setfill, Number(OPAQUE | FILLED | STIP2 | STIP1))},
+	{":Gray25", action(setfill, Number(OPAQUE | FILLED | STIP2 | STIP0))},
+	{":Gray37", action(setfill, Number(OPAQUE | FILLED | STIP2))},
+	{":Gray50", action(setfill, Number(OPAQUE | FILLED | STIP0 | STIP1))},
+	{":Gray62", action(setfill, Number(OPAQUE | FILLED | STIP1))},
+	{":Gray75", action(setfill, Number(OPAQUE | FILLED | STIP0))},
+	{":Gray87", action(setfill, Number(OPAQUE | FILLED))},
+	{":White", setaction(setfill, Number(FILLSOLID))},
 	{" ", noaction},
-	{"Opaque", action(getopaque, Number(OPAQUE))},
+	{"Opaque", action(setopaque, Number(OPAQUE))},
+	{"Transparent", setaction(setopaque, Number(NORMAL))},
 };
 
 menustruct TextMenu[] = {
@@ -284,11 +285,11 @@ menustruct FileMenu[] = {
 	{"Import Xcircuit PS", action(getfile, Number(IMPORT))},
 	{"Import background PS", action(getfile, Number(PSBKGROUND))},
 	{"Execute script", action(getfile, Number(SCRIPT))},
-	{"Write Xcircuit PS (W)", action(dooutput, NULL)},
+	{"Write Xcircuit PS (W)", action(outputpopup, NULL)},
 	{" ", noaction},
 	{"Add To Library", action(getlib, NULL)},
 	{"Load New Library", action(getuserlib, NULL)},
-	{"Save Library", action(savelib, NULL)},
+	{"Save Library", action(savelibpopup, Number(0))},
 	{" ", noaction},
         {"Clear Page", action(resetbutton, NULL)},
 	{" ", noaction},
@@ -301,7 +302,7 @@ menustruct OptionMenu[] = {
 	{"Edit in place", setaction(toggle, &areastruct.editinplace)},
 #ifdef SCHEMA
 	{"Show Pin Positions", action(toggle, &areastruct.pinpointon)},
-	{"Enable XSchema", action(doxschema, NULL)},
+	{"Enable XSchema", action(doxschema, Number(1))},
 #endif
 #ifdef HAVE_XPM
 	{"Disable Toolbar", action(dotoolbar, NULL)},
@@ -346,6 +347,7 @@ menustruct TopButtons[] = {
 
 short maxbuttons = sizeof(TopButtons) / sizeof(menustruct);
 
+#ifdef HAVE_XPM
 /* Toolbar buttons */
 toolbarstruct ToolBar[] = {
 	{"Box",    b_xpm,  toolaction(startbox, NULL),
@@ -394,15 +396,16 @@ toolbarstruct ToolBar[] = {
 		"go to library directory"},
 	{"Pagedir",pl_xpm, toolaction(startcatalog, Number(PAGELIB)),
 		"go to page directory"},
-	{"ZoomI",  z4_xpm, toolaction(zoomin, NULL),
+	{"ZoomI",  z4_xpm, toolaction(zoominrefresh, NULL),
 		"zoom in"},
-	{"ZoomO",  z5_xpm, toolaction(zoomout, NULL),
+	{"ZoomO",  z5_xpm, toolaction(zoomoutrefresh, NULL),
 		"zoom out"},
 	{"Help",   i_xpm,  toolaction(starthelp, NULL),
 		"pop up help window"},
 };
 
 short toolbuttons = sizeof(ToolBar) / sizeof(toolbarstruct);
+#endif
 
 #undef submenu
 #undef action
