@@ -1,6 +1,6 @@
 /*----------------------------------------------------------------------*/
 /* parameter.c								*/
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	*/
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	*/
 /*----------------------------------------------------------------------*/
 
 /*----------------------------------------------------------------------*/
@@ -20,7 +20,11 @@
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#else
 #include "Xw/Xw.h"
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Local includes							*/
@@ -41,7 +45,6 @@
 extern Globaldata xobjs;
 extern Clientdata areastruct;
 extern Widget     menuwidgets[];
-extern objectpair *pushlist;
 extern short textpos, textend;
 extern char _STR[150];
 
@@ -67,6 +70,11 @@ static u_char param_select[] = {
    SEL_ANY 					/* P_COLOR (tbd) */
 };
 
+#ifdef TCL_WRAPPER
+xcWidget *param_buttons[] = {
+   /* To be done---map buttons to Tk_Windows! */
+};
+#else
 Widget *param_buttons[] = {
    &ParametersPositionButton,		/* P_POSITION */
    &ParametersSubstringButton,		/* P_SUBSTRING */
@@ -83,6 +91,7 @@ Widget *param_buttons[] = {
    &ParametersLinewidthButton,		/* P_LINEWIDTH */
 /* &ParametersColorButton (does not exist yet) P_COLOR */
 };
+#endif
 
 /*----------------------------------------------*/
 /* Draw a circle at all parameter positions	*/
@@ -97,7 +106,7 @@ void indicateparams(genericptr thiselem)
       for (i = 0; i < thiselem->num_params; i++) {
          j = thiselem->passed[i].paramno;
 	 k = thiselem->passed[i].pointno;
-         ops = objectdata->params[j];
+         ops = topobject->params[j];
 	 switch(ops->which) {
 	    case P_POSITION: case P_POSITION_X: case P_POSITION_Y:
 	       switch(thiselem->type) {
@@ -118,7 +127,6 @@ void indicateparams(genericptr thiselem)
 		     break;
 	       }
 	       break;
-	       
 	 }
       }
    }
@@ -132,11 +140,18 @@ void indicateparams(genericptr thiselem)
 /* pgen = NULL returns menu to default settings */
 /*----------------------------------------------*/
 
+#ifdef TCL_WRAPPER
+void setparammarks(genericptr thiselem)
+{
+   /* to be done! */
+}
+#else
+
 void setparammarks(genericptr thiselem)
 {
    Widget w;
    Arg	wargs[1];
-   const int rlength = sizeof(param_buttons) / sizeof(int);
+   const int rlength = sizeof(param_buttons) / sizeof(Widget *);
    int i, j, paramno;
    oparamptr ops;
 
@@ -152,7 +167,7 @@ void setparammarks(genericptr thiselem)
    if (thiselem != NULL) {
       for (i = 0; i < thiselem->num_params; i++) {
          j = thiselem->passed[i].paramno;
-         ops = objectdata->params[j];
+         ops = topobject->params[j];
          w = *param_buttons[ops->which];
          XtSetArg(wargs[0], XtNsetMark, True);
          XtSetValues(w, wargs, 1);
@@ -169,6 +184,8 @@ void setparammarks(genericptr thiselem)
    }
 }
 
+#endif
+
 /*------------------------------------------------------*/
 /* Make numerical parameter substitutions into an	*/
 /* element.  "thisinst" may be NULL, in which case all	*/
@@ -375,7 +392,8 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 }
 
 /*------------------------------------------------------*/
-/* Same as above, but determines the object from
+/* Same as above, but determines the object from the	*/
+/* current page hierarchy.				*/
 /*------------------------------------------------------*/
 
 int psubstitute(objinstptr thisinst)
@@ -383,14 +401,32 @@ int psubstitute(objinstptr thisinst)
    objinstptr pinst;
    objectptr thisobj;
 
-   pinst = (thisinst == areastruct.topobject) ? NORMINST : thisinst;
+   pinst = (thisinst == areastruct.topinstance) ? areastruct.topinstance : thisinst;
    if (pinst == NULL) return -1;		/* there is no instance */
-   else if (pinst->params == NULL) return -1;	/* instance has no parameters */
+   else if (pinst->thisobject->params == NULL)
+      return -1;			    /* object has no parameters */
 
-   thisobj = (pinst == NULL) ? objectdata : pinst->thisobject;
+   thisobj = (pinst == NULL) ? topobject : pinst->thisobject;
    return opsubstitute(thisobj, pinst);
 }
 
+/*----------------------------------------------*/
+/* Check if an element contains a parameter.	*/
+/*----------------------------------------------*/
+
+Boolean has_param(genericptr celem)
+{
+   if (celem->type == LABEL) {
+      stringpart *cstr;
+      labelptr clab = (labelptr)celem;
+      for (cstr = clab->string; cstr != NULL; cstr = cstr->nextpart)
+	 if (cstr->type == PARAM_START)
+	    return TRUE;
+   }
+   if (celem->num_params > 0) return TRUE;
+   return FALSE;
+}
+
 /*------------------------------------------------------*/
 /* Find "current working values" in the element list of	*/
 /* an object, and write them into the instance's	*/
@@ -411,8 +447,8 @@ void pwriteback(objinstptr thisinst)
    float *fval;
    XPoint *setpt;
 
-   pinst = (thisinst == areastruct.topobject) ? NORMINST : thisinst;
-   thisobj = (pinst == NULL) ? objectdata : pinst->thisobject;
+   pinst = (thisinst == areastruct.topinstance) ? areastruct.topinstance : thisinst;
+   thisobj = (pinst == NULL) ? topobject : pinst->thisobject;
 
    /* make sure that all instance values exist */
    if (pinst != NULL) copyparams(pinst, pinst);
@@ -635,16 +671,28 @@ void replaceparams(objinstptr thisinst)
 void resolveparams(objinstptr thisinst)
 {
    objectptr thisobj;
+   liblistptr spec;
    oparamptr ops, ips;
    int i, nullparms = 0;
 
    /* If the instance has no parameters itself, ignore it. */
    if (thisinst == NULL || thisinst->params == NULL) return;
 
-   if (checklibtop() != NULL) {
-      /* printf("Came from library or top page:  changing default value\n"); */
-      replaceparams(thisinst);
-      return;
+   /* If the object was pushed into from a library, we want to change	*/
+   /* the default, not the instanced, parameter values.  However, this	*/
+   /* is not true for "virtual" library objects (in the instlist)	*/
+
+   if ((i = checklibtop()) >= 0) {
+      for (spec = xobjs.userlibs[i].instlist; spec != NULL;
+                spec = spec->next)
+         if (spec->thisinst == thisinst)
+	    break;
+
+      if ((spec == NULL) || (spec->virtual == FALSE)) {
+         /* printf("Came from library or top page:  changing default value\n"); */
+         replaceparams(thisinst);
+         return;
+      }
    }
 
    thisobj = thisinst->thisobject;
@@ -682,6 +730,12 @@ void resolveparams(objinstptr thisinst)
       free(thisinst->params);
       thisinst->params = NULL;
    }
+   else {
+      /* Object must recompute bounding box if any instance	*/
+      /* uses a non-default parameter.				*/
+
+      calcbboxvalues(thisinst, NULL);
+   }
 }
 
 /*------------------------------------------------------*/
@@ -758,26 +812,39 @@ void makenumericalp(genericptr *gelem, u_int mode)
    }
 
    /* cannot form a parameter on a top-level page */
-   if (is_page(objectdata) != -1) {
+   if (is_page(topobject) != -1) {
       Wprintf("Cannot form a parameter in a top-level page!");
       return;
    }
 
-   if (objectdata->params == NULL) {
-      objectdata->num_params = 1;
-      objectdata->params = (oparamptr *)malloc(sizeof(oparamptr));
+   /* Make sure the parameter doesn't already exist.	   */
+
+   if ((*gelem)->num_params > 0) {
+      for (epp = (*gelem)->passed; epp < (*gelem)->passed
+		+ (*gelem)->num_params; epp++) {
+	 ops = *(topobject->params + epp->paramno);
+	 if (ops->which == (u_char)mode) {
+	    fprintf(stderr, "Cannot duplicate a parameter!\n");
+	    return;
+	 }
+      }
+   }
+
+   if (topobject->params == NULL) {
+      topobject->num_params = 1;
+      topobject->params = (oparamptr *)malloc(sizeof(oparamptr));
    }
    else {
-      objectdata->num_params++;
-      objectdata->params = (oparamptr *)realloc(objectdata->params,
-		objectdata->num_params * sizeof(oparamptr));
+      topobject->num_params++;
+      topobject->params = (oparamptr *)realloc(topobject->params,
+		topobject->num_params * sizeof(oparamptr));
    }
-   *(objectdata->params + objectdata->num_params - 1) = (oparamptr)
+   *(topobject->params + topobject->num_params - 1) = (oparamptr)
 	malloc(sizeof(oparam));
 
-   initallinstances(objectdata, objectdata->num_params - 1);
+   initallinstances(topobject, topobject->num_params - 1);
 
-   ops = *(objectdata->params + objectdata->num_params - 1);
+   ops = *(topobject->params + topobject->num_params - 1);
    ops->type = XC_INT;		/* most commonly used value */
    ops->which = (u_char)mode;	/* what kind of parameter */
 
@@ -794,7 +861,7 @@ void makenumericalp(genericptr *gelem, u_int mode)
 		(*gelem)->num_params * sizeof(eparam));
    }
    epp = (*gelem)->passed + (*gelem)->num_params - 1;
-   epp->paramno = (u_char)(objectdata->num_params - 1);
+   epp->paramno = (u_char)(topobject->num_params - 1);
    epp->pointno = 0;
 
    switch((*gelem)->type) {
@@ -920,6 +987,7 @@ void makenumericalp(genericptr *gelem, u_int mode)
 	 }
 	 break;
    }
+
 }
 
 /*--------------------------------------------------------------*/
@@ -951,7 +1019,7 @@ void insertparam()
    /* Don't allow nested parameters */
 
    tlab = TOLABEL(EDITPART);
-   if (paramcross(objectdata, tlab)) {
+   if (paramcross(topobject, tlab)) {
       Wprintf("Parameters cannot be nested!");
       return;
    }
@@ -960,8 +1028,8 @@ void insertparam()
    /* If only one, then automatically use it.  Otherwise, prompt	*/
    /* for which parameter to use.					*/
 
-   for (i = 0; i < objectdata->num_params; i++) {
-      ops = *(objectdata->params + i);
+   for (i = 0; i < topobject->num_params; i++) {
+      ops = *(topobject->params + i);
       if (ops->type == XC_STRING) {
 	 nparms++;
 	 selparm = i;
@@ -970,12 +1038,14 @@ void insertparam()
    if (nparms > 1) {
       char *newstr, *sptr;
       char *sstart = (char *)malloc(250);
+      u_char *parammap = (u_char *)malloc(nparms);
       nparms = 0;
       strcpy(sstart, "Choose: ");
       sptr = sstart + 8;
-      for (i = 0; i < objectdata->num_params; i++) {
-	 ops = *(objectdata->params + i);
+      for (i = 0; i < topobject->num_params; i++) {
+	 ops = *(topobject->params + i);
 	 if (ops->type == XC_STRING) {
+	    parammap[nparms] = i;
 	    nparms++;
 	    if (nparms != 1) {
 	       strcat(sptr, ", ");
@@ -993,11 +1063,12 @@ void insertparam()
       free(sstart);
 
       /* This is a bit quirky;  but works for now.  Later, would prefer */
-      /* a GUI-oriented selection mechanism.				*/
-      selparm = getkeynum();
+      /* a GUI-oriented selection mechanism.                            */
+      selparm = parammap[getkeynum()];
+      free(parammap);
    }
 
-   if ((selparm >= 0) && (selparm < objectdata->num_params))
+   if ((selparm >= 0) && (selparm < topobject->num_params))
       labeltext(PARAM_START, (char *)&selparm);
    else
       Wprintf("Parameter number nonexistent.");
@@ -1014,14 +1085,14 @@ void makeparam(labelptr thislabel)
 
    /* cannot form a parameter on a top-level page */
 
-   if (is_page(objectdata) != -1) {
+   if (is_page(topobject) != -1) {
       Wprintf("Cannot form a parameter in a top-level page!");
       return;
    }
 
    /* make sure this does not overlap another parameter */
 
-   if (paramcross(objectdata, thislabel)) {
+   if (paramcross(topobject, thislabel)) {
       Wprintf("Parameters cannot be nested!");
       textend = 0;
       return;
@@ -1031,8 +1102,8 @@ void makeparam(labelptr thislabel)
    /* intended parameter boundaries				*/
 
    if (textend > 0 && textend < textpos) {  	/* partial string */
-      begpart = splitstring(textend, &thislabel->string, NULL);
-      endpart = splitstring(textpos, &thislabel->string, NULL);
+      begpart = splitstring(textend, &thislabel->string, areastruct.topinstance);
+      endpart = splitstring(textpos, &thislabel->string, areastruct.topinstance);
 
       /* note that "partial string" may overlap the beginning	*/
       /* or the end, so deal with these instances.		*/
@@ -1048,32 +1119,32 @@ void makeparam(labelptr thislabel)
       endpart = makesegment(&thislabel->string, NULL);
    }
    begpart->type = PARAM_START;
-   begpart->data.paramno = (objectdata->params == NULL) ? 0 :
-		objectdata->num_params;
+   begpart->data.paramno = (topobject->params == NULL) ? 0 :
+		topobject->num_params;
    endpart->type = PARAM_END;
 
    /* Now move the sections of string to the object parameter */
 
-   if (objectdata->params == NULL) {
-      objectdata->num_params = 1;
-      objectdata->params = (oparamptr *)malloc(sizeof(oparamptr));
+   if (topobject->params == NULL) {
+      topobject->num_params = 1;
+      topobject->params = (oparamptr *)malloc(sizeof(oparamptr));
    }
    else {
-      objectdata->num_params++;
-      objectdata->params = (oparamptr *)realloc(objectdata->params,
-		objectdata->num_params * sizeof(oparamptr));
+      topobject->num_params++;
+      topobject->params = (oparamptr *)realloc(topobject->params,
+		topobject->num_params * sizeof(oparamptr));
 
       /* We have added a parameter.  This requires that all instances	*/
       /* of the object which do not have NULL params must initialize	*/
       /* and NULL the pointer to the new parameter.			*/
 
-      initallinstances(objectdata, objectdata->num_params - 1);
+      initallinstances(topobject, topobject->num_params - 1);
    }
-   *(objectdata->params + objectdata->num_params - 1) = (oparamptr)
+   *(topobject->params + topobject->num_params - 1) = (oparamptr)
 	malloc(sizeof(oparam));
 
 
-   ops = *(objectdata->params + objectdata->num_params - 1);
+   ops = *(topobject->params + topobject->num_params - 1);
    ops->type = XC_STRING;
    ops->parameter.string = begpart->nextpart;
    begpart->nextpart = endpart->nextpart;
@@ -1098,6 +1169,9 @@ void initinst(objectptr topobj, objectptr refobj, short pnum)
 	 tinst = TOOBJINST(pgen);
 	 if (tinst->thisobject == refobj) {
 	    if (tinst->params != NULL) {
+	       /* Make sure we've allocated enough space for this parameter */
+	       tinst->params = (oparamptr *)realloc(tinst->params,
+			tinst->thisobject->num_params * sizeof(oparamptr));
 	       *(tinst->params + pnum) = NULL;
 	    }
 	 }
@@ -1105,6 +1179,37 @@ void initinst(objectptr topobj, objectptr refobj, short pnum)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Destroy the selected parameter in the indicated instance 	*/
+/*--------------------------------------------------------------*/
+
+void destroyinst(objinstptr tinst, objectptr refobj, short pnum)
+{
+   oparamptr *ops;
+   short k;
+
+   if (tinst->thisobject == refobj) {
+      if (tinst->thisobject->num_params > 0) {
+	 if (tinst->params != NULL) {
+	    ops = tinst->params + pnum;
+	    if (*ops != NULL)
+	       freelabel((*ops)->parameter.string);
+	    free(*ops);
+	    *ops = NULL;
+	    if (pnum == 0) {
+	       free(tinst->params);
+	       tinst->params = NULL;
+	    }
+	    else {
+	       /* Re-order the remaining parameters */
+	       for (k = pnum; k < tinst->thisobject->num_params - 1; k++)
+	          *(tinst->params + k) = *(tinst->params + k + 1);
+	    }
+	 }
+      }
+   }
+}
+
 /*--------------------------------------------------------------*/
 /* Search and destroy the selected parameter in all instances 	*/
 /* of the specified object.					*/
@@ -1112,36 +1217,15 @@ void initinst(objectptr topobj, objectptr refobj, short pnum)
 
 void searchinst(objectptr topobj, objectptr refobj, short pnum)
 {
-   oparamptr *ops;
    objinstptr tinst;
    genericptr *pgen;
-   short k;
 
    if (topobj == NULL) return;
 
    for (pgen = topobj->plist; pgen < topobj->plist + topobj->parts; pgen++) {
       if ((*pgen)->type == OBJECT) {
 	 tinst = TOOBJINST(pgen);
-	 if (tinst->thisobject == refobj) {
-	    if (tinst->thisobject->num_params > 0) {
-	       if (tinst->params != NULL) {
-	          ops = tinst->params + pnum;
-	          if (*ops != NULL)
-		     freelabel((*ops)->parameter.string);
-		  free(*ops);
-		  *ops = NULL;
-		  if (pnum == 0) {
-		     free(tinst->params);
-		     tinst->params = NULL;
-		  }
-		  else {
-	             /* Re-order the remaining parameters */
-	             for (k = pnum; k < tinst->thisobject->num_params - 1; k++)
-	                *(tinst->params + k) = *(tinst->params + k + 1);
-		  }
-	       }
-	    }
-	 }
+	 destroyinst(tinst, refobj, pnum);
       }
    }
 }
@@ -1156,12 +1240,14 @@ void initallinstances(objectptr thisobj, int p)
    int j, k;
 
    for (k = 0; k < xobjs.pages; k++) {
-      initinst(xobjs.pagelist[k]->pageobj, thisobj, p);
+      if (xobjs.pagelist[k]->pageinst != NULL)
+         initinst(xobjs.pagelist[k]->pageinst->thisobject, thisobj, p);
    }
    for (j = 0; j < xobjs.numlibs; j++) {
       for (k = 0; k < xobjs.userlibs[j].number; k++) {
          initinst(*(xobjs.userlibs[j].library + k), thisobj, p);
       }
+      initinst(xobjs.libtop[LIBRARY + j]->thisobject, thisobj, p);
    }
    for (k = 0; k < xobjs.delbuffer.number; k++) {
       initinst(*(xobjs.delbuffer.library + k), thisobj, p);
@@ -1190,8 +1276,9 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    genericptr *pgen;
    labelptr plab;
    oparamptr ops;
+   liblistptr spec;
    stringpart *strptr, *lastpart, *nextpart, *endpart, *newstr, *subs;
-   int p, j, k;
+   int p, j, k, l = -1;
    Boolean is_last = True;
    
    /* make sure there is a parameter here */
@@ -1210,7 +1297,7 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    /* instance of the parameter.  Otherwise, we delete the parameter	*/
    /* and also remove it from the parameters list.			*/
 
-   for (pgen = objectdata->plist; pgen < objectdata->plist + objectdata->parts;
+   for (pgen = topobject->plist; pgen < topobject->plist + topobject->parts;
 		pgen++) {
       if ((*pgen)->type == LABEL) {
 	 plab = TOLABEL(pgen);
@@ -1226,7 +1313,7 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
       }
    }
 
-   ops = *(objectdata->params + p);
+   ops = *(topobject->params + p);
    subs = ops->parameter.string;
 
    /* Copy the default parameter into the place we are unparameterizing */
@@ -1266,34 +1353,44 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
    /* substitutions which may have been made.		         */
 
    for (k = 0; k < xobjs.pages; k++) {
-      searchinst(xobjs.pagelist[k]->pageobj, objectdata, p);
+      if (xobjs.pagelist[k]->pageinst != NULL)
+         searchinst(xobjs.pagelist[k]->pageinst->thisobject, topobject, p);
    }
    for (j = 0; j < xobjs.numlibs; j++) {
       for (k = 0; k < xobjs.userlibs[j].number; k++) {
-         searchinst(*(xobjs.userlibs[j].library + k), objectdata, p);
+	 if (*(xobjs.userlibs[j].library + k) == topobject)
+	    l = j;
+	 else
+            searchinst(*(xobjs.userlibs[j].library + k), topobject, p);
       }
    }
    for (k = 0; k < xobjs.delbuffer.number; k++) {
-      searchinst(*(xobjs.delbuffer.library + k), objectdata, p);
+      searchinst(*(xobjs.delbuffer.library + k), topobject, p);
    }
+
+   /* Also check through all instances on the library page */
+   if (l >= 0)
+      for (spec = xobjs.userlibs[l].instlist; spec != NULL; spec = spec->next) 
+         destroyinst(spec->thisinst, topobject, p);
+
    free(ops);
    
    /* Re-order the remaining parameters */
 
-   for (k = p; k < objectdata->num_params - 1; k++)
-      *(objectdata->params + k) = *(objectdata->params + k + 1);
+   for (k = p; k < topobject->num_params - 1; k++)
+      *(topobject->params + k) = *(topobject->params + k + 1);
 
    /* Decrement the number of parameters */
 
-   if (--objectdata->num_params == 0) {
-      free(objectdata->params);
-      objectdata->params = NULL;
+   if (--topobject->num_params == 0) {
+      free(topobject->params);
+      topobject->params = NULL;
    }
 
    /* decrement the parameter ID number of all parameters in all strings */
    /* of this object whose parameter IDs are greater.			 */
 
-   for (pgen = objectdata->plist; pgen < objectdata->plist + objectdata->parts;
+   for (pgen = topobject->plist; pgen < topobject->plist + topobject->parts;
 		pgen++) {
       if ((*pgen)->type == LABEL) {
 	 plab = TOLABEL(pgen);
@@ -1311,7 +1408,7 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
 /* Wrapper for unmakeparam()				*/
 /*------------------------------------------------------*/
 
-void unparameterize(u_int mode)
+void unparameterize(int mode)
 {
    short *fselect, ptype;
    int locpos;
@@ -1330,7 +1427,7 @@ void unparameterize(u_int mode)
 		&& textend < textpos) {
       if (SELECTTYPE(areastruct.selectlist) != LABEL) return;	 /* Not a label */
       settext = SELTOLABEL(areastruct.selectlist);
-      strptr = findstringpart(textend, &locpos, settext->string, NORMINST);
+      strptr = findstringpart(textend, &locpos, settext->string, areastruct.topinstance);
       while (strptr != NULL && strptr->type != PARAM_END)
 	 strptr = strptr->nextpart;
       if (strptr == NULL) return;	/* No parameters */
@@ -1341,10 +1438,10 @@ void unparameterize(u_int mode)
       /* the whole parameter or the parameter start marker.		  */
 
       for (tmpptr = settext->string; tmpptr != NULL && tmpptr != strptr;
-		tmpptr = nextstringpart(tmpptr, NORMINST))
+		tmpptr = nextstringpart(tmpptr, areastruct.topinstance))
 	 if (tmpptr->type == PARAM_START) lastptr = tmpptr;
       /* Finish search, unlinking any parameter we might be inside */
-      for (; tmpptr != NULL; tmpptr = nextstringpart(tmpptr, NORMINST));
+      for (; tmpptr != NULL; tmpptr = nextstringpart(tmpptr, areastruct.topinstance));
 
       if (lastptr != NULL) unmakeparam(settext, lastptr);
    }
@@ -1359,11 +1456,11 @@ void unparameterize(u_int mode)
 	    if (strptr != NULL) unmakeparam(settext, strptr);
 	 }
 	 else if (mode == P_POSITION) {
-	    unmakenumericalp(objectdata->plist + (*fselect), P_POSITION_X);
-	    unmakenumericalp(objectdata->plist + (*fselect), P_POSITION_Y);
+	    unmakenumericalp(topobject->plist + (*fselect), P_POSITION_X);
+	    unmakenumericalp(topobject->plist + (*fselect), P_POSITION_Y);
 	 }
 	 else
-	    unmakenumericalp(objectdata->plist + (*fselect), mode);
+	    unmakenumericalp(topobject->plist + (*fselect), mode);
       }
    }
 }
@@ -1372,7 +1469,7 @@ void unparameterize(u_int mode)
 /* Wrapper for makeparam()					*/
 /*--------------------------------------------------------------*/
 
-void parameterize(u_int mode)
+void parameterize(int mode)
 {
    short *fselect, ptype;
    labelptr settext;
@@ -1393,11 +1490,11 @@ void parameterize(u_int mode)
          makeparam(settext);
       }
       else if (mode == P_POSITION) {
-	 makenumericalp(objectdata->plist + (*fselect), P_POSITION_X);
-	 makenumericalp(objectdata->plist + (*fselect), P_POSITION_Y);
+	 makenumericalp(topobject->plist + (*fselect), P_POSITION_X);
+	 makenumericalp(topobject->plist + (*fselect), P_POSITION_Y);
       }
       else
-	 makenumericalp(objectdata->plist + (*fselect), mode);
+	 makenumericalp(topobject->plist + (*fselect), mode);
    }
    objectdeselect();
 }
@@ -1412,7 +1509,7 @@ Boolean paramcross(objectptr tobj, labelptr tlab)
    stringpart *firstptr, *lastptr;
    int locpos;
 
-   lastptr = findstringpart(textpos, &locpos, tlab->string, NORMINST);
+   lastptr = findstringpart(textpos, &locpos, tlab->string, areastruct.topinstance);
 
    /* This text position can't be inside another parameter */
    for (firstptr = lastptr; firstptr != NULL; firstptr = firstptr->nextpart)
@@ -1420,8 +1517,9 @@ Boolean paramcross(objectptr tobj, labelptr tlab)
 
    /* The area between textend and textpos cannot contain a parameter */
    if (textend > 0)
-      for (firstptr = findstringpart(textend, &locpos, tlab->string, NORMINST);
-		firstptr != lastptr; firstptr = firstptr->nextpart)
+      for (firstptr = findstringpart(textend, &locpos, tlab->string,
+		areastruct.topinstance); firstptr != lastptr;
+		firstptr = firstptr->nextpart)
          if (firstptr->type == PARAM_START || firstptr->type == PARAM_END)
 	    return True;
 
@@ -1432,17 +1530,16 @@ Boolean paramcross(objectptr tobj, labelptr tlab)
 /* Check whether this page object was entered via a library page	*/
 /*----------------------------------------------------------------------*/
 
-objectptr checklibtop()
+int checklibtop()
 {
-   objectptr pobj;
-   objectpair *thispush = pushlist;
+   int i;
+   pushlistptr thispush;
 
-   while (thispush != NULL) {
-      pobj = thispush->thisobject;
-      if (is_library(pobj) >= 0) return pobj;
-      thispush = thispush->nextpair;
-   }
-   return (objectptr)NULL;
+   for (thispush = xobjs.stack; thispush != NULL; thispush = thispush->next)
+      if ((i = is_library(thispush->thisinst->thisobject)) >= 0)
+	 return i;
+
+   return -1;
 }
 
 /*----------------------------------------------------------------------*/
