@@ -3,6 +3,14 @@
 /*     Master list of function prototypes				*/
 /*----------------------------------------------------------------------*/
 
+/* from tclxcircuit.c */
+
+#ifdef TCL_WRAPPER
+extern Tk_EventProc *xctk_drag(ClientData, XEvent *);
+extern Tk_EventProc *xctk_draglscroll(ClientData, XEvent *);
+extern Tk_EventProc *xctk_dragfilebox(ClientData, XEvent *);
+#endif
+
 /* from elements.c: */
 
 extern void copyparams(objinstptr, objinstptr);
@@ -13,25 +21,25 @@ extern void splinedefaults(splineptr, int, int);
 extern void arcdefaults(arcptr, int, int);
 extern void pathdefaults(pathptr, int, int);
 extern void drawdot(int, int);
-extern void startspline(Widget, caddr_t, caddr_t);
-extern void starttext(Widget, u_int, caddr_t);
+extern void startspline(xcWidget, caddr_t, caddr_t);
+extern void starttext(xcWidget, pointertype, caddr_t);
 extern void textbutton(u_char, XButtonEvent *);
 extern void charreport(labelptr);
 extern void labeltext(int, char *);
 extern void textreturn();
 extern void rejustify(short);
 extern void splinebutton(XButtonEvent *);
-extern void trackspline(Widget, caddr_t, caddr_t);
-extern void startarc(Widget, caddr_t, caddr_t);
+extern void trackspline(xcWidget, caddr_t, caddr_t);
+extern void startarc(xcWidget, caddr_t, caddr_t);
 extern void arcbutton(XButtonEvent *);
-extern void trackarc(Widget, caddr_t, caddr_t);
-extern void startbox(Widget, caddr_t, caddr_t);
+extern void trackarc(xcWidget, caddr_t, caddr_t);
+extern void startbox(xcWidget, caddr_t, caddr_t);
 extern void boxbutton(XButtonEvent *);
-extern void trackbox(Widget, caddr_t, caddr_t);
-extern void trackwire(Widget, caddr_t, caddr_t);
+extern void trackbox(xcWidget, caddr_t, caddr_t);
+extern void trackwire(xcWidget, caddr_t, caddr_t);
 extern void startwire(XPoint);
 extern void finddir(polyptr);
-extern void trackpoly(Widget, caddr_t, caddr_t);
+extern void trackpoly(xcWidget, caddr_t, caddr_t);
 extern void setendpoint(short *, short, XPoint **, XPoint *);
 extern void reversepoints(XPoint *, short);
 extern void reversefpoints(XfPoint *, short);
@@ -48,35 +56,38 @@ extern void wirebutton(XButtonEvent *);
 extern Boolean recursefind(objectptr, objectptr);
 extern void transferselects();
 extern void newmatrix();
-extern void setpage();
+extern void setpage(Boolean);
 extern int changepage(short);
 extern void newpage(short);
-extern void startpush(Widget, caddr_t, caddr_t);
+extern void startpush(xcWidget, caddr_t, caddr_t);
 extern void pushobject(XButtonEvent *);
-extern void popobject(Widget, caddr_t, caddr_t);
-extern void resetbutton(Widget, caddr_t, caddr_t);
-extern void drawhbar(Widget, caddr_t, caddr_t);
-extern void drawvbar(Widget, caddr_t, caddr_t);
-extern void panhbar(Widget, caddr_t, XButtonEvent *);
-extern void endhbar(Widget, caddr_t, XButtonEvent *);
-extern void panvbar(Widget, caddr_t, XButtonEvent *);
-extern void endvbar(Widget, caddr_t, XButtonEvent *);
-extern void zoombox(Widget, caddr_t, caddr_t);
-extern void zoomin(Widget, caddr_t, XButtonEvent *);
-extern void zoomout(Widget, caddr_t, XButtonEvent *);
+extern void push_stack(pushlistptr *, objinstptr);
+extern void pop_stack(pushlistptr *);
+extern void free_stack(pushlistptr *);
+extern void popobject(xcWidget, caddr_t, caddr_t);
+extern void resetbutton(xcWidget, pointertype, caddr_t);
+extern void drawhbar(xcWidget, caddr_t, caddr_t);
+extern void drawvbar(xcWidget, caddr_t, caddr_t);
+extern void panhbar(xcWidget, caddr_t, XButtonEvent *);
+extern void endhbar(xcWidget, caddr_t, XButtonEvent *);
+extern void panvbar(xcWidget, caddr_t, XButtonEvent *);
+extern void endvbar(xcWidget, caddr_t, XButtonEvent *);
+extern void zoombox(xcWidget, caddr_t, caddr_t);
+extern void zoomin(xcWidget, caddr_t, XButtonEvent *);
+extern void zoomout(xcWidget, caddr_t, XButtonEvent *);
 extern void panbutton(u_int, XButtonEvent *);
-extern void zoominrefresh(Widget, caddr_t, XButtonEvent *);
-extern void zoomoutrefresh(Widget, caddr_t, XButtonEvent *);
+extern void zoominrefresh(xcWidget, caddr_t, XButtonEvent *);
+extern void zoomoutrefresh(xcWidget, caddr_t, XButtonEvent *);
 extern void panrefresh(u_int, XButtonEvent *);
-extern void centerpan(Widget, caddr_t, caddr_t);
+extern void centerpan(xcWidget, caddr_t, caddr_t);
 extern void checkwarp(XPoint *);
 extern int checkcycle(short, short);
 extern void nextpolycycle(polyptr, short);
 extern void nextsplinecycle(splineptr, short);
 extern void nextarccycle(arcptr, short);
 extern short getkeynum();
-extern void buttonhandler(Widget, caddr_t, XButtonEvent *);
-extern void keyhandler(Widget, caddr_t, XKeyEvent *);
+extern void buttonhandler(xcWidget, caddr_t, XButtonEvent *);
+extern void keyhandler(xcWidget, caddr_t, XKeyEvent *);
 extern void setsnap(short);
 extern void snapobject();
 extern int ipow10(int);
@@ -86,60 +97,67 @@ extern void printpos(short, short);
 extern void findwirex(XPoint *, XPoint *, XPoint *, XPoint *, int *);
 extern void findattach(XPoint *, int *, XPoint *);
 extern XPoint *pathclosepoint(pathptr, XPoint *);
-extern void drag(Widget, caddr_t, XButtonEvent *);
+extern void drag(xcWidget, caddr_t, XButtonEvent *);
 extern void rotatebutton(XButtonEvent *);
 extern void elemrotate(genericptr *, short);
 extern void objectrotate(short);
 extern void edit(XButtonEvent *);
 extern void pathedit(genericptr *, short);
+extern void xc_lower(short *);
+extern void xc_raise(short *);
 extern void exchange();
 extern void elhflip(genericptr *);
 extern void elvflip(genericptr *);
 extern void objectflip();
 extern void objectvflip();
-extern void makepress(XtIntervalId, caddr_t);
-extern void startrotate(Widget, caddr_t, caddr_t);
-extern void startcopy(Widget, caddr_t, caddr_t);
-extern void startedit(Widget, caddr_t, caddr_t);
-extern void startdelete(Widget, caddr_t, caddr_t);
+#ifdef TCL_WRAPPER
+extern xcTimeOutProc makepress(caddr_t);
+#else
+extern xcTimeOutProc makepress(caddr_t, xcIntervalId *);
+#endif
+extern void startrotate(xcWidget, pointertype, caddr_t);
+extern void startcopy(xcWidget, caddr_t, caddr_t);
+extern void startedit(xcWidget, caddr_t, caddr_t);
+extern void startdelete(xcWidget, caddr_t, caddr_t);
 extern void reviseselect(short *);
 extern void objectdelete(short);
 extern void deletebutton(XButtonEvent *);
-extern short xc_undelete(Widget, u_int, XButtonEvent *);
+extern short xc_undelete(xcWidget, pointertype, XButtonEvent *);
 extern void printname(objectptr);
 extern Boolean checkname(objectptr);
 extern objectptr finddot();
 extern void movepoints(genericptr *, short, short);
-extern void domakeobject(Widget, caddr_t);
-extern void selectsave(Widget, caddr_t, caddr_t);
+extern void domakeobject(xcWidget, caddr_t);
+extern void selectsave(xcWidget, caddr_t, caddr_t);
 extern void arceditpush(arcptr);
 extern void splineeditpush(splineptr);
 extern void polyeditpush(polyptr);
 extern void arccopy(arcptr, arcptr);
 extern void polycopy(polyptr, polyptr);
 extern void splinecopy(splineptr, splineptr);
+extern void instcopy(objinstptr, objinstptr);
 extern void copybutton(XButtonEvent *);
 extern void selectbutton(XButtonEvent *);
 extern void releasebutton(XButtonEvent *);
 extern void pathbutton(genericptr *, XButtonEvent *);
-extern void resizearea(Widget, caddr_t, caddr_t);
-extern void drawarea(Widget, caddr_t, caddr_t);
+extern void resizearea(xcWidget, caddr_t, caddr_t);
+extern void drawarea(xcWidget, caddr_t, caddr_t);
 
 /* from filelist.c: */
 
 extern int fcompare(const void *, const void *);
-extern void dragfilebox(Widget, caddr_t, XMotionEvent *);
-extern void startfiletrack(Widget, caddr_t, XCrossingEvent *);
-extern void endfiletrack(Widget, caddr_t, XCrossingEvent *);
+extern void dragfilebox(xcWidget, caddr_t, XMotionEvent *);
+extern void startfiletrack(xcWidget, caddr_t, XCrossingEvent *);
+extern void endfiletrack(xcWidget, caddr_t, XCrossingEvent *);
 extern char *getcrashfilename();
 extern void crashrecover();
 extern void findcrashfiles();
-extern void listfiles(Widget, popupstruct *, caddr_t);
-extern void newfilelist(Widget, popupstruct *);
-extern void fileselect(Widget, popupstruct *, XButtonEvent *);
-extern void showlscroll(Widget, caddr_t, caddr_t);
-extern void draglscroll(Widget, popupstruct *, XButtonEvent *);
-extern void genfilelist(Widget, popupstruct *, Dimension);
+extern void listfiles(xcWidget, popupstruct *, caddr_t);
+extern void newfilelist(xcWidget, popupstruct *);
+extern void fileselect(xcWidget, popupstruct *, XButtonEvent *);
+extern void showlscroll(xcWidget, caddr_t, caddr_t);
+extern void draglscroll(xcWidget, popupstruct *, XButtonEvent *);
+extern void genfilelist(xcWidget, popupstruct *, Dimension);
 extern int lookdirectory(char *);
 
 /* from files.c: */
@@ -147,7 +165,7 @@ extern int lookdirectory(char *);
 extern void ridnewline(char *);
 extern void free_single(genericptr *);
 extern void reset(objectptr, short);
-extern void topreset();
+extern void pagereset(short);
 extern void initmem(objectptr);
 extern void freelabel(stringpart *);
 extern Boolean compare_single(genericptr *, genericptr *);
@@ -155,17 +173,18 @@ extern Boolean elemcompare(genericptr *, genericptr *);
 extern short objcompare(objectptr, objectptr);
 extern float getpsscale(float, short);
 extern void dostcount(FILE *, short *, short);
+extern short printparams(FILE *, objinstptr, short);
 extern void varcheck(FILE *, short, objectptr, short *, genericptr, u_char);
 extern void varfcheck(FILE *, float, objectptr, short *, genericptr, u_char);
 extern void varpcheck(FILE *, short, objectptr, int, int, short *, genericptr, u_char);
-extern void getfile(Widget, void *, caddr_t);
+extern void getfile(xcWidget, pointertype, caddr_t);
 extern Boolean nextfilename();
 extern void loadfontlib();
 extern void loadglib(Boolean, short, short);
 extern void loadulib();
 extern void loadblib();
-extern void getlib(Widget, caddr_t, caddr_t);
-extern void getuserlib(Widget, caddr_t, caddr_t);
+extern void getlib(xcWidget, caddr_t, caddr_t);
+extern void getuserlib(xcWidget, caddr_t, caddr_t);
 extern void loadlibrary(short);
 extern void startloadfile();
 extern void importfile();
@@ -176,18 +195,23 @@ extern char *advancetoken(char *);
 extern char *varpscan(objectptr, char *, short *, genericptr, int, int, u_char);
 extern char *varscan(objectptr, char *, short *, genericptr, u_char);
 extern char *varfscan(objectptr, char *, float *, genericptr, u_char);
+extern objinstptr addtoinstlist(int, objectptr, Boolean);
 extern Boolean objectread(FILE *, objectptr, short, short, short, char *, int);
-extern void setfile(Widget, Widget, caddr_t);
-extern XtTimerCallbackProc savetemp(Widget, XtIntervalId *);
-extern void savelib(Widget, caddr_t, caddr_t);
-extern void savelibrary(Widget, int);
+extern void setfile(xcWidget, xcWidget, caddr_t);
+#ifdef TCL_WRAPPER
+extern xcTimeOutProc savetemp(caddr_t);
+#else
+extern xcTimeOutProc savetemp(caddr_t, xcIntervalId *);
+#endif
+extern void savelibpopup(xcWidget, pointertype, caddr_t);
+extern void savelibrary(xcWidget, int);
 extern void findfonts(objectptr, short *);
-extern void savefile(objectptr, short);
+extern void savefile(short);
 extern int printRGBvalues(char *, int, const char *);
-extern int nosprint(char *);
-extern Boolean tilde_expand(char *);
+extern char *nosprint(char *);
+extern Boolean xc_tilde_expand(char *);
 extern short writelabel(FILE *, stringpart *, short *);
-extern void writesegment(stringpart *, float *, int *);
+extern char *writesegment(stringpart *, float *, int *);
 extern void writelabelsegs(FILE *, short *, stringpart *);
 extern void printobjects(FILE *, objectptr, float, float, float, short, short,
                          objectptr **, short *, int);
@@ -248,51 +272,51 @@ extern void bboxcalc(short, short *, short *);
 extern void calcextents(genericptr *, short *, short *, short *, short *);
 extern void objinstbbox(objinstptr, XPoint *);
 extern void labelbbox(labelptr, XPoint *, objinstptr);
-extern void calcinstbbox(genericptr *, short *, short *, short *, short *);
-extern void calcbboxsingle(genericptr *, short *, short *, short *, short *
-#ifdef SCHEMA
-                           ,short *, short *, short *, short *
-#endif
-);
+extern void calcbboxsingle(genericptr *, objinstptr, short *, short *, short *, short *);
 extern Boolean object_in_library(short, objectptr);
 extern short find_object(objectptr, objectptr);
 extern void updatepagebounds(objectptr);
-extern void calcbbox(objectptr);
+extern void calcbbox(objinstptr);
 extern void calcbboxparam(objectptr, int);
-extern void singlebbox(objectptr, genericptr *);
+extern void singlebbox(genericptr *);
 extern void calcbboxselect();
-extern void calcbboxvalues(objectptr, u_char, genericptr *);
-extern void centerview(objectptr);
-extern void refresh(Widget, caddr_t, caddr_t);
-extern void zoomview(Widget, caddr_t, caddr_t);
-extern void UDrawSimpleLine(objinstptr, XPoint *, XPoint *);
-extern void UDrawLine(objinstptr, XPoint *, XPoint *);
+extern void calcbboxvalues(objinstptr, genericptr *);
+extern void centerview(objinstptr);
+extern void refresh(xcWidget, caddr_t, caddr_t);
+extern void zoomview(xcWidget, caddr_t, caddr_t);
+extern void UDrawSimpleLine(XPoint *, XPoint *);
+extern void UDrawLine(XPoint *, XPoint *);
 extern void UDrawCircle(XPoint *, u_char);
 extern void UDrawX(labelptr);
 #ifdef SCHEMA
 extern void UDrawXDown(labelptr);
+extern int  toplevelwidth(objinstptr);
+extern int  toplevelheight(objinstptr);
+extern void extendschembbox(objinstptr, XPoint *, XPoint *);
 #endif
 extern void pinadjust(short, short *, short *, short);
 extern void UDrawTextLine(labelptr, short);
 extern void UDrawTLine(labelptr);
 extern void UDrawXLine(XPoint, XPoint);
 extern void UDrawBox(XPoint, XPoint);
-extern void UDrawBBox(objectptr);
+extern void UDrawBBox();
 extern void strokepath(XPoint *, short, short, float);
 extern void makesplinepath(splineptr, XPoint *);
-extern void UDrawSpline(objinstptr, splineptr);
-extern void UDrawEditSpline(objinstptr, splineptr);
-extern void UDrawPolygon(objinstptr, polyptr);
-extern void UDrawArc(objinstptr, arcptr);
-extern void UDrawPath(objinstptr, pathptr);
-extern void UDrawObject(objinstptr, objectptr, short, int);
+extern void UDrawSpline(splineptr);
+extern void UDrawEditSpline(splineptr);
+extern void UDrawPolygon(polyptr);
+extern void UDrawArc(arcptr);
+extern void UDrawPath(pathptr);
+extern void UDrawObject(objinstptr, short, int, pushlistptr *);
 
 /* from help.c: */
 
-extern void printhelppix(Widget);
-extern void starthelp(Widget, caddr_t, caddr_t);
-extern void simplescroll(Widget, caddr_t, XPointerMovedEvent *);
-extern void printhelp(Widget, caddr_t, caddr_t);
+extern void showhsb(xcWidget, caddr_t, caddr_t);
+extern void printhelppix();
+extern void starthelp(xcWidget, caddr_t, caddr_t);
+extern void simplescroll(xcWidget, xcWidget, XPointerMovedEvent *);
+extern void exposehelp(xcWidget, caddr_t, caddr_t);
+extern void printhelp(xcWidget);
 
 /* from keybindings.c */
 
@@ -323,7 +347,7 @@ extern int is_page(objectptr);
 extern int is_library(objectptr);
 extern void pagecatbutton(XButtonEvent *);
 extern void pageinstpos(short, short, objinstptr, int, int, int, int);
-extern objinstptr makeinstofpage(objectptr, objectptr);
+extern objinstptr newpageinst(objectptr);
 extern void computespacing(short, int *, int *, int *, int *);
 extern void composepagelib(short);
 extern void updatepagelib(short, short);
@@ -331,119 +355,124 @@ extern void pagecatmove(XKeyEvent *);
 extern void composelib(short);
 extern short finddepend(objinstptr, objectptr **);
 extern void cathide();
+extern void catvirtualcopy();
 extern void catdelete();
 extern void catmove(XKeyEvent *);
 extern void copycat();
 extern void catbutton(u_int, XButtonEvent *);
 extern void changecat();
-extern void startcatalog(Widget, u_int, caddr_t);
+extern void startcatalog(xcWidget, pointertype, caddr_t);
 
 /* from menucalls.c: */
 
-extern void dotoolbar(Widget, caddr_t, caddr_t);
-extern void overdrawpixmap(Widget);
-extern void getgeneric(buttonsave *, Widget, void (*getfunction)(), void *);
-extern void setgrid(Widget, float *);
+extern void dotoolbar(xcWidget, caddr_t, caddr_t);
+extern void overdrawpixmap(xcWidget);
+extern void getgeneric(buttonsave *, xcWidget, void (*getfunction)(), void *);
+extern void setgrid(xcWidget, float *);
 extern void measurestr(float, char *);
-extern void getsnapspace(Widget, caddr_t, caddr_t);
-extern void getgridspace(Widget, caddr_t, caddr_t);
-extern void setfloat(Widget, float *);
-extern void setwidth(Widget, float *);
-extern void settsize(Widget, labelptr);
-extern void autoset(Widget, WidgetList, caddr_t);
-extern void autostop(Widget, caddr_t, caddr_t);
-extern void autoscale();
-extern void setscaley(Widget, float *);
-extern void setscalex(Widget, float *);
-extern void setorient(Widget, short *);
-extern void setpmode(Widget, short *);
-extern void setpagesize(Widget, XPoint *);
-extern void getkern(Widget, caddr_t, caddr_t);
-extern void setkern(Widget, stringpart *);
-extern void getdscale(Widget, caddr_t, caddr_t);
-extern void setdscale(Widget, XPoint *);
-extern void gettsize(Widget, caddr_t, caddr_t);
-extern void setosize(Widget, objinstptr);
-extern void getosize(Widget, caddr_t, caddr_t);
-extern void getwirewidth(Widget, caddr_t, caddr_t);
-extern void setwwidth(Widget, void *);
-extern void getwwidth(Widget, caddr_t, caddr_t);
-extern void getfloat(Widget, float *, caddr_t);
-extern void setfilename(Widget, char **);
-extern void setpagelabel(Widget, char *);
+extern void getsnapspace(xcWidget, caddr_t, caddr_t);
+extern void getgridspace(xcWidget, caddr_t, caddr_t);
+extern void setfloat(xcWidget, float *);
+extern void setwidth(xcWidget, float *);
+extern void settsize(xcWidget, labelptr);
+extern void autoset(xcWidget, xcWidgetList, caddr_t);
+extern void autostop(xcWidget, caddr_t, caddr_t);
+extern void autoscale(int);
+extern void setscaley(xcWidget, float *);
+extern void setscalex(xcWidget, float *);
+extern void setorient(xcWidget, short *);
+extern void setpmode(xcWidget, short *);
+extern void setpagesize(xcWidget, XPoint *);
+extern void getkern(xcWidget, caddr_t, caddr_t);
+extern void setkern(xcWidget, stringpart *);
+extern void getdscale(xcWidget, caddr_t, caddr_t);
+extern void setdscale(xcWidget, XPoint *);
+extern labelptr gettextsize(float **);
+extern void gettsize(xcWidget, caddr_t, caddr_t);
+extern void setosize(xcWidget, objinstptr);
+extern void getosize(xcWidget, caddr_t, caddr_t);
+extern void getwirewidth(xcWidget, caddr_t, caddr_t);
+extern void setwwidth(xcWidget, void *);
+extern void getwwidth(xcWidget, caddr_t, caddr_t);
+extern void getfloat(xcWidget, float *, caddr_t);
+extern void setfilename(xcWidget, char **);
+extern void setpagelabel(xcWidget, char *);
 extern void makenewfontbutton();
 extern void makenewencodingbutton(char *, char);
 extern void togglefontmark(int);
-extern void toggleexcl(Widget, menuptr, int);
-extern void toggle(Widget, Boolean *, caddr_t);
+extern void toggleexcl(xcWidget, menuptr, int);
+extern void toggle(xcWidget, Boolean *, caddr_t);
 extern void setcolorscheme(Boolean);
-extern void inversecolor(Widget, Boolean *, caddr_t);
+extern void inversecolor(xcWidget, Boolean *, caddr_t);
 extern void togglegrid(u_short);
 extern void setgridtype(char *);
-extern void getgridtype(Widget, u_int, caddr_t);
-extern void newlibrary(Widget, caddr_t, caddr_t);
+extern void getgridtype(xcWidget, pointertype, caddr_t);
+extern void newlibrary(xcWidget, caddr_t, caddr_t);
 extern int findemptylib();
 extern int createlibrary();
-extern void newpagemenu(Widget, u_int, caddr_t);
+extern void newpagemenu(xcWidget, pointertype, caddr_t);
 extern void makepagebutton();
 extern void renamepage(short);
 extern void renamelib(short);
-extern void setstylemarks(Widget, u_int, Boolean);
 extern void setcolormark(int);
-extern void setallstylemarks(short);
+extern void setallstylemarks(u_short);
 extern polyptr checkforbbox(objectptr);
-extern void getfill(Widget, u_int, caddr_t);
-extern void makebbox(Widget, u_int, caddr_t);
-extern void getset(Widget, u_int, caddr_t);
-extern void getline(Widget, u_int, caddr_t);
-extern void getopaque(Widget, u_int, caddr_t);
-extern void setcolor(Widget, u_int, caddr_t);
-extern void setnewcolor(Widget, caddr_t);
-extern void addnewcolor(Widget, caddr_t, caddr_t);
+extern void setfill(xcWidget, pointertype, caddr_t);
+extern void makebbox(xcWidget, pointertype, caddr_t);
+extern void setclosure(xcWidget, pointertype, caddr_t);
+extern void setline(xcWidget, pointertype, caddr_t);
+extern void setopaque(xcWidget, pointertype, caddr_t);
+#ifdef TCL_WRAPPER
+extern void setcolor(xcWidget, int);
+#else
+extern void setcolor(xcWidget, pointertype, caddr_t);
+#endif
+extern void setnewcolor(xcWidget, caddr_t);
+extern void addnewcolor(xcWidget, caddr_t, caddr_t);
 extern void setfontmarks(short, short);
-extern void startparam(Widget, u_int, caddr_t);
-extern void startunparam(Widget, u_int, caddr_t);
+extern void startparam(xcWidget, pointertype, caddr_t);
+extern void startunparam(xcWidget, pointertype, caddr_t);
 extern void setdefaultfontmarks();
-extern void dopinvisible(Widget, labelptr);
-extern void doflipflip(Widget, labelptr);
-extern void setjust(Widget, u_int, labelptr, short);
-extern void setvjust(Widget, u_int, caddr_t);
-extern void sethjust(Widget, u_int, caddr_t);
-extern void setflipinv(Widget, u_int, caddr_t);
-extern void setpinvisible(Widget, u_int, caddr_t);
-extern void boxedit(Widget, u_int, caddr_t);
-extern void locloadfont(Widget, char *);
-extern void addnewfont(Widget, caddr_t, caddr_t);
+extern void dopinvisible(xcWidget, labelptr);
+extern void doflipflip(xcWidget, labelptr);
+extern void setjust(xcWidget, pointertype, labelptr, short);
+extern void setvjust(xcWidget, pointertype, caddr_t);
+extern void sethjust(xcWidget, pointertype, caddr_t);
+extern void setflipinv(xcWidget, pointertype, caddr_t);
+extern void setpinvisible(xcWidget, pointertype, caddr_t);
+extern void boxedit(xcWidget, pointertype, caddr_t);
+extern void locloadfont(xcWidget, char *);
+extern void addnewfont(xcWidget, caddr_t, caddr_t);
 extern short findbestfont(short, short, short, short);
-extern void setfontval(Widget, u_int, labelptr);
-extern void setfont(Widget, u_int, caddr_t);
-extern void setfontstyle(Widget, u_int, labelptr);
-extern void fontstyle(Widget, u_int, caddr_t);
-extern void setfontencoding(Widget, u_int, labelptr);
-extern void fontencoding(Widget, u_int, caddr_t);
-extern void addtotext(Widget, u_int, caddr_t);
-extern void position_popup(Widget, Widget);
-extern void border_popup(Widget, caddr_t, caddr_t);
-extern void color_popup(Widget, caddr_t, caddr_t);
-extern void fill_popup(Widget, caddr_t, caddr_t);
-extern void param_popup(Widget, caddr_t, caddr_t);
+extern void setfontval(xcWidget, pointertype, labelptr);
+extern void setfont(xcWidget, pointertype, caddr_t);
+extern void setfontstyle(xcWidget, pointertype, labelptr);
+extern void fontstyle(xcWidget, pointertype, caddr_t);
+extern void setfontencoding(xcWidget, pointertype, labelptr);
+extern void fontencoding(xcWidget, pointertype, caddr_t);
+extern void addtotext(xcWidget, pointertype, caddr_t);
+extern void position_popup(xcWidget, xcWidget);
+extern void border_popup(xcWidget, caddr_t, caddr_t);
+extern void color_popup(xcWidget, caddr_t, caddr_t);
+extern void fill_popup(xcWidget, caddr_t, caddr_t);
+extern void param_popup(xcWidget, caddr_t, caddr_t);
 extern void dospecial(void);
 
 /* from netlist.c: */
 
 #ifdef SCHEMA
-extern void createnets(Boolean, char *);
+extern void createnets();
 extern void destroynets();
 extern void gennet(char *, char *);
 extern Boolean nonnetwork(polyptr);
 extern int globalmax();
-extern void gennetlist(objectptr, char *);
-extern void gencalllist(objectptr, Boolean);
-extern void search_on_siblings(objinstptr, objinstptr, objectpair *,
+extern void gennetlist(objectptr, objinstptr);
+extern void gencalllist(objectptr);
+extern void search_on_siblings(objinstptr, objinstptr, pushlistptr,
 		short, short, short, short);
-extern int is_resolved(genericptr *, objectpair *, objectptr *);
-extern void highlightnet(objectptr, objinstptr, int);
-extern int pushnetwork(objectpair *, objectptr);
+extern int is_resolved(genericptr *, pushlistptr, objectptr *);
+extern Boolean highlightnet(objectptr, objinstptr, int, u_char);
+extern int pushnetwork(pushlistptr, objectptr);
 extern int onsegment(XPoint *, XPoint *, XPoint *);
 extern Boolean neardist(long);
 extern Boolean nearpoint(XPoint *, XPoint *);
@@ -464,7 +493,7 @@ extern void makelocalpins(objectptr, CalllistPtr, char *);
 extern int porttonet(objectptr, int);
 extern int pointtonet(objectptr, XPoint *);
 extern stringpart *nettopin(int, objectptr, char *);
-extern int pintonet(objectptr, labelptr);
+extern int pintonet(objectptr, objinstptr, labelptr);
 extern char *parseinfo(objectptr, CalllistPtr, char *, char *, Boolean);
 extern int writedevice(FILE *, char *, objectptr, CalllistPtr, char *);
 extern void writeflat(objectptr, CalllistPtr, char *, FILE *, char *);
@@ -475,8 +504,10 @@ extern void writepcb(objectptr, CalllistPtr, char *, char *);
 extern void outputpcb(objectptr, FILE *);
 extern void freepcb();
 extern void freenets(objectptr);
+extern void freetemplabels(objectptr);
 extern void freeglobals();
 extern int cleartraversed(objectptr, int);
+extern int checkvalid(objectptr);
 extern void clearpins(objectptr);
 #endif
 
@@ -493,8 +524,8 @@ extern void searchinst(objectptr, objectptr, short);
 extern void initallinstances(objectptr, int);
 extern stringpart *searchparam(stringpart *);
 extern void unmakeparam(labelptr, stringpart *);
-extern void unparameterize(u_int);
-extern void parameterize(u_int);
+extern void unparameterize(int);
+extern void parameterize(int);
 extern genericptr findparam(objectptr, void *, u_char);
 extern Boolean paramcross(objectptr, labelptr);
 extern oparamptr parampos(objectptr, labelptr, char *, short *, short *);
@@ -505,7 +536,7 @@ extern short paramlen(u_char *);
 extern int natstrlen(u_char *);
 extern int natstrcmp(u_char *, u_char *);
 extern void curtail(u_char *);
-extern objectptr checklibtop();
+extern int checklibtop();
 extern void removeinst(objinstptr);
 extern void removeparams(objectptr);
 extern void resolveparams(objinstptr);
@@ -518,7 +549,7 @@ extern void exit_interpreter();
 #endif
 
 #ifdef HAVE_XPM
-extern Widget *pytoolbuttons(int *);
+extern xcWidget *pytoolbuttons(int *);
 #endif
 
 /* from rcfile.c: */
@@ -554,13 +585,13 @@ extern int reset_gs();
 
 /* from schema.c: */
 
-extern void callgennet(Widget, u_int, caddr_t);
-extern void doxschema(Widget, caddr_t, caddr_t);
-extern void startconnect(Widget, caddr_t, caddr_t);
-extern void connectivity(Widget, caddr_t, caddr_t);
-extern Boolean setobjecttype(objectptr, char *);
-extern void pinconvert(labelptr, u_int);
-extern void dopintype(Widget, u_int, caddr_t);
+extern void callgennet(xcWidget, pointertype, caddr_t);
+extern void doxschema(xcWidget, pointertype, caddr_t);
+extern void startconnect(xcWidget, caddr_t, caddr_t);
+extern void connectivity(xcWidget, caddr_t, caddr_t);
+extern Boolean setobjecttype(objectptr);
+extern void pinconvert(labelptr, pointertype);
+extern void dopintype(xcWidget, pointertype, caddr_t);
 extern void setsymschem();
 extern int findpageobj(objectptr);
 extern int findsubschems(int, objectptr, int);
@@ -568,15 +599,15 @@ extern void copypinlabel(labelptr);
 extern int checkschem(objectptr, char *);
 extern int checksym(objectptr, char *);
 extern void changeotherpins(labelptr, stringpart *);
-extern void swapschem(Widget, u_int, caddr_t);
-extern void dobeforeswap(Widget, caddr_t, caddr_t);
+extern void swapschem(xcWidget, pointertype, caddr_t);
+extern void dobeforeswap(xcWidget, caddr_t, caddr_t);
 extern void schemdisassoc();
-extern void startschemassoc(Widget, caddr_t, caddr_t);
+extern void startschemassoc(xcWidget, caddr_t, caddr_t);
 extern void schemassoc(objectptr, objectptr);
 
 /* from selection.c: */
 
-extern void selectfilter(Widget, caddr_t, caddr_t);
+extern void selectfilter(xcWidget, pointertype, caddr_t);
 extern Boolean checkselect(short);
 extern void geneasydraw(short, short, objectptr, objinstptr);
 extern void gendrawselected(short *, objectptr, objinstptr);
@@ -587,14 +618,18 @@ extern int test_insideness(int, int, XPoint *);
 extern Boolean pathselect(genericptr *, short);
 extern Boolean areaelement(genericptr *);
 extern void selectarea();
-extern void startdesel(Widget, caddr_t, caddr_t);
-extern void deselect(Widget, caddr_t, caddr_t);
+extern void startdesel(xcWidget, caddr_t, caddr_t);
+extern void deselect(xcWidget, caddr_t, caddr_t);
 extern void drawselects();
 extern void clearselects();
 extern void objectdeselect();
-extern genericptr recurselect(objectpair **);
-extern void startselect(XtIntervalId, caddr_t);
-extern void trackselarea(Widget, XtPointer, XMotionEvent *);
+extern genericptr recurselect(short, pushlistptr *);
+#ifdef TCL_WRAPPER
+extern xcTimeOutProc startselect(caddr_t);
+#else
+extern xcTimeOutProc startselect(caddr_t, xcIntervalId *);
+#endif
+extern void trackselarea();
 
 /* from text.c: */
 
@@ -613,7 +648,7 @@ extern char *textprint(stringpart *, objinstptr);
 extern int textcomp(stringpart *, char *, objinstptr);
 extern int textncomp(stringpart *, char *, objinstptr);
 extern int stringcomp(stringpart *, stringpart *);
-extern int stringcomprelaxed(stringpart *, stringpart *);
+extern int stringcomprelaxed(stringpart *, stringpart *, objinstptr);
 extern int stringparts(stringpart *);
 extern int stringlength(stringpart *, Boolean, objinstptr);
 extern stringpart *stringcopy(stringpart *);
@@ -621,8 +656,8 @@ extern stringpart *stringcopyall(stringpart *, objinstptr);
 extern stringpart *stringcopyback(stringpart *, objinstptr);
 extern stringpart *deletestring(stringpart *, stringpart **, objinstptr);
 
-extern short UDrawChar(objinstptr, u_char, short, short, int, int);
-extern void UDrawString(objinstptr, labelptr, int);
+extern short UDrawChar(u_char, short, short, int, int);
+extern void UDrawString(labelptr, int, objinstptr);
 extern TextExtents ULength(stringpart *, objinstptr, float, short, XPoint *);
 extern void undrawtext(labelptr);
 extern void redrawtext(labelptr);
@@ -633,23 +668,30 @@ extern void fontcatbutton(XButtonEvent *);
 
 extern void printeventmode();
 extern void Wprintf(char *);
-extern void popupprompt(Widget, char *, char *, void (*function)(),
+extern void popupprompt(xcWidget, char *, char *, void (*function)(),
                         buttonsave *, const char *);
-extern void getproptext(Widget, propstruct *, caddr_t);
+extern void getproptext(xcWidget, propstruct *, caddr_t);
 extern int rgb_alloccolor(int, int, int);
-extern void addtocolorlist(Widget, int);
+extern void addtocolorlist(xcWidget, int);
 extern void addnewcolorentry(int);
-extern void dooutput(Widget, caddr_t, caddr_t);
+extern int xc_getlayoutcolor(int);
+extern void outputpopup(xcWidget, caddr_t, caddr_t);
 extern void docommand();
 extern void W1printf(char *);
 extern void W2printf(char *);
-extern void destroypopup(Widget, popupstruct *, caddr_t);
+extern void destroypopup(xcWidget, popupstruct *, caddr_t);
 extern int xc_alloccolor(char *);
 extern void dointr(int);
-extern void DoNothing(Widget, caddr_t, caddr_t);
+extern void DoNothing(xcWidget, caddr_t, caddr_t);
 extern u_short countchanges(char **);
-extern void quitcheck(Widget, caddr_t, caddr_t);
-extern void quit(Widget, caddr_t);
+extern void quitcheck(xcWidget, caddr_t, caddr_t);
+extern void quit(xcWidget, caddr_t);
 extern void resizetoolbar();
-extern void updatetext(Widget, WidgetList, caddr_t);
+extern void writescalevalues(char *, char *, char *);
+#ifdef TCL_WRAPPER
+extern int Tcl_GetHandleFromObj(Tcl_Interp *, Tcl_Obj *, void **);
+#else
+extern void updatetext(xcWidget, xcWidgetList, caddr_t);
+#endif
+
 extern void makecursors();
