@@ -1,11 +1,11 @@
 /*----------------------------------------------------------------------*/
 /* render.c --- Ghostscript rendering of background PostScript files	*/
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	*/
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	*/
 /* These routines work only if ghostscript is on the system.		*/
 /*----------------------------------------------------------------------*/
 
-#undef GS_DEBUG
-/* #define GS_DEBUG */
+/* #undef GS_DEBUG */
+#define GS_DEBUG
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,6 +29,10 @@
 /* Local includes                                                         */
 /*------------------------------------------------------------------------*/
 
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#endif
+
 #include "colordefs.h"
 #include "xcircuit.h"
 
@@ -45,7 +49,6 @@ extern char _STR2[250], _STR[150];
 extern Globaldata xobjs;
 extern Clientdata areastruct;
 extern Display *dpy;
-extern Window win;
 extern short eventmode;
 extern int *appcolors;
 extern int number_colors;
@@ -92,10 +95,10 @@ void ghostinit()
 		areastruct.width * 75 / 72,
 		areastruct.height * 75 / 72,
 		75.0, 75.0, 0, 0, 0, 0);
-   XChangeProperty(dpy, win, gv, XA_STRING, 8, PropModeReplace,
+   XChangeProperty(dpy, areastruct.areawin, gv, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    sprintf(_STR, "%s %d %d", "Color", (int)FOREGROUND, (int)BACKGROUND);
-   XChangeProperty(dpy, win, gvc, XA_STRING, 8, PropModeReplace,
+   XChangeProperty(dpy, areastruct.areawin, gvc, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    XSync(dpy, False);
 }
@@ -113,7 +116,7 @@ void send_client(Atom msg)
 
    event.xclient.type = ClientMessage;
    event.xclient.display = dpy;
-   event.xclient.window = win;
+   event.xclient.window = areastruct.areawin;
    event.xclient.message_type = msg;
    event.xclient.format = 32;
    event.xclient.data.l[0] = mwin;
@@ -213,8 +216,8 @@ void start_gs()
 #endif
 
    if (bbuf != (Pixmap)NULL) XFreePixmap(dpy, bbuf);
-   bbuf = XCreatePixmap(dpy, win, areastruct.width, areastruct.height,
-      DefaultDepthOfScreen(XtScreen(areastruct.area))); 
+   bbuf = XCreatePixmap(dpy, areastruct.areawin, areastruct.width, areastruct.height,
+      DefaultDepthOfScreen(xcScreen(areastruct.area))); 
 
    ret = pipe(fgs);
    ret = pipe(std_out);
@@ -250,22 +253,24 @@ void start_gs()
    	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
 	 putenv(env_str1);
 #ifdef DOUBLEBUFFER
-   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)win, (long)bbuf);
+   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areastruct.areawin, (long)bbuf);
 	 putenv(env_str2);
 #else
-	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)win);
+	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areastruct.areawin);
 	 putenv(env_str2);
 #endif
 #else
 	 setenv("DISPLAY", XDisplayString(dpy), True);
 #ifdef DOUBLEBUFFER
-	 sprintf(_STR, "%ld %ld", (long)win, (long)bbuf);
+	 sprintf(_STR, "%ld %ld", (long)areastruct.areawin, (long)bbuf);
 	 setenv("GHOSTVIEW", _STR, True);
 #else
  	 sprintf(_STR, "%ld", (long)win);
 	 setenv("GHOSTVIEW", _STR, True);
 #endif
 #endif
+	 fprintf(stderr, "Did putenv \"%s\"\n", env_str2);
+	 fflush(stderr);
          execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", NULL);
 	 gsproc = -1;
 	 printf("Exec of gs failed\n");
@@ -315,10 +320,10 @@ void parse_bg(FILE *fi, FILE *fbg) {
 	       printf("BBox %d %d %d %d XCircuit coordinates\n", llx, lly, urx, ury);
 #endif
 	
-	       xobjs.pagelist[areastruct.page]->background.lowerleft.x = llx;
-	       xobjs.pagelist[areastruct.page]->background.lowerleft.y = lly;
-	       xobjs.pagelist[areastruct.page]->background.upperright.x = urx;
-	       xobjs.pagelist[areastruct.page]->background.upperright.y = ury;
+	       xobjs.pagelist[areastruct.page]->background.bbox.lowerleft.x = llx;
+	       xobjs.pagelist[areastruct.page]->background.bbox.lowerleft.y = lly;
+	       xobjs.pagelist[areastruct.page]->background.bbox.width = (urx - llx);
+	       xobjs.pagelist[areastruct.page]->background.bbox.height = (ury - lly);
 	       if (fbg == (FILE *)NULL) break;
 	    }
 	 }
@@ -351,55 +356,26 @@ void bg_get_bbox()
 
 void backgroundbbox(int mpage)
 {
-   short llx, lly, urx, ury;
-   objectptr thisobj = xobjs.pagelist[mpage]->pageobj;
+   int llx, lly, urx, ury, tmp;
+   objectptr thisobj = xobjs.pagelist[mpage]->pageinst->thisobject;
    psbkground *thisbg = &xobjs.pagelist[mpage]->background;
 
-#ifdef SCHEMA
-   llx = thisobj->lleft2.x;
-   lly = thisobj->lleft2.y;
-   urx = thisobj->width2 + llx;
-   ury = thisobj->height2 + lly;
-#else
-   llx = thisobj->lowerleft.x;
-   lly = thisobj->lowerleft.y;
-   urx = thisobj->width + llx;
-   ury = thisobj->height + lly;
-#endif
+   llx = thisobj->bbox.lowerleft.x;
+   lly = thisobj->bbox.lowerleft.y;
+   urx = thisobj->bbox.width + llx;
+   ury = thisobj->bbox.height + lly;
 
-   if (thisbg->lowerleft.x < llx) llx = thisbg->lowerleft.x;
-   if (thisbg->lowerleft.y < lly) lly = thisbg->lowerleft.y;
-   if (thisbg->upperright.x > urx) urx = thisbg->upperright.x;
-   if (thisbg->upperright.y > ury) ury = thisbg->upperright.y;
+   if (thisbg->bbox.lowerleft.x < llx) llx = thisbg->bbox.lowerleft.x;
+   if (thisbg->bbox.lowerleft.y < lly) lly = thisbg->bbox.lowerleft.y;
+   tmp = thisbg->bbox.width + thisbg->bbox.lowerleft.x;
+   if (tmp > urx) urx = tmp;
+   tmp = thisbg->bbox.height + thisbg->bbox.lowerleft.y;
+   if (tmp > ury) ury = tmp;
 
-#ifdef SCHEMA
-   thisobj->lleft2.x = llx;
-   thisobj->lleft2.y = lly;
-   thisobj->width2 = urx - llx;
-   thisobj->height2 = ury - lly;
-#else
-   thisobj->lowerleft.x = llx;
-   thisobj->lowerleft.y = lly;
-   thisobj->width = urx - llx;
-   thisobj->height = ury - lly;
-#endif
-
-#ifdef SCHEMA
-   llx = thisobj->lowerleft.x;
-   lly = thisobj->lowerleft.y;
-   urx = thisobj->width + llx;
-   ury = thisobj->height + lly;
-
-   if (thisbg->lowerleft.x < llx) llx = thisbg->lowerleft.x;
-   if (thisbg->lowerleft.y < lly) lly = thisbg->lowerleft.y;
-   if (thisbg->upperright.x > urx) urx = thisbg->upperright.x;
-   if (thisbg->upperright.y > ury) ury = thisbg->upperright.y;
-
-   thisobj->lowerleft.x = llx;
-   thisobj->lowerleft.y = lly;
-   thisobj->width = urx - llx;
-   thisobj->height = ury - lly;
-#endif
+   thisobj->bbox.lowerleft.x = llx;
+   thisobj->bbox.lowerleft.y = lly;
+   thisobj->bbox.width = urx - llx;
+   thisobj->bbox.height = ury - lly;
 }
 
 /*------------------------------------------------------*/
@@ -487,7 +463,7 @@ void loadbackground()
 {
    call_loadbg(_STR2);
    bg_get_bbox();
-   updatepagebounds(objectdata);
+   updatepagebounds(topobject);
    zoomview(NULL, NULL, NULL);
 }
 
@@ -518,8 +494,7 @@ int renderbackground()
    defscale = (xobjs.pagelist[areastruct.page]->coordstyle == CM) ?
 	CMSCALE : INCHSCALE;
 
-   psnorm = xobjs.pagelist[areastruct.page]->outscale * (*areastruct.vscale)
-	* (1.0 / defscale) * devres;
+   psnorm = (*areastruct.vscale) * (1.0 / defscale) * devres;
 
    psxpos = (float)(-areastruct.pcorner->x) * (*areastruct.vscale) * devres;
    psypos = (float)(-areastruct.pcorner->y) * (*areastruct.vscale) * devres
@@ -533,7 +508,7 @@ int renderbackground()
    else if (areastruct.lastbackground == xobjs.pagelist[areastruct.page]->background.name)
       return 0;
 
-   if (is_page(objectdata) == -1)
+   if (is_page(topobject) == -1)
       return -1;
 
    ask_for_next();
@@ -561,11 +536,11 @@ int renderbackground()
 
    /* ask ghostscript to produce the next page */
 
-   XDefineCursor(dpy, win, WAITFOR);
+   XDefineCursor(dpy, areastruct.areawin, WAITFOR);
    Wprintf("Rendering background image.");
    wait_for_page();
    Wprintf("Background finished.");
-   XDefineCursor(dpy, win, CROSS);
+   XDefineCursor(dpy, areastruct.areawin, CROSS);
 
    /* Mark this as the most recently rendered background, so we don't	*/
    /* have to render more than necessary.				*/
@@ -581,14 +556,14 @@ int renderbackground()
 int copybackground()
 {
    /* only draw on a top-level page */
-   if (is_page(objectdata) == -1)
+   if (is_page(topobject) == -1)
       return -1;
 
 #ifdef DOUBLEBUFFER
    XCopyArea(dpy, bbuf, dbuf, areastruct.gc, 0, 0,
              areastruct.width, areastruct.height, 0, 0);
 #else
-   XCopyArea(dpy, bbuf, win, areastruct.gc, 0, 0,
+   XCopyArea(dpy, bbuf, areastruct.areawin, areastruct.gc, 0, 0,
              areastruct.width, areastruct.height, 0, 0);
 #endif
 
