@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* schema.c --- xcircuit routines specific to the schematic capture system */
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	   */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -18,8 +18,13 @@
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#else
 #include "Xw/Xw.h"
 #include "Xw/MenuBtn.h"
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -42,22 +47,21 @@ extern Globaldata xobjs;
 extern Clientdata areastruct;
 extern short	  eventmode;
 extern int	  *appcolors;
-extern Widget     menuwidgets[];
-extern Widget	  netbutton;
-extern objectpair *pushlist;
+extern xcWidget     menuwidgets[];
+extern xcWidget	  netbutton;
 extern char	  _STR[150];
 extern char	  _STR2[250];
 
 /*-------------------------------------------------------------------------*/
 #ifdef SCHEMA
 
-extern Widget wsymb, wschema;
+extern xcWidget wsymb, wschema;
 
 /*--------------------------------------------------------*/
 /* Menu calls (procedure wrappers)			  */
 /*--------------------------------------------------------*/
 
-void callgennet(Widget w, u_int mode, caddr_t calldata)
+void callgennet(xcWidget w, pointertype mode, caddr_t calldata)
 {
    switch(mode) {
      case 0:
@@ -67,7 +71,7 @@ void callgennet(Widget w, u_int mode, caddr_t calldata)
         gennet("sim", "sim");
 	break;
      case 2:
-        gennet("pcb", "pcb");
+        gennet("pcb", "pcbnet");
 	break;
      case 3:
         gennet("flatspice", "fspc");
@@ -78,37 +82,43 @@ void callgennet(Widget w, u_int mode, caddr_t calldata)
    }
 }
 
-/*------------------------------------------------------------*/
-/* Enable schematic capture (set schemon and realize widgets) */
-/*------------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Enable schematic capture (set schemon and realize widgets)	*/
+/* If mode == 1, do a redraw (e.g., not called during a file	*/
+/* load)							*/
+/*--------------------------------------------------------------*/
 
-void doxschema(Widget w, caddr_t clientdata, caddr_t calldata)
+void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
 {
    Arg wargs[1];
 
    if (areastruct.schemon) {
       areastruct.schemon = False;
+#ifndef TCL_WRAPPER
       XtUnmanageChild(wschema);
       XtUnmanageChild(wsymb);
       XtUnmanageChild(netbutton);
 
       XtSetArg(wargs[0], XtNlabel, "Enable XSchema");
       XtSetValues(OptionsEnableXSchemaButton, wargs, 1);
+#endif
    }
    else {
       areastruct.schemon = True;
+#ifndef TCL_WRAPPER
       XtManageChild(wschema);
       XtManageChild(wsymb);
       XtManageChild(netbutton);
 
       XtSetArg(wargs[0], XtNlabel, "Disable XSchema");
       XtSetValues(OptionsEnableXSchemaButton, wargs, 1);
+#endif
    }
 
    /* Redraw the screen, since this action may effect the presence of	*/
    /* pins and schematic informational labels.				*/
 
-   drawarea(NULL, NULL, NULL);
+   if (mode) drawarea(NULL, NULL, NULL);
 }
 
 /*------------------------------------------------------------------------*/
@@ -132,8 +142,8 @@ int checkpagename(objectptr thispageobj)
 
    /* Find the page number of this page object */
    for (p = 0; p < xobjs.pages; p++) {
-      if (xobjs.pagelist[p]->pageobj != NULL) {
-	 if (xobjs.pagelist[p]->pageobj == thispageobj) {
+      if (xobjs.pagelist[p]->pageinst != NULL) {
+	 if (xobjs.pagelist[p]->pageinst->thisobject == thispageobj) {
 	    thispage = p;
 	    break;
 	 }
@@ -149,8 +159,9 @@ int checkpagename(objectptr thispageobj)
       changed = False;
       for (p = 0; p < xobjs.pages; p++) {
 	 if (p == thispage) continue;
-         if (xobjs.pagelist[p]->pageobj != NULL) {
-	    if (!strcmp(xobjs.pagelist[p]->pageobj->name, thispageobj->name)) {
+         if (xobjs.pagelist[p]->pageinst != NULL) {
+	    if (!strcmp(xobjs.pagelist[p]->pageinst->thisobject->name,
+			thispageobj->name)) {
 	       /* append ":2" to name or update suffix to ensure uniqueness */
 	       if (clnptr == NULL)
 		  sprintf(thispageobj->name, "%s:2", thispageobj->name);
@@ -174,7 +185,7 @@ int checkpagename(objectptr thispageobj)
 /* Find connectivity of a selection	*/
 /*--------------------------------------*/
 
-void startconnect(Widget button, caddr_t clientdata, caddr_t calldata)
+void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    if (areastruct.selects > 0)
       connectivity(button, clientdata, calldata);
@@ -189,27 +200,29 @@ void startconnect(Widget button, caddr_t clientdata, caddr_t calldata)
 /* (does recursive search on object heirarchy)				*/
 /*----------------------------------------------------------------------*/
 
-void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
+void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    short *gsel;
    genericptr ggen;
    int depth, netid;
-   objectpair *seltop, *nextptr;
+   pushlistptr seltop, nextptr;
    objectptr nettop, thisobj;
    char *snew;
    stringpart *ppin;
 
-   seltop = (objectpair *)malloc(sizeof(objectpair));
-   seltop->thisobject = objectdata;
-   seltop->thisinst = NULL;
-   seltop->nextpair = NULL;
+   /* erase any existing highlighted network */
+   highlightnet(topobject, areastruct.topinstance, -1, 0);
+
+   seltop = (pushlistptr)malloc(sizeof(pushlist));
+   seltop->thisinst = areastruct.topinstance;
+   seltop->next = NULL;
 
    /* pick the first selection that looks like a valid network part */
 
    if (areastruct.selects > 0) {
       for (gsel = areastruct.selectlist; gsel < areastruct.selectlist +
 		areastruct.selects; gsel++) {
-	 ggen = *(objectdata->plist + *gsel);
+	 ggen = *(topobject->plist + *gsel);
          if (SELECTTYPE(gsel) == LABEL) {
 	    labelptr glab = SELTOLABEL(gsel);
 	    if (glab->pin == LOCAL || glab->pin == GLOBAL) break;
@@ -224,7 +237,7 @@ void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
       }
    }
    else
-      ggen = recurselect(&seltop);
+      ggen = recurselect(POLYGON | LABEL, &seltop);
 
    /* Determine the netid and the topmost object in which that	*/
    /* net appears.  Then build the transformation matrix down	*/
@@ -232,15 +245,22 @@ void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
    /* entire matrix stack.					*/
 
    if (ggen != NULL) {
-      createnets(True, NULL);
+      if (checkvalid(topobject) == -1) {
+         destroynets();
+         createnets();
+      }
       if ((netid = is_resolved(&ggen, seltop, &nettop)) != 0) {
          /* printf("Net ID is %d in object %s\n", netid, nettop->name); */
          depth = pushnetwork(seltop, nettop);
          /* printf(">> Pushed network %d levels deep\n", depth); */
          nextptr = seltop;
-         while (nextptr->thisobject != nettop)
-	    nextptr = nextptr->nextpair;
-         highlightnet(nettop, nextptr->thisinst, netid);
+         while (nextptr->thisinst->thisobject != nettop)
+	    nextptr = nextptr->next;
+
+	 nextptr->thisinst->thisobject->highlight.netid = netid;
+	 nextptr->thisinst->thisobject->highlight.thisinst = nextptr->thisinst;
+	    
+         highlightnet(nettop, nextptr->thisinst, netid, 1);
 
          /* pop the matrix stack */
          while (depth-- > 0) 
@@ -249,14 +269,13 @@ void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
 	 /* print the net name to the message window */
 
 	 ppin = nettopin(netid, nettop, NULL);
-         snew = textprint(ppin, NORMINST);
+         snew = textprint(ppin, areastruct.topinstance);
 	 sprintf(_STR, "Network is \"%s\" in %s", snew, nettop->name);
 	 Wprintf(_STR);
          free(snew);
       }
       else
 	 Wprintf("Selected element is not part of a valid network.");
-      destroynets();
    }
    else 
       Wprintf("No networks found near the cursor position");
@@ -264,7 +283,7 @@ void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
    /* free up linked list */
 
    while (seltop != NULL) {
-      nextptr = seltop->nextpair;
+      nextptr = seltop->next;
       free(seltop);
       seltop = nextptr;
    }
@@ -272,29 +291,27 @@ void connectivity(Widget button, caddr_t clientdata, caddr_t calldata)
 
 /*--------------------------------------------------------------*/
 /* Set object type to FUNDAMENTAL if it contains one or more	*/
-/* info labels for the given netlist mode.			*/
+/* info labels and is not associated with a schematic/symbol.	*/
 /*								*/
-/* Return TRUE if the object has eletrically relevant networks,	*/
-/* FALSE if not.						*/
+/* Return TRUE if the object has electrically relevant		*/
+/* networks, FALSE if not.					*/
 /*--------------------------------------------------------------*/
 
-Boolean setobjecttype(objectptr cschem, char *mode)
+Boolean setobjecttype(objectptr cschem)
 {
    genericptr *cgen;
    labelptr clab;
 
    /* Apply only to schematic objects */
-   if (cschem->schemtype != SCHEMATIC && cschem->symschem == NULL) {
 
+   if (cschem->schemtype != SCHEMATIC && cschem->symschem == NULL) {
       if (cschem->schemtype == FUNDAMENTAL) cschem->schemtype = SYMBOL;
       for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
          if ((*cgen)->type == LABEL) {
             clab = TOLABEL(cgen);
 	    if (clab->pin == INFO) {
-	       if ((mode == NULL) || !textncomp(clab->string, mode, NULL)) {
-	          cschem->schemtype = FUNDAMENTAL;
-	          break;
-	       }
+	       cschem->schemtype = FUNDAMENTAL;
+	       break;
 	    }
          }
       }
@@ -312,7 +329,7 @@ Boolean setobjecttype(objectptr cschem, char *mode)
 /* Pin conversion subroutine for dopintype()		*/
 /*------------------------------------------------------*/
 
-void pinconvert(labelptr thislab, u_int mode)
+void pinconvert(labelptr thislab, pointertype mode)
 {
    thislab->pin = mode;
    switch (mode) {
@@ -320,13 +337,13 @@ void pinconvert(labelptr thislab, u_int mode)
 	 thislab->color = DEFAULTCOLOR;		/* nominally black */
 	 break;
       case GLOBAL:
-	 thislab->color = BBOXCOLOR;		/* orange */
+	 thislab->color = GLOBALPINCOLOR;	/* orange */
          break;
       case LOCAL:
-	 thislab->color = SNAPCOLOR;		/* red */
+	 thislab->color = LOCALPINCOLOR;	/* red */
          break;
       case INFO:
-	 thislab->color = AUXCOLOR;		/* green */
+	 thislab->color = INFOLABELCOLOR;	/* green */
          break;
    }
 }
@@ -335,7 +352,7 @@ void pinconvert(labelptr thislab, u_int mode)
 /* Change a label's type to NORMAL, GLOBAL, INFO, or LOCAL */
 /*---------------------------------------------------------*/
 
-void dopintype(Widget w, u_int mode, caddr_t calldata)
+void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
 {
    short *gsel;
    char typestr[40];
@@ -368,7 +385,7 @@ void dopintype(Widget w, u_int mode, caddr_t calldata)
 	 labelptr glab = SELTOLABEL(gsel);
 	 savetype = glab->pin;
 	 pinconvert(glab, mode);
-	 setobjecttype(objectdata, NULL);
+	 setobjecttype(topobject);
       }
 
    if (savetype >= 0) {
@@ -385,14 +402,23 @@ void dopintype(Widget w, u_int mode, caddr_t calldata)
 /* Set colors on the symbol/schematic buttons appropriately */
 /*----------------------------------------------------------*/
 
+#ifdef TCL_WRAPPER
+
+void setsymschem()
+{
+   /* to be done! */
+}
+
+#else
+
 void setsymschem()
 {
    Arg aargs[2], bargs[2];
 
    /* Set menu items appropriately for this object */
 
-   if (objectdata->symschem != NULL) {
-      if (objectdata->schemtype == SCHEMATIC) {
+   if (topobject->symschem != NULL) {
+      if (topobject->schemtype == SCHEMATIC) {
          XtSetArg(aargs[0], XtNlabel, "Go To Symbol");
          XtSetArg(bargs[0], XtNlabel, "Disassociate Symbol");
       }
@@ -402,7 +428,7 @@ void setsymschem()
       }
    }
    else {
-      if (objectdata->schemtype == SCHEMATIC) {
+      if (topobject->schemtype == SCHEMATIC) {
          XtSetArg(aargs[0], XtNlabel, "Make Matching Symbol");
          XtSetArg(bargs[0], XtNlabel, "Associate with Symbol");
       }
@@ -416,8 +442,8 @@ void setsymschem()
 
    /* Set colors on the symbol and schematic buttons */
 
-   if (objectdata->schemtype == SCHEMATIC) {
-      if (objectdata->symschem == NULL) {
+   if (topobject->schemtype == SCHEMATIC) {
+      if (topobject->symschem == NULL) {
          XtSetArg(aargs[0], XtNbackground, OFFBUTTONCOLOR);
 	 XtSetArg(aargs[1], XtNforeground, OFFBUTTONCOLOR);
       }
@@ -430,7 +456,7 @@ void setsymschem()
       XtSetArg(bargs[0], XtNbackground, SNAPCOLOR);
    }
    else {
-      if (objectdata->symschem != NULL) {
+      if (topobject->symschem != NULL) {
          XtSetArg(bargs[0], XtNbackground, BACKGROUND);
 	 XtSetArg(bargs[1], XtNforeground, FOREGROUND);
       }
@@ -440,9 +466,9 @@ void setsymschem()
       }
 
       XtSetArg(aargs[1], XtNforeground, FOREGROUND);
-      if (objectdata->schemtype == FUNDAMENTAL)
+      if (topobject->schemtype == FUNDAMENTAL)
          XtSetArg(aargs[0], XtNbackground, AUXCOLOR);
-      else if (objectdata->schemtype == TRIVIAL || objectdata->symschem != NULL)
+      else if (topobject->schemtype == TRIVIAL || topobject->symschem != NULL)
          XtSetArg(aargs[0], XtNbackground, SNAPCOLOR);
       else {
          XtSetArg(aargs[0], XtNbackground, OFFBUTTONCOLOR);
@@ -456,6 +482,8 @@ void setsymschem()
    XtSetValues(wschema, bargs, 2);
 }
 
+#endif
+
 /*--------------------------------------------------------*/
 /* Find the page number for an object			  */
 /*--------------------------------------------------------*/
@@ -464,9 +492,11 @@ int findpageobj(objectptr pobj)
 {
    int tpage;
 
-   for (tpage = 0; tpage < xobjs.pages; tpage++) {
-      if (xobjs.pagelist[tpage]->pageobj == pobj) return tpage;
-   }
+   for (tpage = 0; tpage < xobjs.pages; tpage++)
+      if (xobjs.pagelist[tpage]->pageinst != NULL)
+         if (xobjs.pagelist[tpage]->pageinst->thisobject == pobj)
+	    return tpage;
+
    return -1;
 }
 
@@ -532,7 +562,7 @@ void copypinlabel(labelptr pinlab)
 {
    labelptr *newlabel, tlabel;
    genericptr *tgen;
-   objectptr schemobj = objectdata->symschem;
+   objectptr schemobj = topobject->symschem;
 
    if (areastruct.schemon && schemobj != NULL && (pinlab->pin == LOCAL)) {
 
@@ -558,12 +588,13 @@ void copypinlabel(labelptr pinlab)
 
       /* place label just outside bounding box, then recompute bbox */
 
-      (*newlabel)->position.x = schemobj->lowerleft.x + (schemobj->width >> 1);
-      (*newlabel)->position.y = schemobj->lowerleft.y - TEXTHEIGHT
+      (*newlabel)->position.x = schemobj->bbox.lowerleft.x + (schemobj->bbox.width >> 1);
+      (*newlabel)->position.y = schemobj->bbox.lowerleft.y - TEXTHEIGHT
 	   * (*newlabel)->scale;
       schemobj->parts++;
       incr_changes(schemobj);
-      singlebbox(objectdata, ENDPART);
+      schemobj->valid = False;
+      singlebbox(ENDPART);
    }
 }
 
@@ -613,8 +644,8 @@ int checksym(objectptr symobj, char *cname)
    if (areastruct.schemon == False || symobj->symschem != NULL) return 0;
 
    for (cpage = 0; cpage < xobjs.pages; cpage++) {
-      checkpage = xobjs.pagelist[cpage]->pageobj;
-      if (checkpage != NULL) {
+      if (xobjs.pagelist[cpage]->pageinst != NULL) {
+         checkpage = xobjs.pagelist[cpage]->pageinst->thisobject;
          if (!strcmp(checkpage->name, cname)) {
 	    symobj->symschem = checkpage;
 	    symobj->schemtype = SYMBOL;
@@ -635,7 +666,7 @@ int checksym(objectptr symobj, char *cname)
 
 void changeotherpins(labelptr newlabel, stringpart *oldstring)
 {
-   objectptr other = objectdata->symschem;
+   objectptr other = topobject->symschem;
    genericptr *tgen;
    labelptr tlab;
 
@@ -662,12 +693,13 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
 /*  there is no corresponding schematic/symbol, nothing happens.	*/
 /*----------------------------------------------------------------------*/
 
-void swapschem(Widget w, u_int mode, caddr_t calldata)
+void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 {
-   objectptr savepage = objectdata;
+   objectptr savepage = topobject;
    labelptr  *pinlab;
    genericptr *plab;
    Boolean lflag;
+   pushlistptr stacktop;
 
    if (areastruct.schemon == False) return;
 
@@ -676,15 +708,23 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
 
    /* Create symbol or schematic, if allowed by mode */
 
-   if ((objectdata->symschem == NULL) && (mode != 0)) {
-      if (objectdata->schemtype != SCHEMATIC) {
+   if ((topobject->symschem == NULL) && (mode != 0)) {
+      if (topobject->schemtype != SCHEMATIC) {
 	 int tpage;
 
 	 /* create a new page for the new schematic */
 
 	 for (tpage = 0; tpage < xobjs.pages; tpage++)
-	    if (xobjs.pagelist[tpage]->pageobj == NULL) break;
+	    if (xobjs.pagelist[tpage]->pageinst == NULL) break;
+
+	 /* Push the current instance onto the push stack */
+	 /* Change the page without destroying the pushlist */
+
+	 push_stack(&xobjs.stack, areastruct.topinstance);
+	 stacktop = xobjs.stack;
+	 xobjs.stack = NULL;
          changepage(tpage);
+	 xobjs.stack = stacktop;
       }
       else {
 	 objectptr *newobject;
@@ -702,20 +742,27 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
 	 (*newobject)->schemtype = SYMBOL;
 	 (*newobject)->hidden = False;
 	 (*newobject)->devname = NULL;
-	 objectdata = *newobject;
+
+	 incr_changes(*newobject);
+
+	 /* Generate a library instance for this object and set the */
+	 /* top instance to point to it.			    */
+
+	 push_stack(&xobjs.stack, areastruct.topinstance);
+	 areastruct.topinstance = addtoinstlist(libnum, *newobject, FALSE);
 
 	 /* Generate the default bounding box for a size-zero object */
-	 calcbbox(*newobject);
+	 calcbbox(areastruct.topinstance);
       }
 
       /* set links between the two objects */
 
-      savepage->symschem = objectdata;
-      objectdata->symschem = savepage;
+      savepage->symschem = topobject;
+      topobject->symschem = savepage;
 
       /* make the name of the new object equal to that of the old */
 
-      strcpy(objectdata->name, savepage->name);
+      strcpy(topobject->name, savepage->name);
 
       /* copy all pin labels into the new object */
 
@@ -730,8 +777,8 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
       	       /* Only make one copy of each pin name */
 
 	       lflag = False;
-               for (tgen = objectdata->plist; tgen <
-			objectdata->plist + objectdata->parts; tgen++) {
+               for (tgen = topobject->plist; tgen <
+			topobject->plist + topobject->parts; tgen++) {
 		  if ((*tgen)->type == LABEL) {
 		     tlab = TOLABEL(tgen);
 	             if (!stringcomp(tlab->string, lpin->string)) lflag = True;
@@ -739,36 +786,81 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
       	       }
 	       if (lflag == True) continue;
 
-	       NEW_LABEL(pinlab, objectdata);
+	       NEW_LABEL(pinlab, topobject);
 	       (*pinlab)->pin = lpin->pin;
 	       (*pinlab)->color = lpin->color;
 	       (*pinlab)->rotation = 0;
 	       (*pinlab)->scale = 1.0;
 	       (*pinlab)->justify = areastruct.justify; 
 	       (*pinlab)->position.x = 0;
-	       (*pinlab)->position.y = objectdata->parts * (TEXTHEIGHT + 10);
+	       (*pinlab)->position.y = topobject->parts * (TEXTHEIGHT + 10);
 	       (*pinlab)->num_params = 0;
 	       (*pinlab)->passed = NULL;
 	       u2u_snap(&((*pinlab)->position));
 	       (*pinlab)->string = stringcopy(lpin->string);
-	       objectdata->parts++;
-	       incr_changes(objectdata);
+	       topobject->parts++;
+	       incr_changes(topobject);
 	    }
          }
-	 calcbbox(objectdata);
       }
+      calcbbox(areastruct.topinstance);
 
       /* Recreate the user library with the new symbol */
       if (savepage->schemtype != SYMBOL) composelib(USERLIB);
    }
-   else if (objectdata->symschem != NULL) {
-      objectdata = objectdata->symschem;
+   else if (topobject->symschem != NULL) {
+
+      /* If symschem matches the last entry on the push stack, then we	*/
+      /* pop; otherwise, we push.					*/
+
+      if (xobjs.stack && xobjs.stack->thisinst->thisobject == topobject->symschem) {
+	 areastruct.topinstance = xobjs.stack->thisinst;
+	 pop_stack(&xobjs.stack);
+      }
+      else {
+	 int p;
+	 objinstptr syminst = NULL;
+	 liblistptr symlist;
+
+	 /* If symschem is a schematic, find the appropriate page */
+
+	 for (p = 0; p < xobjs.pages; p++) {
+	    syminst = xobjs.pagelist[p]->pageinst;
+	    if (syminst != NULL)
+	       if (syminst->thisobject == topobject->symschem)
+		  break;
+	 }
+	 if (p == xobjs.pages) {
+
+	    /* If symschem is a symbol, and it wasn't on the push stack, */
+	    /* get the library default symbol and go there.		 */
+
+	    for (p = 0; p < xobjs.numlibs; p++) {
+	       for (symlist = xobjs.userlibs[p].instlist; symlist != NULL;
+			symlist = symlist->next) {
+	          syminst = symlist->thisinst;
+	          if (syminst->thisobject == topobject->symschem &&
+			symlist->virtual == FALSE)
+		     break;
+	       }
+	       if (symlist != NULL) break;
+	    }
+	    if (p == xobjs.numlibs) {
+	       fprintf(stderr, "swapschem(): BAD SYMSCHEM\n");
+	       return;
+	    }
+         }
+	       
+	 push_stack(&xobjs.stack, areastruct.topinstance);
+	 areastruct.topinstance = syminst;
+      }
    }
 
-   if (areastruct.selects > 0) free(areastruct.selectlist);
-   areastruct.selects = 0;
+   /* If there was no action, then there is nothing more to do. */
 
-   setpage();
+   if (topobject == savepage) return;
+
+   setpage(TRUE);
    transferselects();
    refresh(NULL, NULL, NULL);
 
@@ -779,13 +871,13 @@ void swapschem(Widget w, u_int mode, caddr_t calldata)
 /* Wrapper for swapschem() when generating a new symbol.		*/
 /*----------------------------------------------------------------------*/
 
-void makesymbol(Widget w, caddr_t calldata)
+void makesymbol(xcWidget w, caddr_t calldata)
 {
    /* copy name from popup prompt buffer and check */
 
-   strcpy(objectdata->name, _STR2);
-   checkname(objectdata);
-   swapschem(w, (u_int)1, calldata);
+   strcpy(topobject->name, _STR2);
+   checkname(topobject);
+   swapschem(w, (pointertype)1, calldata);
 }
 
 /*----------------------------------------------------------------------*/
@@ -793,7 +885,7 @@ void makesymbol(Widget w, caddr_t calldata)
 /* for the object name, as you would when doing selectsave().		*/ 
 /*----------------------------------------------------------------------*/
 
-void dobeforeswap(Widget w, caddr_t clientdata, caddr_t calldata)
+void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    buttonsave *popdata = (buttonsave *)malloc(sizeof(buttonsave));
 
@@ -803,8 +895,8 @@ void dobeforeswap(Widget w, caddr_t clientdata, caddr_t calldata)
 
    /* Check for requirement to change the name before creating the symbol */
 
-   if ((objectdata->symschem == NULL) && (objectdata->schemtype == SCHEMATIC)
-		&& (strstr(objectdata->name, "Page ") != NULL)) {
+   if ((topobject->symschem == NULL) && (topobject->schemtype == SCHEMATIC)
+		&& (strstr(topobject->name, "Page ") != NULL)) {
 
       /* Get a name for the new object */
 
@@ -814,7 +906,7 @@ void dobeforeswap(Widget w, caddr_t clientdata, caddr_t calldata)
       popupprompt(w, "Enter name for new object:", "\0", makesymbol, popdata, NULL);
    }
    else
-      swapschem(w, (u_int)1, calldata);
+      swapschem(w, (pointertype)1, calldata);
 }
 
 /*------------------------------------------*/
@@ -827,8 +919,8 @@ void schemdisassoc()
       Wprintf("Cannot disassociate schematics in this mode");
    }
    else {
-      objectdata->symschem->symschem = NULL;
-      objectdata->symschem = NULL;
+      topobject->symschem->symschem = NULL;
+      topobject->symschem = NULL;
       setsymschem();
       Wprintf("Schematic and symbol are now unlinked.");
    }
@@ -839,13 +931,13 @@ void schemdisassoc()
 /* context (associate/disassociate, symbol or schematic)	*/
 /*--------------------------------------------------------------*/
 
-void startschemassoc(Widget w, caddr_t clientdata, caddr_t calldata)
+void startschemassoc(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
-   if (objectdata->symschem != NULL)
+   if (topobject->symschem != NULL)
       schemdisassoc();
    else {
       eventmode = ASSOC_MODE;
-      if (objectdata->schemtype == SCHEMATIC) {
+      if (topobject->schemtype == SCHEMATIC) {
 	 /* Find a symbol to associate */
 	 startcatalog(w, LIBLIB, NULL);
 	 Wprintf("Click on library page, then symbol to associate.");
