@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------*/
 /* selection.c --- xcircuit routines handling element selection etc.	   */
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University       	   */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University       	   */
 /*-------------------------------------------------------------------------*/
 
 /*-------------------------------------------------------------------------*/
@@ -19,7 +19,12 @@
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#else
 #include "Xw/Xw.h"
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -39,22 +44,20 @@
 
 extern short eventmode;	/* keep track of the mode the screen is in */
 extern Display	*dpy;   /* Works well to make this globally accessible */
-extern Window win;
 extern int *appcolors;
 extern Cursor	appcursors[NUM_CURSORS];
 extern Clientdata areastruct;
 extern char _STR[150];
 extern short attachto;
 extern short textpos, textend;
-extern objectpair *pushlist;
 
 /*----------------------------------------------------------------------*/
 /* Change filter to determine what types can be selected		*/
 /*----------------------------------------------------------------------*/
 
-void selectfilter(Widget w, caddr_t value, caddr_t calldata)
+void selectfilter(xcWidget w, pointertype value, caddr_t calldata)
 {
-   short bitwise = (int) value & 0xffff;
+   short bitwise = (short)value;
    Boolean bval = (areastruct.filter & bitwise) ? True : False;
 
    if (bval)
@@ -84,6 +87,23 @@ Boolean checkselect(short value)
    else return True;
 }
 
+/*-----------------------------------------------------------*/
+/* Select list numbering revision for routine objectdelete() */
+/*-----------------------------------------------------------*/
+
+void reviseselect(short *selectobj)
+{
+   short *chkselect;
+
+   for (chkselect = selectobj + 1; chkselect < areastruct.selectlist
+	  + areastruct.selects; chkselect++)
+      if (*chkselect > *selectobj) (*chkselect)--;
+
+   for (chkselect = areastruct.selectlast; chkselect < areastruct.selectlast
+	  + areastruct.lastselects; chkselect++)
+      if (*chkselect > *selectobj) (*chkselect)--;
+}
+
 /*----------------------*/
 /* Draw a selected item */
 /*----------------------*/
@@ -94,23 +114,22 @@ void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curins
 
    switch ((*(curobj->plist + instance))->type) {
       case ARC:
-         UDrawArc(curinst, (arcptr)elementptr);
+         UDrawArc((arcptr)elementptr);
 	 break;
       case POLYGON:
-         UDrawPolygon(curinst, (polyptr)elementptr);
+         UDrawPolygon((polyptr)elementptr);
 	 break;
       case SPLINE:
-	 UDrawSpline(curinst, (splineptr)elementptr);
+	 UDrawSpline((splineptr)elementptr);
 	 break;
       case PATH:
-	 UDrawPath(curinst, (pathptr)elementptr);
+	 UDrawPath((pathptr)elementptr);
 	 break;
       case LABEL:
-         UDrawString(curinst, (labelptr)elementptr, mode);
+         UDrawString((labelptr)elementptr, mode, curinst);
 	 break;
       case OBJECT:
-         UDrawObject((objinstptr)elementptr, 
-	    ((objinstptr)elementptr)->thisobject, SINGLE, mode);
+         UDrawObject((objinstptr)elementptr, SINGLE, mode, NULL);
          break;
    }
 }
@@ -130,7 +149,9 @@ void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
    XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
-   XSetForeground(dpy, areastruct.gc, PARAMCOLOR ^ BACKGROUND);
+   /* XSetForeground(dpy, areastruct.gc, PARAMCOLOR ^ BACKGROUND); */
+   XSetFunction(dpy, areastruct.gc, GXcopy);
+   XSetForeground(dpy, areastruct.gc, PARAMCOLOR);
    indicateparams(*(curobj->plist + *newselect));
 }
 
@@ -334,7 +355,7 @@ short *selectobject(short class, objectptr selobj, objinstptr selinst)
    Boolean	selected, unselect = False;
    short	found = 0;
 
-   XDefineCursor(dpy, win, QUESTION);
+   XDefineCursor(dpy, areastruct.areawin, QUESTION);
 
    /* Definition for unselecting an element */
 
@@ -374,9 +395,15 @@ short *selectobject(short class, objectptr selobj, objinstptr selinst)
       }
 
       else if ((*curgen)->type == (class & LABEL)) {
+
+         /* Look among the labels */
+
 	 labelptr curlab = TOLABEL(curgen);
 
-         /* Now look among the labels */
+#ifdef SCHEMA
+	 /* Don't select temporary labels from schematic capture system */
+	 if (curlab->string->type != FONT_NAME) continue;
+#endif
 
 	 labelbbox(curlab, newboxpts, selinst);
 
@@ -503,7 +530,7 @@ short *selectobject(short class, objectptr selobj, objinstptr selinst)
    if (found > 1) {
       short *nselect;
 
-      XDefineCursor(dpy, win, CIRCLE);
+      XDefineCursor(dpy, areastruct.areawin, CIRCLE);
       if (eventmode == PRESS_MODE)
 	 Wprintf("Click and hold button 1 to continue move.");
       else
@@ -538,9 +565,9 @@ short *selectobject(short class, objectptr selobj, objinstptr selinst)
    setoptionmenu();
 
    if (eventmode == DELETE_MODE)
-      XDefineCursor(dpy, win, SCISSORS);
+      XDefineCursor(dpy, areastruct.areawin, SCISSORS);
    else
-      XDefineCursor(dpy, win, CROSS);
+      XDefineCursor(dpy, areastruct.areawin, CROSS);
 
    /* snap the coordinates */
 
@@ -627,8 +654,8 @@ void selectarea()
    }
 
    textpos = textend = 0;
-   for (curgen = objectdata->plist; curgen < objectdata->plist
-	+ objectdata->parts; curgen++) {
+   for (curgen = topobject->plist; curgen < topobject->plist
+	+ topobject->parts; curgen++) {
 
       /* apply the selection filter */
       if (!((*curgen)->type & areastruct.filter)) continue;
@@ -650,6 +677,11 @@ void selectarea()
 
 	   selected = False;
 
+#ifdef SCHEMA
+	   /* Ignore temporary labels created by the netlist generator */
+	   if (slab->string->type != FONT_NAME) break;
+#endif
+
 	   /* create a 4-point polygon out of the select box information */
 	   bboxpts[0].x = bboxpts[3].x = areastruct.origin.x;
 	   bboxpts[0].y = bboxpts[1].y = areastruct.origin.y;
@@ -668,7 +700,7 @@ void selectarea()
 			&(newboxpts[j].y), -1);
 	   }
 #endif
-	   tmpext = ULength(slab->string, NORMINST, 0.0, 0, NULL);
+	   tmpext = ULength(slab->string, areastruct.topinstance, 0.0, 0, NULL);
 	   adj.x = (slab->justify & NOTLEFT ? (slab->justify & RIGHT ? 
 			tmpext.width : tmpext.width >> 1) : 0);
 	   adj.y = (slab->justify & NOTBOTTOM ? (slab->justify & TOP ? 
@@ -679,12 +711,12 @@ void selectarea()
 	   /* do an insideness test with the select box.		*/
 
 	   state = tmpl2 = 0;
-	   for (j = 0; j < stringlength(slab->string, True, NORMINST); j++) {
-	      strptr = findstringpart(j, &locpos, slab->string, NORMINST);
+	   for (j = 0; j < stringlength(slab->string, True, areastruct.topinstance); j++) {
+	      strptr = findstringpart(j, &locpos, slab->string, areastruct.topinstance);
 	      if (locpos < 0) continue;	  /* only look at printable characters */
 	      if (strptr->type == RETURN) tmpl2 = 0;
 	      tmpl1 = tmpl2;
-	      tmpext = ULength(slab->string, NORMINST, 0.0, j + 1, NULL);
+	      tmpext = ULength(slab->string, areastruct.topinstance, 0.0, j + 1, NULL);
 	      tmpl2 = tmpext.width;
 	      isect = test_insideness(((tmpl1 + tmpl2) >> 1) - adj.x,
 			(tmpext.base + (BASELINE >> 1)) - adj.y, newboxpts);
@@ -730,14 +762,14 @@ void selectarea()
       if (selected)
          for (newselect = areastruct.selectlist; newselect <
               areastruct.selectlist + areastruct.selects; newselect++)
-            if (*newselect == (short)(curgen - objectdata->plist))
+            if (*newselect == (short)(curgen - topobject->plist))
                   selected = False;
 
       /* add to list of selections */
 
       if (selected) {
          newselect = allocselect();
-         *newselect = (short)(curgen - objectdata->plist);
+         *newselect = (short)(curgen - topobject->plist);
       }
    }
    setoptionmenu();
@@ -757,7 +789,7 @@ void selectarea()
 /* start deselection mode */
 /*------------------------*/
 
-void startdesel(Widget w, caddr_t clientdata, caddr_t calldata)
+void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
       if (areastruct.selects == 0)
@@ -792,6 +824,26 @@ void drawselects(objectptr thisobj, objinstptr thisinst)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Free memory from the previous selection list, copy the	*/
+/* current selection list to the previous selection list, and	*/
+/* zero out the current selection list.				*/
+/* Normally one would use clearselects();  use freeselects()	*/
+/* only if the menu/toolbars are going to be updated later in	*/
+/* the call.							*/
+/*--------------------------------------------------------------*/
+
+void freeselects()
+{
+   if (areastruct.selects > 0) {
+      if (areastruct.lastselects > 0)
+         free(areastruct.selectlast);
+      areastruct.selectlast = areastruct.selectlist;
+      areastruct.lastselects = areastruct.selects;
+      areastruct.selects = 0;
+   }
+}
+
 /*--------------------------------------------------------------*/
 /* Free memory from the selection list and set menu/toolbar	*/
 /* items back to default values.				*/
@@ -799,11 +851,26 @@ void drawselects(objectptr thisobj, objinstptr thisinst)
 
 void clearselects()
 {
-   if (areastruct.selects == 0) return;
+   if (areastruct.selects > 0) {
+      freeselects();
+      setallstylemarks(areastruct.style);
+      setcolormark(areastruct.color);
+      setdefaultfontmarks();
+   }
+}
 
+/*--------------------------------------------------------------*/
+/* Free memory from both the current and previous selection	*/
+/* lists, because a page change has invalidated both.		*/
+/*--------------------------------------------------------------*/
+
+void destroyselects()
+{
+   if (areastruct.selects > 0) free(areastruct.selectlist);
+   if (areastruct.lastselects > 0) free(areastruct.selectlast);
    areastruct.selects = 0;
-   free(areastruct.selectlist);
-
+   areastruct.lastselects = 0;
+ 
    setallstylemarks(areastruct.style);
    setcolormark(areastruct.color);
    setdefaultfontmarks();
@@ -816,20 +883,20 @@ void clearselects()
 
 void objectdeselect()
 {
-   drawselects(objectdata, NORMINST);
+   drawselects(topobject, areastruct.topinstance);
    clearselects();
 }
 
 /*----------------------------------------------------------------------*/
 /* Select the nearest polygon, searching the hierarchy if necessary.	*/
-/* Return an objectpair pointer to a linked list containing the 	*/
-/* hierarchy of objects, with the topmost objectpair also containing a	*/
+/* Return an pushlist pointer to a linked list containing the 		*/
+/* hierarchy of objects, with the topmost pushlist also containing a	*/
 /* pointer to the polygon found.					*/
 /* Allocates memory for the returned linked list which must be freed by	*/
 /* the calling routine.							*/
 /*----------------------------------------------------------------------*/
 
-genericptr recurselect(objectpair **seltop)
+genericptr recurselect(short class, pushlistptr *seltop)
 {
    genericptr rgen = NULL;
    short *selectobj;
@@ -838,14 +905,14 @@ genericptr recurselect(objectpair **seltop)
    XPoint savesave, tmppt;
 
    if (*seltop == NULL) {
-      fprintf(stderr, "Error: recurselect called with NULL objectpair pointer\n");
+      fprintf(stderr, "Error: recurselect called with NULL pushlist pointer\n");
       return NULL;
    }
 
-   selobj = (*seltop)->thisobject;
    selinst = (*seltop)->thisinst;
+   selobj = selinst->thisobject;
 
-   if ((selectobj = selectobject(POLYGON | LABEL, selobj, selinst)) != NULL) {
+   if ((selectobj = selectobject(class, selobj, selinst)) != NULL) {
       rgen = *(selobj->plist + (*selectobj));
       /* printf("%s found in object %s\n", (rgen->type == LABEL) ? "label" : "polygon",
 			selobj->name); */
@@ -853,17 +920,16 @@ genericptr recurselect(objectpair **seltop)
       clearselects();
    }
    else if ((selectobj = selectobject(OBJECT, selobj, selinst)) != NULL) {
-      objectpair *selnew = (objectpair *)malloc(sizeof(objectpair));
+      pushlistptr selnew = (pushlistptr)malloc(sizeof(pushlist));
 
       selinst = TOOBJINST(selobj->plist + (*selectobj));	
       drawselects(selobj, selinst);
       clearselects();
 
-      /* Link hierarchy information to the objectpair linked list */
+      /* Link hierarchy information to the pushlist linked list */
       selnew->thisinst = selinst;
-      selnew->thisobject = selinst->thisobject;
-      selnew->nextpair = NULL;
-      (*seltop)->nextpair = selnew;
+      selnew->next = NULL;
+      (*seltop)->next = selnew;
 	 
       /* Translate areastruct.save into object's own coordinate system */
       savesave.x = areastruct.save.x;
@@ -879,7 +945,7 @@ genericptr recurselect(objectpair **seltop)
 		areastruct.save.x, areastruct.save.y); */
       UPushCTM();
       UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
-      rgen = recurselect(&selnew);
+      rgen = recurselect(class, &selnew);
       UPopCTM();
       areastruct.save.x = savesave.x;
       areastruct.save.y = savesave.y;
@@ -894,22 +960,39 @@ genericptr recurselect(objectpair **seltop)
 /* Start drawing a select area box. */
 /*----------------------------------*/
 
-void startselect(XtIntervalId id, caddr_t clientdata)
+#ifdef TCL_WRAPPER
+
+xcTimeOutProc startselect(caddr_t clientdata)
 {
    if (eventmode != PENDING_MODE) return;
    eventmode = SELAREA_MODE;
    areastruct.origin.x = areastruct.save.x;
    areastruct.origin.y = areastruct.save.y;
    UDrawBox(areastruct.origin, areastruct.save);
-   XtAddEventHandler(areastruct.area, ButtonMotionMask, False, 
-	(XtEventHandler)trackselarea, NULL);
+
+   Tk_CreateEventHandler(areastruct.area, ButtonMotionMask,
+		(Tk_EventProc *)xctk_drag, NULL);
 }
 
+#else
+
+xcTimeOutProc startselect(caddr_t clientdata, xcIntervalId *id)
+
+{
+   if (eventmode != PENDING_MODE) return;
+   eventmode = SELAREA_MODE;
+   areastruct.origin.x = areastruct.save.x;
+   areastruct.origin.y = areastruct.save.y;
+   UDrawBox(areastruct.origin, areastruct.save);
+}
+
+#endif
+
 /*-------------------------*/
 /* Track a select area box */
 /*-------------------------*/
 
-void trackselarea(Widget w, XtPointer clientdata, XMotionEvent *event)
+void trackselarea()
 {
    XPoint newpos;
    u_int  nullui;
