@@ -1,6 +1,6 @@
 /*-----------------------------------------------------------------------*/
 /* text.c --- text processing routines for xcircuit		 	 */
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	 */
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	 */
 /*-----------------------------------------------------------------------*/
 
 #include <stdio.h>
@@ -14,6 +14,10 @@
 /* Local includes                                                         */
 /*------------------------------------------------------------------------*/
 
+#ifdef TCL_WRAPPER 
+#include <tk.h>
+#endif
+
 #include "colordefs.h"
 #include "xcircuit.h"
 
@@ -36,7 +40,6 @@ extern short eventmode;
 extern int *appcolors;
 extern colorindex *colorlist;
 extern char _STR[150];
-extern objectpair *pushlist;
 
 /* Global value of distance between characters in the font catalog */
 short del;
@@ -56,6 +59,56 @@ Boolean hasparameter(labelptr curlabel)
    return False;
 }
 
+/*----------------------------------------------------------------------*/
+/* Join selected labels together.					*/
+/*----------------------------------------------------------------------*/
+
+void joinlabels()
+{
+   genericptr *genobj;
+   short *jl;
+   stringpart *endpart;
+   int tpos;
+   labelptr dest, source;
+
+   if (areastruct.selects < 2) {
+      Wprintf("Not enough labels selected for joining");
+      return;
+   }
+
+   XcSetFunction(GXcopy);
+   XSetForeground(dpy, areastruct.gc, BACKGROUND);
+
+   for (jl = areastruct.selectlist; jl < areastruct.selectlist +
+		areastruct.selects; jl++) {
+      if (SELECTTYPE(jl) == LABEL) {
+	 dest = SELTOLABEL(jl);
+	 UDrawString(dest, DOFORALL, areastruct.topinstance);
+	 for (endpart = dest->string; endpart->nextpart != NULL; endpart =
+			endpart->nextpart);
+	 break;
+      }
+   }
+      
+   for (++jl; jl < areastruct.selectlist + areastruct.selects; jl++) {
+      if (SELECTTYPE(jl) == LABEL) {
+	 source = SELTOLABEL(jl);
+	 UDrawString(source, DOFORALL, areastruct.topinstance);
+	 endpart->nextpart = source->string;
+	 for (; endpart->nextpart != NULL; endpart = endpart->nextpart);
+	 free(source);
+	 removep(jl, 0);
+         reviseselect(jl);
+      }
+   }
+
+   XSetForeground(dpy, areastruct.gc, dest->color);
+   UDrawString(dest, dest->color, areastruct.topinstance);
+
+   incr_changes(topobject);
+   clearselects();
+}
+
 /*----------------------------------------------------------------------*/
 /* Insert a new segment into a string					*/
 /*----------------------------------------------------------------------*/
@@ -73,18 +126,24 @@ stringpart *makesegment(stringpart **strhead, stringpart *before)
    }
    else {				/* otherwise */
       for(lastptr = *strhead; lastptr != NULL;) {
-	 nextptr = nextstringpart(lastptr, NORMINST);
+	 nextptr = nextstringpart(lastptr, areastruct.topinstance);
 	 if (nextptr == before) {
 	    if (lastptr->type == PARAM_START) {
 	       oparamptr obs = NULL;
 	       int paramno = lastptr->data.paramno;
-	       if (NORMINST == NULL)
-		  obs = objectdata->params[paramno];
+	       if (areastruct.topinstance == NULL) {
+		  if (topobject->params != NULL)
+		     obs = topobject->params[paramno];
+		  else {
+		     Wprintf("Error:  Bad parameter!");
+		     break;
+		  }
+	       }
 	       else {
-	          if (NORMINST->params != NULL)
-	             obs = NORMINST->params[paramno];
+	          if (areastruct.topinstance->params != NULL)
+	             obs = areastruct.topinstance->params[paramno];
 	          if (obs == NULL)
-		     obs = NORMINST->thisobject->params[paramno];
+		     obs = areastruct.topinstance->thisobject->params[paramno];
 	       }
 	       if (obs == NULL) {
 		  Wprintf("Error:  Bad parameter");
@@ -113,12 +172,12 @@ stringpart *makesegment(stringpart **strhead, stringpart *before)
 /* Split a string across text segments					*/
 /*----------------------------------------------------------------------*/
 
-stringpart *splitstring(int tpos, stringpart **strtop, objinstptr localdata)
+stringpart *splitstring(int tpos, stringpart **strtop, objinstptr localinst)
 {
    int locpos, slen;
    stringpart *newpart, *ipart;
 
-   ipart = findstringpart(tpos, &locpos, *strtop, localdata);
+   ipart = findstringpart(tpos, &locpos, *strtop, localinst);
    if (locpos > 0) {        /* split the string */
       newpart = makesegment(strtop, ipart);
       newpart->type = TEXT_STRING;
@@ -179,8 +238,8 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
 	 paramno = strptr->data.paramno;
 	 if (thisinst == NULL)
 	 {
-	    if (objectdata->params != NULL)
-	       objectdata->params[paramno]->parameter.string = dstr->nextpart;
+	    if (topobject->params != NULL)
+	       topobject->params[paramno]->parameter.string = dstr->nextpart;
 	    else
 	       fprintf(stderr, "Error in deletestring:  Bad parameter number found\n"); 
 	 }
@@ -240,7 +299,7 @@ stringpart *mergestring(stringpart *firststr)
 /* Link a parameter to a string						*/
 /*----------------------------------------------------------------------*/
 
-stringpart *linkstring(objinstptr localdata, stringpart *strstart)
+stringpart *linkstring(objinstptr localinst, stringpart *strstart)
 {
    int paramno;
    stringpart *nextptr = NULL, *tmpptr;
@@ -252,21 +311,39 @@ stringpart *linkstring(objinstptr localdata, stringpart *strstart)
    /* In case of no calling instance, always get the default from the	*/
    /* current page object.						*/
 
-   if (localdata == NULL) {
-      if (objectdata->num_params > paramno)
-         nextptr = objectdata->params[paramno]->parameter.string;
+   if (localinst == NULL) {
+      if (topobject->num_params > paramno) {
+         if (topobject->params[paramno]->type != XC_STRING)
+	    goto linkerror;
+	 else
+            nextptr = topobject->params[paramno]->parameter.string;
+      }
       else
 	 return NULL;  /* There is no parameter here. */
    }
    else {
-      if (localdata->params != NULL)
-	 if (localdata->params[paramno] != NULL)
-            nextptr = localdata->params[paramno]->parameter.string;
+      if (localinst->params != NULL) {
+	 if (localinst->params[paramno] != NULL) {
+	    if (localinst->params[paramno]->type != XC_STRING)
+	       goto linkerror;
+            nextptr = localinst->params[paramno]->parameter.string;
+	 }
+      }
 
       /* If there's no instance, substitute the default parameter */
 
-      if (nextptr == NULL)
-         nextptr = localdata->thisobject->params[paramno]->parameter.string;
+      if (nextptr == NULL) {
+	 if (localinst->thisobject->params != NULL) {
+	    if (localinst->thisobject->params[paramno]->type != XC_STRING)
+	       goto linkerror;
+            nextptr = localinst->thisobject->params[paramno]->parameter.string;
+	 }
+	 else
+	    /* We get here in cases where the object definition is being read,	*/
+	    /* and there is no instance of the object to link to.  In that	*/
+	    /* case, we ignore parameters and move on to the next part.		*/
+	    return strstart->nextpart;
+      }
    }
 
    /* If the parameter exists, link the end of the parameter back to	*/
@@ -281,6 +358,11 @@ stringpart *linkstring(objinstptr localdata, stringpart *strstart)
       return nextptr;
    }
    return NULL;
+
+linkerror:
+   fprintf(stderr, "linkstring() error: parameter %d is not a substring!\n",
+		paramno);
+   return strstart->nextpart;
 }
 
 /*----------------------------------------------------------------------*/
@@ -308,12 +390,12 @@ int findcurfont(int tpos, stringpart *strtop, objinstptr thisinst)
 /*----------------------------------------------------------------------*/
 
 stringpart *findstringpart(int tpos, int *locpos, stringpart *strtop,
-	objinstptr localdata)
+	objinstptr localinst)
 {
    stringpart *strptr = strtop;
    int testpos = 0, tmplen;
 
-   for (strptr = strtop; strptr != NULL; strptr = nextstringpart(strptr, localdata)) {
+   for (strptr = strtop; strptr != NULL; strptr = nextstringpart(strptr, localinst)) {
       if ((strptr->type == TEXT_STRING) && strptr->data.string) {
 	 tmplen = strlen(strptr->data.string);
 	 if (testpos + tmplen > tpos) {
@@ -391,7 +473,7 @@ void charprint(char *sout, stringpart *strptr, int locpos)
 /* the calling routine).						*/
 /*----------------------------------------------------------------------*/
 
-char *xcstringtostring(stringpart *strtop, objinstptr localdata, Boolean textonly)
+char *xcstringtostring(stringpart *strtop, objinstptr localinst, Boolean textonly)
 {
    stringpart *strptr;
    int pos = 0, locpos;
@@ -400,7 +482,7 @@ char *xcstringtostring(stringpart *strtop, objinstptr localdata, Boolean textonl
    sout = (char *)malloc(1);
    sout[0] = '\0';
 
-   while ((strptr = findstringpart(pos++, &locpos, strtop, localdata)) != NULL) {
+   while ((strptr = findstringpart(pos++, &locpos, strtop, localinst)) != NULL) {
       if (!textonly || strptr->type == TEXT_STRING) {
          charprint(_STR, strptr, locpos);
          sout = (char *)realloc(sout, strlen(sout) + strlen(_STR) + 1);
@@ -421,9 +503,9 @@ char *xcstringtostring(stringpart *strtop, objinstptr localdata, Boolean textonl
 /*  for printing in the message window (charreport())			*/
 /*----------------------------------------------------------------------*/
 
-char *stringprint(stringpart *strtop, objinstptr localdata)
+char *stringprint(stringpart *strtop, objinstptr localinst)
 {
-   return xcstringtostring(strtop, localdata, False);
+   return xcstringtostring(strtop, localinst, False);
 }
 
 /*----------------------------------------------------------------------*/
@@ -431,9 +513,9 @@ char *stringprint(stringpart *strtop, objinstptr localdata)
 /*  appropriate for netlist information.				*/
 /*----------------------------------------------------------------------*/
 
-char *textprint(stringpart *strtop, objinstptr localdata)
+char *textprint(stringpart *strtop, objinstptr localinst)
 {
-   return xcstringtostring(strtop, localdata, True);
+   return xcstringtostring(strtop, localinst, True);
 }
 
 /*----------------------------------------------------------------------*/
@@ -443,14 +525,14 @@ char *textprint(stringpart *strtop, objinstptr localdata)
 /*    that the label only match text to the length of text.		*/
 /*----------------------------------------------------------------------*/
 
-int textcompx(stringpart *string, char *text, Boolean exact, objinstptr localdata)
+int textcompx(stringpart *string, char *text, Boolean exact, objinstptr localinst)
 {
    stringpart *strptr;
    char *tptr = text;
    char *sptr;
    int rval, llen = strlen(text), slen;
 
-   for (strptr = string; strptr != NULL; strptr = nextstringpart(strptr, localdata)) {
+   for (strptr = string; strptr != NULL; strptr = nextstringpart(strptr, localinst)) {
       if (strptr->type == TEXT_STRING) {
 	 sptr = strptr->data.string;
 	 slen = min(strlen(sptr), llen);
@@ -472,16 +554,16 @@ int textcompx(stringpart *string, char *text, Boolean exact, objinstptr localdat
 /* Wrappers for textcompx(), equivalent to strcmp() and strncmp().	*/
 /*----------------------------------------------------------------------*/
 
-int textcomp(stringpart *string, char *text, objinstptr localdata)
+int textcomp(stringpart *string, char *text, objinstptr localinst)
 {
-   return textcompx(string, text, True, localdata);
+   return textcompx(string, text, True, localinst);
 }
 
 /*----------------------------------------------------------------------*/
 
-int textncomp(stringpart *string, char *text, objinstptr localdata)
+int textncomp(stringpart *string, char *text, objinstptr localinst)
 {
-   return textcompx(string, text, False, localdata);
+   return textcompx(string, text, False, localinst);
 }
 
 /*----------------------------------------------------------------------*/
@@ -555,7 +637,8 @@ Boolean isisolatin1(int fontnumber)
 /*    underlining, tabbing, and kerning.				*/
 /*----------------------------------------------------------------------*/
 
-int stringcomprelaxed(stringpart *string1, stringpart *string2)
+int stringcomprelaxed(stringpart *string1, stringpart *string2,
+			objinstptr thisinst)
 {
    stringpart *strptr1 = string1, *strptr2 = string2;
    Boolean font1 = False, font2 = False;
@@ -570,13 +653,13 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2)
 		strptr1->type != OVERLINE) {
 	 if (strptr1->type == FONT_NAME)
 	    font1 = issymbolfont(strptr1->data.font);
-	 strptr1 = strptr1->nextpart;
+	 strptr1 = nextstringpart(strptr1, thisinst);
       }
       while (strptr2 != NULL && strptr2->type != TEXT_STRING &&
 		strptr2->type != OVERLINE) {
 	 if (strptr2->type == FONT_NAME)
 	    font2 = issymbolfont(strptr2->data.font);
-	 strptr2 = strptr2->nextpart;
+	 strptr2 = nextstringpart(strptr2, thisinst);
       }
       if (strptr1 == NULL || strptr2 == NULL) break;
       if (font1 != font2) return 1;
@@ -590,8 +673,8 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2)
 	       if (strptr1->type != strptr2->type) return 1;
 	       break;
          }
-	 strptr1 = strptr1->nextpart;
-	 strptr2 = strptr2->nextpart;
+	 strptr1 = nextstringpart(strptr1, thisinst);
+	 strptr2 = nextstringpart(strptr2, thisinst);
       }
    }
 
@@ -735,12 +818,12 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 	    newpart->data.string = NULL;
       }
       else if (strptr->type == PARAM_END) {
-	 /* If thisinst is NULL, then use objectdata (top level object) */
+	 /* If thisinst is NULL, then use topobject (top level object) */
 
 	 if (paramno >= 0) {
 	    if (thisinst == NULL) {
-	       if (objectdata->params != NULL)
-	          pparam = objectdata->params[paramno];
+	       if (topobject->params != NULL)
+	          pparam = topobject->params[paramno];
 	       else
 		  fprintf(stderr, "Error:  Bad parameter number encountered!\n");
 	    }
@@ -772,12 +855,7 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 /* draw a single character at 0, 0 using current transformation matrix */
 /*---------------------------------------------------------------------*/
 
-short UDrawChar(localdata, code, styles, ffont, groupheight, passcolor)
-  objinstptr 	localdata;
-  u_char	code;
-  short 	styles, ffont;
-  int		groupheight;
-  int		passcolor;
+short UDrawChar(u_char code, short styles, short ffont, int groupheight, int passcolor)
 {
    objectptr drawchar;
    XPoint alphapts[2];
@@ -798,7 +876,7 @@ short UDrawChar(localdata, code, styles, ffont, groupheight, passcolor)
    drawchar = fonts[ffont].encoding[(u_char)code];
    charinst.thisobject = drawchar;
 
-   localwidth = (drawchar->lowerleft.x + drawchar->width) * fonts[ffont].scale;
+   localwidth = (drawchar->bbox.lowerleft.x + drawchar->bbox.width) * fonts[ffont].scale;
 
    if ((fonts[ffont].flags & 0x22) == 0x22) { /* font is derived and italic */
       USlantCTM(DCTM, 0.25);  		/* premultiply by slanting function */
@@ -806,7 +884,7 @@ short UDrawChar(localdata, code, styles, ffont, groupheight, passcolor)
 
    if (!(styles & 64)) {
       
-      UDrawObject(&charinst, drawchar, SINGLE, passcolor);
+      UDrawObject(&charinst, SINGLE, passcolor, NULL);
 
       /* under- and overlines */
       if (styles & 8)
@@ -815,7 +893,7 @@ short UDrawChar(localdata, code, styles, ffont, groupheight, passcolor)
          alphapts[0].y = alphapts[1].y = groupheight + 4;
       if (styles & 24) {
          alphapts[0].x = 0; alphapts[1].x = localwidth;
-         UDrawSimpleLine(localdata, &alphapts[0], &alphapts[1]);
+         UDrawSimpleLine(&alphapts[0], &alphapts[1]);
       }
    }
    return localwidth;
@@ -825,7 +903,7 @@ short UDrawChar(localdata, code, styles, ffont, groupheight, passcolor)
 /* Draw an entire string, including parameter substitutions		*/
 /*----------------------------------------------------------------------*/
 
-void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
+void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 {
    stringpart *strptr;
    char *textptr;
@@ -841,11 +919,13 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
    short *tabstops = NULL;
    short tabno, numtabs = 0;
 
-   /* Instance from which we get the parameter list */
-   objinstptr pinst = (localdata == areastruct.topobject) ? NORMINST : localdata;
-
    if (fontcount == 0) return;
 
+#ifdef SCHEMA
+   /* Don't draw temporary labels from schematic capture system */
+   if (drawlabel->string->type != FONT_NAME) return;
+#endif
+
    if (passcolor == DOSUBSTRING)
       defaultcolor = curcolor = drawlabel->color;
    else
@@ -871,7 +951,7 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
    tmpjust = flipadjust(drawlabel->justify);
 
    /* "natural" (unscaled) length */
-   tmpext = ULength(drawlabel->string, pinst, 0.0, 0, NULL);
+   tmpext = ULength(drawlabel->string, localinst, 0.0, 0, NULL);
 
    newpoint.x = (tmpjust & NOTLEFT ?
        (tmpjust & RIGHT ? -tmpext.width : -tmpext.width >> 1) : 0);
@@ -909,7 +989,7 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 
        pos = 0;
        for (strptr = drawlabel->string; strptr != NULL;
-		strptr = nextstringpart(strptr, pinst)) {
+		strptr = nextstringpart(strptr, localinst)) {
 
 	  /* All segments other than text cancel any	*/
 	  /* existing overline/underline in effect.	*/
@@ -1019,10 +1099,11 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 		   int tmpheight;
 
 		   group = 0;
-		   for (textptr = strptr->nextpart->data.string; textptr && *textptr != '\0';
-				textptr++) {
+		   for (textptr = strptr->nextpart->data.string;
+				textptr && *textptr != '\0'; textptr++) {
 		      charptr = fonts[ffont].encoding[*(unsigned char *)textptr];
-		      tmpheight = (int)((float)charptr->height * fonts[ffont].scale);
+		      tmpheight = (int)((float)charptr->bbox.height
+				* fonts[ffont].scale);
 		      if (group < tmpheight) group = tmpheight;
 		   }
 	           fstyle |= 16;
@@ -1036,7 +1117,7 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
 		short addx;
 		UPushCTM();
 		UPreMultCTM(DCTM, newpoint, tmpscale, 0);
-		addx = UDrawChar(localdata, (u_char)32, fstyle, ffont, group,
+		addx = UDrawChar((u_char)32, fstyle, ffont, group,
 			curcolor);
 		newpoint.x += addx >> ((strptr->type == HALFSPACE) ? 1 : 2);
 		UPopCTM();
@@ -1059,7 +1140,7 @@ void UDrawString(objinstptr localdata, labelptr drawlabel, int passcolor)
       		         XTopSetForeground(curcolor);
 		      }
 	           }
-	           newpoint.x += UDrawChar(localdata, *textptr, fstyle, ffont,
+	           newpoint.x += UDrawChar(*textptr, fstyle, ffont,
 			group, scolor) * tmpscale;
 
 		   UPopCTM();
@@ -1094,7 +1175,7 @@ enddraw:
 /* Compute the actual length of a string or portion thereof.		*/
 /*----------------------------------------------------------------------*/
 
-TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
+TextExtents ULength(stringpart *string, objinstptr localinst, float newscale,
 	short dostop, XPoint *tbreak)
 {
    float oldscale, strscale, natscale, locscale = 1.0, xtotal = 0.5;
@@ -1109,7 +1190,12 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
    short tabno, numtabs = 0;
 
    retext.width = retext.ascent = retext.descent = retext.base = 0;
+
    if (fontcount == 0) return retext;
+#ifdef SCHEMA
+   /* Don't draw temporary labels from schematic capture system */
+   else if (string->type != FONT_NAME) return retext;
+#endif
 
    if (newscale > 0.0) natscale = UTopScale() * newscale;
    else natscale = 1.0;
@@ -1117,7 +1203,7 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
    oldscale = strscale = natscale;
 
    for (strptr = string; strptr != NULL;
-		strptr = nextstringpart(strptr, localdata)) {
+		strptr = nextstringpart(strptr, localinst)) {
       switch (strptr->type) {
 	 case SUPERSCRIPT:
 	    natscale *= SUBSCALE;
@@ -1143,12 +1229,12 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
 	    break;
 	 case HALFSPACE: 
 	    chptr = (*(somebet + 32));
-	    xtotal += (float)(chptr->width + chptr->lowerleft.x)
+	    xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x)
 			* locscale * natscale / 2;
 	    break;
 	 case QTRSPACE:
 	    chptr = (*(somebet + 32));
-	    xtotal += (float)(chptr->width + chptr->lowerleft.x) 
+	    xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x) 
 			* locscale * natscale / 4;
 	    break;
 	 case TABBACKWARD:	/* find first tab value with x < xtotal */
@@ -1196,13 +1282,13 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
 	       locpos++;
 
 	       chptr = (*(somebet + *textptr));
-	       xtotal += (float)(chptr->width + chptr->lowerleft.x)
+	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x)
 			* locscale * strscale;
 	       retext.ascent = max(retext.ascent, (short)(retext.base + ykern +
-			(float)((chptr->height + chptr->lowerleft.y)
+			(float)((chptr->bbox.height + chptr->bbox.lowerleft.y)
 			* locscale * strscale)));
 	       retext.descent = min(retext.descent, (short)(retext.base + ykern +
-			(float)(chptr->lowerleft.y * locscale * strscale)));
+			(float)(chptr->bbox.lowerleft.y * locscale * strscale)));
 
                if (tbreak != NULL) 
 	          if ((xtotal > tbreak->x) && (retext.base <= tbreak->y)) break;
@@ -1218,7 +1304,7 @@ TextExtents ULength(stringpart *string, objinstptr localdata, float newscale,
 
    /* special case: return character position in retext.width */
    if (tbreak != NULL) {
-      int slen = stringlength(string, True, localdata);
+      int slen = stringlength(string, True, localinst);
       if ((tbreak->x - lasttotal) < (xtotal - tbreak->x))
 	 locpos = lastpos + 1;
       if (locpos < 1) locpos = 1;
@@ -1238,14 +1324,14 @@ void undrawtextsimple(labelptr settext)
 {
    XSetFunction(dpy, areastruct.gc, GXcopy);
    XTopSetForeground(BACKGROUND);
-   UDrawString(areastruct.topobject, settext, DOFORALL);
+   UDrawString(settext, DOFORALL, areastruct.topinstance);
 }
 
 /*----------------------------------------------------------------------*/
 
 void redrawtextsimple(labelptr settext)
 {
-   UDrawString(NORMINST, settext, settext->color);
+   UDrawString(settext, settext->color, areastruct.topinstance);
 }
 
 /*----------------------------------------------------------------------*/
@@ -1264,7 +1350,7 @@ void drawtextandupdate(labelptr curlabel, void (*function)(labelptr))
    genericptr *pgen;
    labelptr slab;
 
-   for (pgen = objectdata->plist; pgen < objectdata->plist + objectdata->parts;
+   for (pgen = topobject->plist; pgen < topobject->plist + topobject->parts;
 		pgen++) {
       if ((*pgen)->type == LABEL) {
 	 slab = TOLABEL(pgen);
@@ -1303,14 +1389,15 @@ void redrawtext(labelptr curlabel)
 
 void composefontlib(short cfont)
 {
+   genericptr *pgen;
    objinstptr *drawinst;
-   objectptr *curlib, libobj, saveobj, nullobj;
-   objectptr libinst = xobjs.libtop[FONTLIB];
+   objectptr *curlib, libobj, nullobj;
+   objectptr directory = xobjs.libtop[FONTLIB]->thisobject;
    short visobjects, i, qdel;
    polyptr *drawbox;
    pointlist pointptr;
 
-   reset(libinst, NORMAL);
+   reset(directory, NORMAL);
 
    /* Create a pointer to the font library */
 
@@ -1330,9 +1417,9 @@ void composefontlib(short cfont)
 
    /* generate the list of object instances */
 
-   libinst->plist = (genericptr *) realloc(libinst->plist, visobjects
+   directory->plist = (genericptr *) realloc(directory->plist, visobjects
 		* sizeof(genericptr));
-   libinst->parts = 0;
+   directory->parts = 0;
 
    /* 0.5 is the default vscale;  16 is no. characters per line */
    del = min(areastruct.width, areastruct.height) / (0.5 * 16);
@@ -1342,31 +1429,25 @@ void composefontlib(short cfont)
 
       if ((libobj = fonts[cfont].encoding[i]) == nullobj) continue;
       
-      drawinst = (objinstptr *)libinst->plist + libinst->parts;
-      *drawinst = (objinstptr) malloc(sizeof(objinst));
-      (*drawinst)->type = OBJECT;
+      NEW_OBJINST(drawinst, directory);
+      objectdefaults(*drawinst, libobj,
+		(i % 16) * del + qdel,		/* X position */
+		-(i / 16) * del + qdel);	/* Y position */
+      drawinst = (objinstptr *)directory->plist + directory->parts;
       (*drawinst)->color = DEFAULTCOLOR;
-      (*drawinst)->rotation = 0;
-      (*drawinst)->scale = 1.0;
-      (*drawinst)->thisobject = libobj;
-      (*drawinst)->params = NULL;
-      (*drawinst)->position.x = (i % 16) * del + qdel;
-      (*drawinst)->position.y = -(i / 16) * del + qdel;
-
-      libinst->parts++;
+      directory->parts++;
    }
 
    /* separate characters with gridlines (17 vert., 17 horiz.) */
 
    for (i = 0; i < 34; i++) {
-      drawbox = (polyptr *)libinst->plist + libinst->parts;
-      *drawbox = (polyptr) malloc(sizeof(polygon));
-      (*drawbox)->type = POLYGON;
+      NEW_POLY(drawbox, directory);
+      polydefaults(*drawbox, 2, 0, 0);
+
       (*drawbox)->color = SNAPCOLOR;   /* default red */
       (*drawbox)->style = UNCLOSED;
       (*drawbox)->width = 1.0;
-      (*drawbox)->number = 2;
-      (*drawbox)->points = (pointlist) malloc(2 * sizeof(XPoint));
+
       if (i < 17) {
          pointptr = (*drawbox)->points;
          pointptr->x = i * del;
@@ -1383,15 +1464,24 @@ void composefontlib(short cfont)
          pointptr->x = 16 * del;
          pointptr->y = (17 - i) * del;
       }
-      libinst->parts++;
+      directory->parts++;
    }
 
-   /* calculate a bounding box for this display */
+   /* Set the bounding box for this display. 				*/
+   /* This is just the bounds of the grid built above, so there's no	*/
+   /* need to call any of the bounding box calculation routines.     	*/
 
-   saveobj = objectdata;
-   objectdata = libinst;
-   calcbbox(objectdata);
-   objectdata = saveobj;
+   directory->bbox.lowerleft.x = 0;
+   directory->bbox.lowerleft.y = pointptr->y;
+   directory->bbox.width = pointptr->x;
+   directory->bbox.height = pointptr->x;
+
+   xobjs.libtop[FONTLIB]->bbox.lowerleft.x = 0;
+   xobjs.libtop[FONTLIB]->bbox.lowerleft.y = pointptr->y;
+   xobjs.libtop[FONTLIB]->bbox.width = pointptr->x;
+   xobjs.libtop[FONTLIB]->bbox.height = pointptr->x;
+
+   centerview(xobjs.libtop[FONTLIB]);
 }
 
 /*------------------------------------------------------*/
