@@ -1,12 +1,19 @@
 /*----------------------------------------------------------------------*/
 /* xcircuit.h 								*/
-/* Copyright (c) 2001  Tim Edwards, Johns Hopkins University        	*/
+/* Copyright (c) 2002  Tim Edwards, Johns Hopkins University        	*/
 /*----------------------------------------------------------------------*/
 
 /*----------------------------------------------------------------------*/
 /*      written by Tim Edwards, 8/20/93    				*/
 /*----------------------------------------------------------------------*/
 
+#ifndef HAVE_U_CHAR
+typedef unsigned char	u_char;
+typedef unsigned short	u_short;
+typedef unsigned int	u_int;
+typedef unsigned long	u_long;
+#endif
+
 /*----------------------------------------------------------------------*/
 /* Definition shortcuts 						*/
 /*----------------------------------------------------------------------*/
@@ -16,41 +23,52 @@
 #define sign(a)			((a) <= 0 ? -1 : 1)
 #define max(a,b)		((a) < (b) ? (b) : (a))
 #define min(a,b)		((a) < (b) ? (a) : (b))
+#if SIZEOF_VOID_P == SIZEOF_UNSIGNED_INT
 #define Number(a)		(void *)((u_int) a)
+typedef u_int pointertype;
+#elif SIZEOF_VOID_P == SIZEOF_UNSIGNED_LONG
+#define Number(a)		(void *)((u_long) a)
+typedef u_long pointertype;
+#else
+ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Lengthier define constructs						*/
 /*----------------------------------------------------------------------*/
 
-#define objectdata      areastruct.topobject->thisobject
+/* Names used for convenience throughout the source code */
+
+#define topobject	areastruct.topinstance->thisobject
+
 #define PLIST_INCR(a) \
-	a->plist = (genericptr *) realloc (a->plist, \
-	(a->parts + 1) * sizeof(genericptr))
-#define SELECTTYPE(a)   (*(objectdata->plist + *a))->type
-#define SELTOCOLOR(a)	(*(objectdata->plist + *a))->color
-#define GSELTOCOLOR(b, a) (*(b->plist + *a))->color
-#define ENDPART		objectdata->plist + objectdata->parts
-#define EDITPART	objectdata->plist + areastruct.editpart
+	(a)->plist = (genericptr *) realloc ((a)->plist, \
+	((a)->parts + 1) * sizeof(genericptr))
+#define SELECTTYPE(a)   (*(topobject->plist + *(a)))->type
+#define SELTOCOLOR(a)	(*(topobject->plist + *(a)))->color
+#define GSELTOCOLOR(b, a) (*((b)->plist + *(a)))->color
+#define ENDPART		topobject->plist + topobject->parts
+#define EDITPART	topobject->plist + areastruct.editpart
 
 /* conversions from generic to specific types */
 /* specifically, from type (generic *) to type (polyptr), etc. */
 
-#define TOPOLY(a)	(*((polyptr *)a))
-#define TOLABEL(a)	(*((labelptr *)a))
-#define TOOBJINST(a)	(*((objinstptr *)a))
-#define TOARC(a)	(*((arcptr *)a))
-#define TOSPLINE(a)	(*((splineptr *)a)) 
-#define TOPATH(a)	(*((pathptr *)a))
+#define TOPOLY(a)	(*((polyptr *)(a)))
+#define TOLABEL(a)	(*((labelptr *)(a)))
+#define TOOBJINST(a)	(*((objinstptr *)(a)))
+#define TOARC(a)	(*((arcptr *)(a)))
+#define TOSPLINE(a)	(*((splineptr *)(a))) 
+#define TOPATH(a)	(*((pathptr *)(a)))
 
 /* conversions from a selection to a specific type */
 
-#define SELTOPOLY(a)	TOPOLY(objectdata->plist + (*a))
-#define SELTOLABEL(a)	TOLABEL(objectdata->plist + (*a))
-#define SELTOOBJINST(a)	TOOBJINST(objectdata->plist + (*a))
-#define SELTOARC(a)	TOARC(objectdata->plist + (*a))
-#define SELTOSPLINE(a)	TOSPLINE(objectdata->plist + (*a))
-#define SELTOPATH(a)	TOPATH(objectdata->plist + (*a))
-#define SELTOGENERIC(a) (*(objectdata->plist + (*a)))
+#define SELTOPOLY(a)	TOPOLY(topobject->plist + (*(a)))
+#define SELTOLABEL(a)	TOLABEL(topobject->plist + (*(a)))
+#define SELTOOBJINST(a)	TOOBJINST(topobject->plist + (*(a)))
+#define SELTOARC(a)	TOARC(topobject->plist + (*(a)))
+#define SELTOSPLINE(a)	TOSPLINE(topobject->plist + (*(a)))
+#define SELTOPATH(a)	TOPATH(topobject->plist + (*(a)))
+#define SELTOGENERIC(a) (*(topobject->plist + (*(a))))
 
 /* creation of new elements */
 
@@ -85,25 +103,14 @@
 	*a = (pathptr) malloc(sizeof(path)); \
 	(*a)->type = PATH
 
-/* normal choice of calling object instance */
-
-#define NORMINST ((pushlist == NULL) ? NULL : pushlist->thisinst)
-		   
 /* selection from top-level object */
 
-#define objectselect(a)		selectobject(a, objectdata, NORMINST)
-#define easydraw(a, b)		geneasydraw(a, b, objectdata, NORMINST)
-#define drawselected(a)		gendrawselected(a, objectdata, NORMINST)
+#define objectselect(a)	 selectobject(a, topobject, areastruct.topinstance)
+#define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
+#define drawselected(a)	 gendrawselected(a, topobject, areastruct.topinstance)
 
 /*----------------------------------------------------------------------*/
 
-#ifndef HAVE_U_CHAR
-typedef unsigned char	u_char;
-typedef unsigned short	u_short;
-typedef unsigned int	u_int;
-typedef unsigned long	u_long;
-#endif
-
 typedef struct {
    float x, y;
 } XfPoint;
@@ -130,9 +137,10 @@ typedef struct {
 #define DELBUFSIZE 10 /* Number of delete events to save for undeleting */
 #define MINAUTOSCALE 0.75 /* Won't automatically scale closer than this */
 #define MAXCHANGES 20 /* Number of changes to induce a temp file save	*/
+#define STIPPLES    8 /* Number of predefined stipple patterns		*/
 
 #ifdef SCHEMA
-#define PADSPACE   20 /* Spacing of pinlabels from their origins	*/
+#define PADSPACE   10 /* Spacing of pinlabels from their origins	*/
 #endif
 
 #ifdef HAVE_XPM
@@ -174,10 +182,10 @@ enum editmode {NORMAL_MODE = 0,
 PENDING_MODE, PRESS_MODE, DELETE_MODE, COPY_MODE, COPY2_MODE, ROTATE_MODE,
 PUSH_MODE, EDIT_MODE, SELAREA_MODE, SELAREA2_MODE, DESEL_MODE, PAN_MODE,
 CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
-TEXT1_MODE, TEXT2_MODE, TEXT3_MODE, WIRE_MODE, BOX0_MODE, BOX_MODE,
-EBOX_MODE, ARC0_MODE, ARC_MODE, EARC_MODE, SPLINE0_MODE, SPLINE_MODE,
-ESPLINE_MODE, EPATH_MODE, LPARAM_MODE, IPARAM_MODE, ULPARAM_MODE,
-UIPARAM_MODE
+TEXT1_MODE, TEXT2_MODE, TEXT3_MODE, WIRE_MODE, POLY_MODE, BOX0_MODE,
+BOX_MODE, EBOX_MODE, ARC0_MODE, ARC_MODE, EARC_MODE, SPLINE0_MODE,
+SPLINE_MODE, ESPLINE_MODE, EPATH_MODE, LPARAM_MODE, IPARAM_MODE,
+ULPARAM_MODE, UIPARAM_MODE
 
 #ifdef SCHEMA
 , CONNECT_MODE, ASSOC_MODE
@@ -449,6 +457,15 @@ typedef struct {
    u_char	num_params;
 } generic, *genericptr;		/* (convenience function for retypecasting) */
 
+/*----------------------------------------------------------------------*/
+/* Bounding box								*/
+/*----------------------------------------------------------------------*/
+
+typedef struct {
+   XPoint	lowerleft;
+   Dimension	width, height;
+} BBox;
+
 /*----------------------------------------------------------------------*/
 /* Object instance type							*/
 /*----------------------------------------------------------------------*/
@@ -465,15 +482,41 @@ typedef struct {
    float	scale;
    objectptr	thisobject;
    oparamptr    *params;	/* parameter substitutions for this instance */
+   BBox		bbox;		/* per-instance bounding box information */
+#ifdef SCHEMA
+   BBox		*schembbox;	/* Extra bounding box for pin labels */
+#endif
 } objinst, *objinstptr;
 
-typedef struct _objectpair *objectpairptr;
+/* Linked-list for objects */
 
-typedef struct _objectpair {
-   objectptr    thisobject;
+typedef struct _objlist *objlistptr;
+
+typedef struct _objlist {
+   objectptr	thisobject;
+   objlistptr	next;
+} objlist;
+
+/* Linked-list for object instances */
+
+typedef struct _pushlist *pushlistptr;
+
+typedef struct _pushlist {
    objinstptr	thisinst;
-   objectpairptr nextpair;
-} objectpair;
+   pushlistptr	next;
+} pushlist;
+
+/* Same as above, but for the list of instances in a library, so it	*/
+/* needs an additional record showing whether or not the instance is	*/
+/* "virtual" (not the primary library instance of the object)		*/
+
+typedef struct _liblist *liblistptr;
+
+typedef struct _liblist {
+   objinstptr	thisinst;
+   u_char	virtual;
+   liblistptr	next;
+} liblist;
 
 /*----------------------------------------------------------------------*/
 /* Label								*/
@@ -503,7 +546,7 @@ typedef struct {
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    u_char	num_params;
-   short	style;
+   u_short	style;
    float	width;
    short	number;
    pointlist	points;
@@ -518,7 +561,7 @@ typedef struct {
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    u_char	num_params;
-   short	style;
+   u_short	style;
    float	width;
    XPoint	ctrl[4];
    /* the following are for rendering only */
@@ -534,7 +577,7 @@ typedef struct {
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    u_char	num_params;
-   short	style;
+   u_short	style;
    float	width;
    short	radius; 	/* x-axis radius */
    short	yaxis;		/* y-axis radius */
@@ -555,7 +598,7 @@ typedef struct {
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    u_char	num_params;
-   short	style;
+   u_short	style;
    float	width;
    short	parts;
    genericptr	*plist;		/* to be retypecast to polygon, arc, or spline */
@@ -639,6 +682,15 @@ struct Ptab {
    struct Ptab *next;
 };
 
+/*----------------------------------------------------------------------*/
+/* Information needed to highlight a net				*/
+/*----------------------------------------------------------------------*/
+
+typedef struct {
+   int		netid;
+   objinstptr	thisinst;
+} Highlight;
+
 #endif
 
 /*----------------------------------------------------------------------*/
@@ -651,25 +703,24 @@ typedef struct _xcobject {
    Boolean	hidden;
    float	viewscale;
    XPoint	pcorner;	/* position relative to window */
-   XPoint	lowerleft;	/* bounding box */
-   Dimension	width, height;  /* information  */
+   BBox		bbox;		/* bounding box information (excluding */
+				/* parameterized elements) */
    short	parts;
    genericptr	*plist;	 	/* to be retypecast to label, polygon, etc. */
    u_char	num_params;	/* number of calling parameters */
    oparamptr	*params;	/* list of parameter defaults */
    
 #ifdef SCHEMA
-   XPoint	lleft2;		 /* bounding box info excluding pins */
-   Dimension	width2, height2;
-
+   Highlight	highlight;	/* net to be highlighted on redraw */
    u_char	schemtype;
-   objectptr	symschem;	 /* schematic page support */
+   objectptr	symschem;	/* schematic page support */
 
-   char		*devname;	 /* if non-null, name of device in netlist */
+   char		*devname;	/* if non-null, name of device in netlist */
+   Boolean	valid;		/* Is current netlist valid? */
+   Boolean	traversed;	/* Flag to indicate object was processed */
    NetlistPtr   netlist;
    PortlistPtr  portlist;
    CalllistPtr  calllist;
-   short traversed;
 #endif
 } object;
 
@@ -719,21 +770,62 @@ typedef struct _alias {
    aliasptr next;
 } alias;    
 
+/*----------------------------------------------------------------------*/
+/* To facilitate compiling the Tcl/Tk version, we make some convenient	*/
+/* definitions for types "xc..." and "Xc..." which can be mapped both	*/
+/* to X and Xt or to Tk functions and types.				*/
+/*----------------------------------------------------------------------*/
+
+#ifdef TCL_WRAPPER
+
+#define xcWidget 			Tk_Window
+#define xcWidgetList 			Tk_Window *
+#define xcAddEventHandler(a,b,c,d,e) 	Tk_CreateEventHandler(a,b,d,e)
+#define xcRemoveEventHandler(a,b,c,d,e) Tk_DeleteEventHandler(a,b,d,e)
+#define xcEventHandler			Tk_EventProc *
+#define xcWindow			Tk_WindowId
+#define xcIsRealized			Tk_IsMapped  /* not sure this is right */
+#define xcScreen			Tk_Screen
+#define xcParent			Tk_Parent
+#define xcAddTimeOut(a, b, c, d)	Tcl_CreateTimerHandler(b, c, d)
+#define xcRemoveTimeOut			Tcl_DeleteTimerHandler
+#define xcTimeOutProc			Tcl_TimerProc *
+#define xcIntervalId			Tcl_TimerToken
+
+#else
+
+#define xcWidget 			Widget
+#define xcWidgetList 			WidgetList
+#define xcAddEventHandler 		XtAddEventHandler
+#define xcRemoveEventHandler 		XtRemoveEventHandler
+#define xcEventHandler			XtEventHandler
+#define xcWindow			XtWindow
+#define xcIsRealized			XtIsRealized
+#define xcScreen			XtScreen
+#define xcParent			XtParent
+#define xcAddTimeOut			XtAppAddTimeOut
+#define xcRemoveTimeOut			XtRemoveTimeOut
+#define xcTimeOutProc			XtTimerCallbackProc
+#define xcIntervalId			XtIntervalId
+
+#endif
+
 /*----------------------------------------------------------------------*/
 /* structures for managing the popup prompt widgets 			*/
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   Widget	button;
+   xcWidget	button;
    int		foreground;
    void		(*buttoncall)();
    void		*dataptr;
 } buttonsave;
 
 typedef struct {
-   Widget	popup;		/* Popup widget		*/
-   Widget	textw;		/* Text entry widget	*/
-   Widget	filew;		/* File list window 	*/
+   xcWidget	popup;		/* Popup widget		*/
+   xcWidget	textw;		/* Text entry widget	*/
+   xcWidget	filew;		/* File list window 	*/
+   xcWidget	scroll;		/* Scrollbar widget	*/
    void		(*setvalue)();	/* Callback function	*/
    buttonsave	*buttonptr;	/* Button widget calling popup	     */
    char		*filter;	/* Extension filter for highlighting */
@@ -742,8 +834,8 @@ typedef struct {
 } popupstruct;
 
 typedef struct {
-   Widget	textw;
-   Widget	buttonw;
+   xcWidget	textw;
+   xcWidget	buttonw;
    void		(*setvalue)();
    void		*dataptr;
 } propstruct;
@@ -760,18 +852,26 @@ enum {DIRECTORY = 0, MATCH, NONMATCH};	/* file types */
 /*----------------------------------------------------------------------*/
 
 typedef struct {
+#ifdef SCHEMA
+   /* schematic layout colors */
+   Pixel	globalcolor, localcolor, infocolor;
+#endif
+
+   /* non-schematic layout color(s) */
+   Pixel	bboxpix;
+
    /* color scheme 1 */
    Pixel	fg, bg;
    Pixel	gridpix, snappix, selectpix, axespix;
    Pixel	buttonpix, querypix, filterpix;
-   Pixel	auxpix, barpix, bboxpix, parampix;
+   Pixel	auxpix, barpix, parampix;
 
    /* color scheme 2 */
 
    Pixel	fg2, bg2;
    Pixel	gridpix2, snappix2, selectpix2, axespix2;
    Pixel	buttonpix2, querypix2, filterpix2;
-   Pixel	auxpix2, barpix2, bboxpix2, parampix2;
+   Pixel	auxpix2, barpix2, parampix2;
 
    int		width, height, timeout;
    XFontStruct	*xcfont, *helpfont, *filefont, *textfont, *titlefont;
@@ -802,7 +902,7 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   Widget	cbutton;
+   xcWidget	cbutton;
    XColor	color;
 } colorindex;
 
@@ -832,8 +932,7 @@ typedef struct {
 
 typedef struct {
    char *name;
-   XPoint lowerleft;
-   XPoint upperright;
+   BBox bbox;
 } psbkground;
 
 /*----------------------------------------------------------------------*/
@@ -860,11 +959,13 @@ enum {
    XCF_Overline, XCF_ISO_Encoding, XCF_Return, XCF_Halfspace, XCF_Quarterspace,
    XCF_Special, XCF_TabStop, XCF_TabForward, XCF_TabBackward,
    XCF_Text_Return, XCF_Text_Delete, XCF_Text_Right, XCF_Text_Left,
+   XCF_Text_Up, XCF_Text_Down, XCF_Text_Split,
    XCF_Text_Home, XCF_Text_End, XCF_Return_Char, XCF_Parameter,
    XCF_Edit_Param, XCF_Edit_Break, XCF_Edit_Delete, XCF_Edit_Insert,
    XCF_Edit_Next, XCF_Attach, XCF_Next_Library, XCF_Library_Directory,
    XCF_Library_Move, XCF_Library_Copy, XCF_Library_Edit, XCF_Library_Delete,
-   XCF_Library_Duplicate, XCF_Library_Hide, XCF_Page_Directory, XCF_Library_Pop,
+   XCF_Library_Duplicate, XCF_Library_Hide, XCF_Library_Virtual,
+   XCF_Page_Directory, XCF_Library_Pop,
    XCF_Help, XCF_Redraw, XCF_View, XCF_Zoom_In, XCF_Zoom_Out, XCF_Pan,
    XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
    XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
@@ -875,8 +976,8 @@ enum {
    XCF_Cancel, XCF_Nothing, XCF_Exit,
 
 #ifdef SCHEMA
-   XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Info_Label, XCF_Connectivity,
-   XCF_Sim, XCF_SPICE, XCF_PCB, XCF_SPICEflat,
+   XCF_Netlist, XCF_Swap, XCF_Pin_Label, XCF_Pin_Global, XCF_Info_Label,
+   XCF_Connectivity, XCF_Sim, XCF_SPICE, XCF_PCB, XCF_SPICEflat,
 #endif
    NUM_FUNCTIONS
 };
@@ -887,7 +988,7 @@ enum {
 
 typedef struct {
    /* per-drawing-page parameters */
-   objectptr	pageobj;
+   objinstptr	pageinst;
    char		*filename;	/* file to save as */
    psbkground	background;	/* background rendered file info */
    float	wirewidth;
@@ -909,11 +1010,12 @@ typedef struct {
 typedef struct {
 
    /* widgets and X11 parameters */
-   Widget	area;
-   Widget	scrollbarh, scrollbarv;
+   xcWidget	area;
+   xcWidget	scrollbarh, scrollbarv;
+   Window	areawin;
    GC 		gc;
    int		gccolor, gctype;
-   XtIntervalId time_id;
+   xcIntervalId time_id;
 
    /* global page parameters */
    short	width, height;
@@ -927,7 +1029,7 @@ typedef struct {
    float	zoomfactor;
    short	psfont;
    short	justify;
-   short	style;
+   u_short	style;
    short	filter;		/* selection filter */
    int		color;
    Boolean	manhatn;
@@ -949,12 +1051,14 @@ typedef struct {
    Boolean	toolbar_on;
 #endif
 
-   /* buffers, pages, and associated variables */
+   /* buffers and associated variables */
    XPoint	save, origin;
    short	selects;
    short	*selectlist;
-   objinstptr	topobject;
-   objectpair	*hierarchy;
+   short	lastselects;
+   short	lastlibrary;
+   short	*selectlast;
+   objinstptr	topinstance;
    objectptr	editstack;
    Matrixptr	MatStack;
    short	editcycle;
@@ -962,9 +1066,16 @@ typedef struct {
    char		*lastbackground;
 } Clientdata;
 
+/* This whole thing needs to be cleaned up. . . now I've got objects in */
+/* "library" which are repeated both in the "instlist" pair and the	*/
+/* "thisobject" pointer in each instance.  The instance is repeated on	*/
+/* the library page.  Ideally, the instance list should only exist on	*/
+/* the library page, and not be destroyed on every call to "composelib"	*/
+
 typedef struct {
    short	number;
    objectptr	*library;
+   liblistptr	instlist;  /* List of instances */
 } Library;
 
 /*----------------------------------------------------------------------*/
@@ -974,7 +1085,7 @@ typedef struct {
 typedef struct {
    char		*tempfile;
    char		*tempdir;
-   int		timeout_id;
+   xcIntervalId	timeout_id;
    int		save_interval;
    u_short	new_changes;
    short	pages;
@@ -983,7 +1094,8 @@ typedef struct {
    Library	delbuffer;
    Library	fontlib;
    Library	*userlibs;
-   objectptr	*libtop;
+   objinstptr   *libtop;
+   pushlistptr	stack;
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
@@ -993,6 +1105,8 @@ typedef struct {
 /* Menu Definitions for hierarchical pulldown menus                     */
 /*----------------------------------------------------------------------*/
 
+/* #ifndef TCL_WRAPPER */	/* to be reinstated: need to compile */
+
 typedef struct _menustruct *menuptr;
 
 typedef struct _menustruct {
@@ -1015,4 +1129,7 @@ typedef struct _toolbarstruct {
 	char *hint;
 } toolbarstruct;
 
+/* #endif */	/* to be reinstated */
+/* Menus and Toolbars are taken care of entirely by scripts in the Tcl/	*/
+/* Tk version of xcircuit.						*/
 /*----------------------------------------------------------------------*/
